/*
    Copyright 2007, 2008, 2009 Geyer Klaus

    This file is part of Cat'sEyE.

    Cat'sEyE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Cat'sEyE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Cat'sEyE.  If not, see <http://www.gnu.org/licenses/>.
*/



#include <gtk/gtk.h>
#include <gio/gio.h>
#include <string.h>
#include <stdlib.h>

#include "helpers.h"
#include "Lists.h"
#include "mytrace.h"
#include "CommandParser.h"

extern GtkWindow 	*win;

/*	This is the routine to check and process the 'new usercommand'. With this the User can specify more complicated
 * calls to system, this is because of predefined variables:
 * 	<ThisDirectory>, <OtherDirectory>, <MarkedItems>, <Home> and a Dailog to get more Input
 * The mainsyntax in conf file is the same as ever, a pair of 'name' and 'command' seperated by a ',', but
 * this time 'commad' can be much more compicated but therefore much more powerfull.
 * The 'new usercommand' consists of one or two, so called, Blocks. Each Block begins with '<' and ends with '/>'
 * The Blocks hold the information to parse the usercommand and define the Dialog.
 *
 * The most importend Block is the CommandBlock, it begins with '<command'. Within the '<command' and the '/>' only one
 * property is expected: 'text'. This property defines the command the system executes with all its Options. You can use
 * the predefined variables above just type in their name, the parser will exchange them to real values as expected.
 * Furthermore you can add Variables specified and filled with a dialog.
 *
 * The other Block is the DialogBlock, it begins with '<dialog'. In this block there are more properties available.
 * 1'st The 'caption' property with which you can define the name of the dialog, if 'caption' is not defined
 * 		it's automaticaly filled with 'usercommand', so it doesn't matter if you don't use this property.
 * 2'nd The 'label' property. Here you can define some text which is displayed in the dialog.
 * 		you can use the newline '\n' and tab `\t` but a " will not be displayed! Again: Do not use '"' or '\"' !!
 * 3'rd The most interesting property 'question' defining a questiontext which is displayd just before a entrybox where
 * 		the user can enter some text. After specifying the question a Variablename is expected the identifier, which you can
 * 		choose by yourself, between a '<' and '>'. With this identifier you can use the text, the user entered, in the
 * 		commandblock, just use the same identifier (which should be unique - of course) as defined in the dialogblock.
 * 		Up to here it is the minimum for a question. Now you can give a text with wich the entrybox is filled when the dialog
 * 		pops up. The property is called 'predefine' and expects some Text.
 * 4'th	you can repeat the property question for as much input you need while the predefine property is not nessecarily to
 * 		define for each one. If you don't need a predefine for a question, just leave it away.
 *
 * Here is an example for a DialogBlock, defining 3 varaibles:
 * <dialog caption="Header" label="might be a little text" question="ask"<STR_1> question="ask2"<STR_2> predefine="answer_2" question="folder"<FOLDER> predefine=<HOME>/>
 *
 * Here is an example for a CommandBlock using the variables defined in dialogblock and predefined variables
 * <command text="syscommand -o<STR_1> -input<MarkedItems> -u<STR_2> -f<FOLDER>"
 * 		where syscommand is an command you can use in the shell *
 *
 */


int checkIsNewUserCommand(GString *command);
int getBlock(char *Blockname, GString *Block, char *WholeCommand);
char *getText(char *StartPoint, GString *gstrText, int sizeToJump);
char *getVarLabel(char *StartPoint,GString *VarLabel);
int decryptPredefine(GString *Predefine, struct predefineStruct *predefStruct);
char *getPredefine(char *StartPoint, GString *Predefine, struct predefineStruct *predefStruct );
int getOptions (GString *Block, GString **gstrCaption, GString **gstrLabel, GString **gstrCall);
int getUserInput(GString *Block, struct userDialogEntitiesList **inputList, gboolean *bShowCommand, struct predefineStruct *predefStruct);
int exchangeNameWithValue(GString *gstrText, GString *gstrEnemy, GString *NewString);
int exchangePredefine(GString *gstrText, struct predefineStruct *predefStruct);
int buildUserCommand(GString *Block, struct userDialogEntitiesList *inputList, struct predefineStruct *predefStruct);
void getDynamicOptions (GString *Block, GString **ActionNoteText, GString **QuitReplyText, struct updateList **UpdateFolderWhenFinished_List, struct userDialogEntitiesList *inputList/*int iPNumberVariables, GString **gstrDPVariables, GString **gstrDPVarTypeLabel*/, struct predefineStruct *predefStruct);
int processBackSpaces(GString *gstrStartText);
int exchangeUserVarsInText(GString *text, struct userDialogEntitiesList *inputList);

int ParseUserCommand(GString *Command, GString *FileList, GString *MarkedItemsWithPath, GString *WorkPath, GString *OtherSidePath, GString **ActionNoteText, GString **QuitReplyText, struct updateList **UpdateFolderWhenFinished_List){
	TRACEIT(10,"ParseUserCommand		start");
	GString *Block = g_string_sized_new(20);
	GString *OptionsBlock = g_string_sized_new(20);
	//GString **gstrDPVariables = NULL;
	//GString **gstrDPVarTypeLabel = NULL;
	GString *MarkedItemsUnQuoted = g_string_new(FileList->str);
	GString *MarkedItemsWithPathUnQuoted = g_string_new(MarkedItemsWithPath->str);
	struct userDialogEntitiesList *inputList=NULL;
	//int iNumberVariables=0;
	int i;
	int ret=-10;
	gboolean bShowCommand=FALSE;

	if (!checkIsNewUserCommand(Command)){
		TRACEIT(10,"ParseUserCommand		end, no new usrcommand");
		g_string_free(Block,TRUE);
		return 1;
	}


	cutCharFromGString(MarkedItemsUnQuoted, '\"', 0);

	// change: 1.4.1.1 : when there is more than one item in the unquoted-list, the list is mostly useless. On my current point of view, we
	// use the unquoted list just for single items, to add it to another path and requote it within the newusercommand. Therefor I choosed to
	// cut the list down till the LAST items name. This only has an effect to items from Fileshelf which come with their whole path, but the items
	// of the sideobject just come with their names.
	getLastStringInPath(MarkedItemsUnQuoted);

    cutCharFromGString(MarkedItemsWithPathUnQuoted, '\"', 0);

	struct predefineStruct predefStruct;
	predefStruct.ThisDirectory = WorkPath;
    predefStruct.OtherDirectory = OtherSidePath;
    predefStruct.MarkedItems = FileList;
    predefStruct.MarkedItemsUnQuoted = MarkedItemsUnQuoted;
    predefStruct.MarkedItemsWithPath = MarkedItemsWithPath;
    predefStruct.MarkedItemsWithPathUnQuoted = MarkedItemsWithPathUnQuoted;




	g_string_erase(Block,0,-1);
	if (getBlock(DIALOGBLOCK,Block,Command->str) == 0){
		//i = getUserInput(Block,&iNumberVariables, &gstrDPVariables, &gstrDPVarTypeLabel, &bShowCommand, &predefStruct);
        i = getUserInput(Block, &inputList, &bShowCommand, &predefStruct);
		if (i==-100){	//User pressed button cancel
			g_string_free(Block,TRUE);
			g_string_free(MarkedItemsUnQuoted,TRUE);
			g_string_free(OptionsBlock, TRUE);
			TRACEIT(10,"ParseUserCommand		ends, user cancelled");
			return -100;
		}
	}
	g_string_erase(Block,0,-1);
	if (getBlock(COMMANDBLOCK,Block,Command->str) == 0){
		//ret = buildUserCommand(Block,iNumberVariables, gstrDPVariables, gstrDPVarTypeLabel, &predefStruct );
		ret = buildUserCommand(Block, inputList, &predefStruct );

	}
	else{
		//free allocated memory and GStrings
		//if (gstrDPVariables != NULL){
		//	for (i=0; i<iNumberVariables; i++){
		//		if (gstrDPVariables[i] != NULL)
		//			g_string_free(gstrDPVariables[i],TRUE);
		//	}
		//}
		//free (gstrDPVariables);
		//if (gstrDPVarTypeLabel != NULL){
		//	for (i=0; i<iNumberVariables; i++){
		//		if (gstrDPVarTypeLabel[i] != NULL)
		//			g_string_free(gstrDPVarTypeLabel[i],TRUE);
		//	}
		//}
		//free(gstrDPVarTypeLabel);
		if (inputList)
            userDialogEntitiesList_deleteList( &inputList );
		g_string_free(Block,TRUE);
		g_string_free(MarkedItemsUnQuoted,TRUE);
		g_string_free(OptionsBlock, TRUE);
		TRACEIT(10,"ParseUserCommand		end, no new usercommand");
		return -12;
	}

	if (ret == 0 && getBlock(OPTIONSBLOCK,OptionsBlock,Command->str) == 0){
		getDynamicOptions (OptionsBlock, ActionNoteText, QuitReplyText, UpdateFolderWhenFinished_List, inputList,/*iNumberVariables, gstrDPVariables, gstrDPVarTypeLabel,*/ &predefStruct);
	}


	//free allocated memory and GStrings
/*	if (gstrDPVariables != NULL){
		for (i=0; i<iNumberVariables; i++){
			if (gstrDPVariables[i] != NULL)
				g_string_free(gstrDPVariables[i],TRUE);
		}
	}
	free (gstrDPVariables);
	if (gstrDPVarTypeLabel != NULL){
		for (i=0; i<iNumberVariables; i++){
			if (gstrDPVarTypeLabel[i] != NULL)
				g_string_free(gstrDPVarTypeLabel[i],TRUE);
		}
	}
	free(gstrDPVarTypeLabel);
*/
    if (inputList)
        userDialogEntitiesList_deleteList( &inputList );
	if (ret == 0){
		if (bShowCommand){
			GtkWidget *dialog, *entrybox;

			dialog = CreateMyDialog("Usercommand ready for take off",win,2,"Go",GTK_RESPONSE_ACCEPT,"Stop execution",GTK_RESPONSE_REJECT,TRUE);
			//label = gtk_label_new("If you press YES, then the currently processed item will be finished\nbefore all the other are stopped.\nTherefore you might want to make a 'kill' on the current process\n");
			entrybox = gtk_entry_new();
			gtk_entry_set_text((GtkEntry *)entrybox,Block->str);
			gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),entrybox);
			gtk_widget_show_all(dialog);
			if (gtk_dialog_run(GTK_DIALOG(dialog))== GTK_RESPONSE_ACCEPT){
				g_string_assign(Command,gtk_entry_get_text ((GtkEntry *)entrybox));
			}
			else{
				ret = -100; //cancel whole execution
			}
			gtk_widget_destroy(dialog);
		}
		else{
			//printf("%s\n",Block->str);
			g_string_assign(Command,Block->str);
		}
	}
	g_string_free(Block,TRUE);
	g_string_free(MarkedItemsUnQuoted,TRUE);
	g_string_free(MarkedItemsWithPathUnQuoted,TRUE);
	g_string_free(OptionsBlock, TRUE);
	TRACEIT(10,"ParseUserCommand		end");
	return ret;
}


int buildUserCommand(GString *Block, struct userDialogEntitiesList *inputList, struct predefineStruct *predefStruct){
	TRACEIT(10,"buildUserCommand		start");
	char *cbuf1, *cbuf2;
	GString *gstrText = g_string_sized_new(20);
	GString *gstrBuf = g_string_sized_new(20);
	int ret=0;
	cbuf2 = Block->str;
	cbuf1 = strstr(cbuf2,"text=");
	if (cbuf1 != NULL){
		cbuf2 = getText(cbuf1,gstrText, strlen("text="));
		if (cbuf2 == NULL || gstrText->str[0] == '\0'){
			g_string_free(gstrText,TRUE);
			g_string_free(gstrBuf,TRUE);
			MyMessageWidget (win,"UserCommandParser","No text found in usercommand, configfile (\")");
			TRACEIT(5,"buildUserCommand		end syntaxerror No Text Found");
			return	-2;	//No ' " ' found, syntaxerror
		}
	}
	else{
		g_string_free(gstrBuf,TRUE);
		g_string_free(gstrText,TRUE);
		MyMessageWidget(win,"UserCommandParser","No property 'text=' found in usercommand, configfile");
		TRACEIT(5,"buildUserCommand		end syntaxerror No Text Property");
		return	-1;	// no text found, syntaxerror
	}
	ret = 0;
	int i=0;
	//make it up to 5 times allows us to work with userdefined variables in true-false values of the checkbox
	do{
        ret = exchangeUserVarsInText(gstrText, inputList);
        i++;
	}while (ret > 0 && i < 5);

    if (ret < 0){
        MyMessageWidget(win,"UserCommandParser","Parser for newusercommand has found an error.\nCheck your definitions of your variables of the questions\nfor the command you just started.");
        TRACEIT(5,"buildUserCommand		end syntaxerror in the definition of one question variable");
        return -1; //syntacx error
    }

	exchangePredefine(gstrText, predefStruct);

    //now check for BackSpaces and process them
	if ( 0 > processBackSpaces(gstrText) ){  //returns: 1:exchanged, 0:nothing done, -1,-2 errors
        MyMessageWidget(win,"UserCommandParser","In command text-property:\nBackspaces lead to an error, check them.");
        TRACEIT(5,"buildUserCommand		end Backspace lead to an error");
        return -1; //systax error
	}
//	if (ret == -2){
//		g_string_free(gstrText,TRUE);
//		g_string_free(gstrBuf,TRUE);
//		return -3;		//Wether Variabel nor Predefine is used
//	}

	g_string_assign(Block,gstrText->str);
	g_string_free(gstrBuf,TRUE);
	g_string_free(gstrText,TRUE);
	TRACEIT(10,"buildUserCommand		end");
	return 0;
}


int exchangePredefine(GString *gstrText, struct predefineStruct *predefStruct){
	TRACEIT(10,"exchangePredefine		start");
	char *KnownPredefines[] = KNOWNPREDEFINES;
	int Count = NUMBEROFPREDEFINES;
	int i;
//	int ret=0;
	GString *gstrBuf = g_string_sized_new(20);
	GString *gstrBuf2 = g_string_sized_new(20);

	for (i=0;i<Count;i++){
		g_string_assign(gstrBuf,KnownPredefines[i]);
		g_string_assign(gstrBuf2,KnownPredefines[i]);
		g_string_append_c(gstrBuf2,'>');
		g_string_prepend_c(gstrBuf2,'<');
		decryptPredefine(gstrBuf, predefStruct);
		exchangeNameWithValue(gstrText, gstrBuf2, gstrBuf);
	}

	g_string_free(gstrBuf2,TRUE);
	g_string_free(gstrBuf,TRUE);

	//if (ret == -Count){	//not one Predefine was found
	//	TRACEIT(10,"exchangePredefine		end");
	//	return -1;
	//}
	TRACEIT(10,"exchangePredefine		end");
	return 0;
}


int exchangeNameWithValue(GString *gstrText, GString *gstrEnemy, GString *NewString){
	//gstrText holds the String in wich gstrEnemy is searched for and gstrEnemy will be exchanged by NewString
	TRACEIT(10,"exchangeNameWithValue		start");
	char *cStart;
	int iPosition=0;
	int iSize;
	int firstTime = 1;
	do {
		cStart = strstr(gstrText->str,gstrEnemy->str);
		if (cStart == NULL && firstTime==0){
			break;
		}
		else if(firstTime == 1 && cStart==NULL){
			return 0;	//EnemyString not found !!! unused Variable?
		}
		firstTime = 0;
		iSize = strlen(gstrEnemy->str);
		iPosition = 0;
		while( (gstrText->str)+iPosition != cStart){
			iPosition++;
		}
		g_string_erase(gstrText,iPosition,iSize);
		g_string_insert(gstrText,iPosition,NewString->str);
	}while (cStart != NULL);
	TRACEIT(10,"exchangeNameWithValue		end");
	return 1;
}


int exchangeUserVarsInText(GString *text, struct userDialogEntitiesList *inputList){
    TRACEIT(10,"exchangeUserVarsInText		start");
    if (text == NULL ){
        TRACEIT(2,"exchangeUserVarsInText		end, text = NULL");
        return -1;
    }
    if (inputList == NULL){
        TRACEIT(10,"exchangeUserVarsInText		end");
        return 0;
    }
    int ret=0;
    struct userDialogEntitiesList *bufList = inputList;
    GString *gstrBuf = g_string_sized_new(10);
    GString *gstrEmpty = g_string_new("");
    while (bufList){
        if (bufList->iValid == 1 && (bufList->iType==0 || bufList->iType==1) ){ //0=qestion, 1=checkbox, 2=text
            g_string_assign(gstrBuf, bufList->gstrVarIdentifier->str);
            g_string_append_c(gstrBuf,'>');
            g_string_prepend_c(gstrBuf,'<');
            if (bufList->gstrValue == NULL){    //we exchange the varlabel with an empty string (means we just delete the varlable in the command string)
                ret += exchangeNameWithValue(text, gstrBuf, gstrEmpty);
            }
            else{
                ret += exchangeNameWithValue(text, gstrBuf, bufList->gstrValue);
            }
        }
        else if (bufList->iValid == 0){
            ret = -1;    //some error
        }

        bufList=bufList->NextItem;
    }
    g_string_free(gstrBuf, TRUE);
    g_string_free(gstrEmpty, TRUE);
    TRACEIT(10,"exchangeUserVarsInText		end");
    return ret;
}


int processBackSpaces(GString *gstrStartText){
    // Replaces all \\b and delets the leading character
    // returns -1 when the text ends up in an empty string and leave the gstrText untouched
    // returns 0 if nothing was done
    // returns 1 when the string was processed
    TRACEIT(10,"processBackSpaces		start");
    if (gstrStartText==NULL){
        TRACEIT(2,"processBackSpaces		end gstrText=NULL, SHOULD NOT HAPPEN");
        return -2;
    }
    GString *gstrTextBuf = g_string_new(gstrStartText->str);
    GString *gstrbuf = g_string_sized_new(10);

    char Enemy[]="\\b";
    char *cStart;
	int iPosition=0;
	int iSize;
	int firstTime = 1;
	int diff=0;
	do {
		cStart = strstr(gstrTextBuf->str,Enemy);
		if (cStart == NULL && firstTime==0){
			break;
		}
		else if(firstTime == 1 && cStart==NULL){
		    TRACEIT(10,"processBackSpaces		end, no backspace");
		    g_string_free(gstrTextBuf, TRUE);
            g_string_free(gstrbuf, TRUE);
			return 0;	//No backspace found !!!
		}
		firstTime = 0;
		iSize = strlen(Enemy);
		iPosition = 0;
		while( (gstrTextBuf->str)+iPosition != cStart){
			iPosition++;
		}
		diff=(iPosition==0)?0:1;
		iPosition-=diff;
		iSize+=diff;
		g_string_erase(gstrTextBuf,iPosition,iSize);
	}while (cStart != NULL);

    g_string_assign(gstrbuf, gstrTextBuf->str);
	cutSpacesFromGString(gstrbuf);
	if (strlen(gstrbuf->str) == 0){
        TRACEIT(5,"processBackSpaces		end, strlen ends up in 0");
        MyMessageWidget (win,"UserCommandParser","Backspace error\nThe Backspaces lead to an empty string.\nCheck config file.");
        g_string_free(gstrTextBuf, TRUE);
        g_string_free(gstrbuf, TRUE);
        return -1;
	}
    g_string_assign(gstrStartText, gstrTextBuf->str);

    g_string_free(gstrTextBuf, TRUE);
    g_string_free(gstrbuf, TRUE);
	TRACEIT(10,"exchangeNameWithValue		end");
	return 0;
}


int UserCommandCallType(GString *Command, int *ip_CallType, GString **gstrCaption, GString **gstrLabel, GString **gstrCall){	//OPTIONSBLOCK
	TRACEIT(10,"UserCommandCallType		start");
	GString *Block = g_string_sized_new(20);

	if (!checkIsNewUserCommand(Command)){
		g_string_free(Block,TRUE);
		TRACEIT(10,"UserCommandCallType		end, no new usrcommand");
		return 1;
	}
	if (ip_CallType == NULL){
		g_string_free(Block,TRUE);
		TRACEIT(10,"UserCommandCallType		end, ip_CallType");
		return -1;
	}

	if (getBlock(OPTIONSBLOCK,Block,Command->str) == 0){
		getOptions (Block, gstrCaption, gstrLabel, gstrCall);
	}
	else{
		g_string_free(Block,TRUE);
		TRACEIT(10,"UserCommandCallType		end, No OptionBlock");
		return 2;
	}

	if (*gstrCall != NULL){
		if (0 == strcmp((*gstrCall)->str,"all")){
			*ip_CallType = USERCOMMAND_CALLAS_CALLALL;
		}
		else if (0 == strcmp((*gstrCall)->str,"each")){
			*ip_CallType =USERCOMMAND_CALLAS_CALLEACH;
		}
		else if (0 == strcmp((*gstrCall)->str,"list")){
			*ip_CallType = USERCOMMAND_CALLAS_LIST;
		}
	}
	g_string_free(Block,TRUE);
	TRACEIT(10,"UserCommandCallType		end");
	return 0;
}


int getOptions (GString *Block, GString **gstrCaption, GString **gstrLabel, GString **gstrCall){
	TRACEIT(10,"getOptions		start");
	char *cblock = Block->str;
	char *cbuf1;

	if (NULL != *gstrCall){
		g_string_free(*gstrCall,TRUE);
		*gstrCall = NULL;
	}
	if (NULL != *gstrCaption){
		g_string_free(*gstrCaption,TRUE);
		*gstrCaption = NULL;
	}
	if (NULL != *gstrLabel){
		g_string_free(*gstrLabel,TRUE);
		*gstrLabel = NULL;
	}

	cbuf1 = strstr(cblock, "call=");
	if (cbuf1 != NULL){
		*gstrCall = g_string_sized_new(4);
		getText(cbuf1,*gstrCall, strlen("call=") );
	}
	cbuf1 = strstr(cblock, "caption=");
	if (cbuf1 != NULL){
		*gstrCaption = g_string_sized_new(10);
		getText(cbuf1,*gstrCaption, strlen("caption="));
	}
	cbuf1 = strstr(cblock, "label=");
	if (cbuf1 != NULL){
		*gstrLabel = g_string_sized_new(10);
		getText(cbuf1,*gstrLabel, strlen("label="));
	}
	TRACEIT(10,"getOptions		end");
	return 0;
}


void getDynamicOptions (GString *Block, GString **ActionNoteText, GString **QuitReplyText, struct updateList **UpdateFolderWhenFinished_List, struct userDialogEntitiesList *inputList/*int iPNumberVariables, GString **gstrDPVariables, GString **gstrDPVarTypeLabel*/, struct predefineStruct *predefStruct){
    char *cbuf1=NULL;
    GString *gstrBuf = g_string_sized_new(10);
    GString *gstrBufFolder = g_string_sized_new(10);

    cbuf1 = strstr(Block->str, "quitreply=");
    if (cbuf1 != NULL){
        *QuitReplyText = g_string_sized_new(10);
		getText(cbuf1,*QuitReplyText, strlen("quitreply="));
        exchangeUserVarsInText(*QuitReplyText, inputList);

/*        for (i=0;i<iPNumberVariables;i++){
            if (gstrDPVarTypeLabel[i] != NULL){
                g_string_assign(gstrBuf, gstrDPVarTypeLabel[i]->str);
                g_string_append_c(gstrBuf,'>');
                g_string_prepend_c(gstrBuf,'<');
                exchangeNameWithValue(*QuitReplyText, gstrBuf, gstrDPVariables[i]);
            }
        }*/
        exchangePredefine(*QuitReplyText, predefStruct);
    }
    cbuf1 = strstr(Block->str, "actionnote=");
    if (cbuf1 != NULL){
		*ActionNoteText = g_string_sized_new(10);
		getText(cbuf1,*ActionNoteText, strlen("actionnote="));
		exchangeUserVarsInText(*ActionNoteText, inputList);


/*        for (i=0;i<iPNumberVariables;i++){
            if (gstrDPVarTypeLabel[i] != NULL){
                g_string_assign(gstrBuf, gstrDPVarTypeLabel[i]->str);
                g_string_append_c(gstrBuf,'>');
                g_string_prepend_c(gstrBuf,'<');
                exchangeNameWithValue(*ActionNoteText, gstrBuf, gstrDPVariables[i]);
            }
        }*/
        exchangePredefine(*ActionNoteText, predefStruct);
    }

    cbuf1 = strstr(Block->str, "folderupdate=");
    while(cbuf1 != NULL){
        getText(cbuf1,gstrBufFolder, strlen("folderupdate="));
        exchangeUserVarsInText(gstrBufFolder, inputList);
        /*for (i=0;i<iPNumberVariables;i++){
            if (gstrDPVarTypeLabel[i] != NULL){
                g_string_assign(gstrBuf, gstrDPVarTypeLabel[i]->str);
                g_string_append_c(gstrBuf,'>');
                g_string_prepend_c(gstrBuf,'<');
                exchangeNameWithValue(gstrBufFolder, gstrBuf, gstrDPVariables[i]);
            }
        }*/
        exchangePredefine(gstrBufFolder, predefStruct);
        updateList_addItem(UpdateFolderWhenFinished_List, gstrBufFolder->str, NULL, NULL, 0);

        cbuf1+=strlen("folderupdate=");
        cbuf1 = strstr(cbuf1, "folderupdate=");
    }


    g_string_free(gstrBufFolder, TRUE);
    g_string_free(gstrBuf,TRUE);
}


int getUserInput(GString *Block, struct userDialogEntitiesList **inputList, /*int *iPNumberVariables, GString ***gstrTPVariables, GString ***gstrTPVarTypeLabel,*/ gboolean *bShowCommand, struct predefineStruct *predefStruct){
	TRACEIT(10,"getUserInput		start");
	char *cblock = Block->str;
	char *cbuf1, *cbuf2;
	int ret;

	GString *gstrCaption = g_string_sized_new(30);
	gstrCaption->str[0]='\0';

	GString *gstrLabel = g_string_sized_new(30);
	gstrLabel->str[0]='\0';
	int iLabelSet = 0;

	GString *gstrBuf = g_string_sized_new(30);
	gstrBuf->str[0]='\0';

	//Get caption of the Dialog, if nothing is specified "usercommand" will be displayed
	cbuf1 = strstr(cblock, "caption=");
	if (cbuf1 != NULL){
		getText(cbuf1,gstrCaption, strlen("caption="));
	}
	else{
		g_string_assign(gstrCaption, "usercommand");
	}

	//Get the label, only one Text will be read
	cbuf1 = strstr(cblock, "label=");
	if (cbuf1 != NULL){
		getText(cbuf1,gstrLabel, strlen("label="));
		if (strlen(gstrLabel->str) != 0){
			iLabelSet = 1;
		}
	}

	//Search questions and start filling the list   TYPE = 0
	struct userDialogEntitiesList *bufList=NULL;
	cbuf2 = cblock;
	do{
		cbuf1 = strstr(cbuf2,"question=");
		if (cbuf1 != NULL){
			userDialogEntitiesList_addItem(inputList, 0, cbuf1);
		}
		cbuf2 = cbuf1+1;
	}while(cbuf1 != NULL);
	//search checkboxes TYPE = 1
	cbuf2 = cblock;
	do{
		cbuf1 = strstr(cbuf2,"checkbox=");
		if (cbuf1 != NULL){
			userDialogEntitiesList_addItem(inputList, 1, cbuf1);
		}
		cbuf2 = cbuf1+1;
	}while(cbuf1 != NULL);
	//search texts  TYPE = 2
	cbuf2 = cblock;
	do{
		cbuf1 = strstr(cbuf2,"text=");
		if (cbuf1 != NULL){
			userDialogEntitiesList_addItem(inputList, 2, cbuf1);
		}
		cbuf2 = cbuf1+1;
	}while(cbuf1 != NULL);

    //sort list
	userDialogEntitiesList_sortViaStartPointer(inputList);

	//collect information (Text, varlabel, predefines ...)
	int error=0;
	bufList = *inputList;	//inputList already is the first item, because the sort function writes it back
	do{
		if (bufList == NULL)
			break;
		cbuf1 = bufList->cpStart;
		//check errorvalue
		if (error != 0){
            switch (error){
                case 1:
                    MyMessageWidget (win,"UserCommandParser","dialog-block has no text or a \" is missing (configfile).");
                    break;
                case 2:
                    MyMessageWidget (win,"UserCommandParser","dialog-block does not define a variable name (configfile).");
                    break;
                case 3:
                    MyMessageWidget (win,"UserCommandParser","dialog-block: variable name invalid. Size 0. (configfile).");
                    break;
                case 4:
                    MyMessageWidget (win,"UserCommandParser","Question: Error when parsing predefine value. (\" is missing?).");
                    break;
                case 5:
                    MyMessageWidget (win,"UserCommandParser","Checkbox: No True value defined, syntax error. (configfile).");
                    break;
                case 6:
                    MyMessageWidget (win,"UserCommandParser","Checkbox: No True value defined or a \" is missing, syntax error. (configfile).");
                    break;
                case 7:
                    MyMessageWidget (win,"UserCommandParser","Checkbox: Error when parsing predefine value. (\" is missing?).");
                    break;
                case 8:
                    MyMessageWidget (win,"UserCommandParser","Checkbox: exptecting false value (existing \":\"?).");
                    break;
                case 9:
                    MyMessageWidget (win,"UserCommandParser","Checkbox: error reading false value (missing \" ?).");
                    break;
                case 10:
                    MyMessageWidget (win,"UserCommandParser","There may be a \" missing.\nOr reordering failed, which is an internal error.");
                    break;
                default:
                    MyMessageWidget (win,"UserCommandParser","Error while parsing questions or checkboxes (configfile).");
            }
		}


		//get Strings
        cbuf2=NULL;
		if (bufList->iType == 0){ // 0=question, 1=checkbox, 2=text
            cbuf2 = getText(cbuf1,gstrBuf, strlen("question="));
        }
        else if (bufList->iType == 1){ // 0=question, 1=checkbox, 2=text
            cbuf2 = getText(cbuf1,gstrBuf, strlen("checkbox="));
        }
        else if (bufList->iType == 2){ // 0=question, 1=checkbox, 2=text
            cbuf2 = getText(cbuf1,gstrBuf, strlen("text="));
        }
        if (cbuf2 == NULL){
            error=1;	//No Text found
            bufList = bufList->NextItem;
            continue;
        }
        else if(bufList->NextItem != NULL && cbuf2 > bufList->NextItem->cpStart){
                error = 10;	// " missing or reordering failed, internal error
                bufList = bufList->NextItem;
                continue;
        }
        bufList->gstrText = g_string_new(gstrBuf->str);

        //Varlabels
        if (bufList->iType == 0 || bufList->iType == 1){ // 0=question, 1=checkbox, 2=text
            cbuf2 = getVarLabel(cbuf2,gstrBuf);
            if (cbuf2 == NULL){
                error=2;	//No Varlabel found <VARLABEL>
                bufList = bufList->NextItem;
                continue;
            }
            else if(bufList->NextItem != NULL && cbuf2 > bufList->NextItem->cpStart){
                    error = 2;	//No Varlabel found <VARLABEL>
                    bufList = bufList->NextItem;
                    continue;
            }
            if (strlen(gstrBuf->str) == 0){
                error = 3;	//Varlabel not valid
                bufList = bufList->NextItem;
                continue;
            }
            bufList->gstrVarIdentifier = g_string_new(gstrBuf->str);
        }
        //more specific properties
		cbuf2++; //it points to '>', but this can lead to a problem when searching for text.
		if (bufList->iType == 0){ // 0=question, 1=checkbox, 2=text
		//predefines replaces the string found in predefine-property of the question with the predefined variables (like homedir ...). Its a little bit confusing, sorry about that.
			cbuf2 = getPredefine(cbuf2,gstrBuf, predefStruct);
			if (cbuf2 == NULL){
				//no error, predefine is optional
				bufList = bufList->NextItem;
				continue;
			}
			else if(bufList->NextItem != NULL && cbuf2 > bufList->NextItem->cpStart){
				error = 4;	//Predefine is too long
				bufList = bufList->NextItem;
				continue;
            }
			if (gstrBuf->str[0] == '\0'){
				//iPredefines++;
				bufList = bufList->NextItem;
				continue;
			}
		    //we found a predefine-property now we want to replace the backspaces
		    if ( 0 >= processBackSpaces(gstrBuf) ){  //returns: 1:exchanged, 0:nothing done, -1,-2 errors
				bufList->gstrPredefine = g_string_new(gstrBuf->str);
		    }
		}
		else if(bufList->iType == 1){	//0=question, 1=checkbox
			//find the truevalue
			cbuf2 = getText(cbuf2, gstrBuf, 0);
			if (cbuf2 == NULL){
				error = 5;	//checkbox has no trueValue, syntaxerror
				bufList = bufList->NextItem;
				continue;
			}
			else if(bufList->NextItem != NULL && cbuf2 > bufList->NextItem->cpStart){
				error = 6;	//checkbox has no trueValue or a " is missing. Syntaxerror
				bufList = bufList->NextItem;
				continue;
			}
			bufList->gstrTrueValue = g_string_new(gstrBuf->str);
			//exchangePredefine(gstrBuf, predefStruct);
			//processBackSpaces(gstrBuf);	//I think this is better done when collecting all date when building the command. In this case it is possible to use Variables defined by a question
			//find the false value, if there is any
			if (*(cbuf2+1) == ':'){ //if there is a false value it is leaded by a :
                cbuf2 = getText(cbuf2, gstrBuf,2);  //jump over 2 chars ) " and :
                if (cbuf2 == NULL){
                    error = 8;  //exptecting false value (existing :)
                    bufList = bufList->NextItem;
                    continue;	//no falsevalue;
                }
                else if(bufList->NextItem != NULL && cbuf2 > bufList->NextItem->cpStart){
                    error = 9;  //error reading falsevalue (missing "?)
                    bufList = bufList->NextItem;
                    continue;	//no false value
                }
                else{
                    bufList->gstrFalseValue = g_string_new(gstrBuf->str);
                }
			}
			//look for the predefine
			cbuf2 = getPredefine(cbuf2, gstrBuf, predefStruct);
			if (cbuf2 == NULL){
				//no error, predefine is optional
				bufList = bufList->NextItem;
				continue;
			}
			else if(bufList->NextItem != NULL && cbuf2 > bufList->NextItem->cpStart){
				error = 7;	//Predefine is too long
				bufList = bufList->NextItem;
				continue;
            }
            if (gstrBuf->str[0] == '\0'){
				//iPredefines++;
				bufList = bufList->NextItem;
				continue;
			}
		    //we found a predefine-property now we want to replace the backspaces
		    if ( 0 >= processBackSpaces(gstrBuf) ){  //returns: 1:exchanged, 0:nothing done, -1,-2 errors
				bufList->gstrPredefine = g_string_new(gstrBuf->str);
		    }
		}
		//else if(bufList->iType == 1){	//0=question, 1=checkbox, 2=text
        //    buflist->

		//}
		bufList = bufList->NextItem;
	}while (bufList);

	//Now that we have the Information for the Dialog we need to build it.
	GtkWidget *dialog, *label;
	GtkWidget* CheckButton;
	GtkWidget *sep1;


	dialog = CreateMyDialog(gstrCaption->str,win,2,"Ok",GTK_RESPONSE_ACCEPT,"Cancel",GTK_RESPONSE_REJECT,TRUE);

	if (iLabelSet == 1){
		label = gtk_label_new(gstrLabel->str);
		gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),label);
	}

	bufList = *inputList;	//inputList already is the first item, because the sort function writes it back
	do{
		if (bufList == NULL)
			break;
		if (bufList->iType == 0){	//0=question, 1=checkbox, 2=text
			if (bufList->gstrText != NULL && bufList->gstrVarIdentifier != NULL){
				bufList->iValid=1;
				bufList->QuestionLabel =  gtk_label_new(bufList->gstrText->str);
				gtk_misc_set_alignment ( (GtkMisc *)bufList->QuestionLabel, 0.0f, 0.0f);
                gtk_misc_set_padding ( (GtkMisc *)bufList->QuestionLabel, 4, 0);
				bufList->Box = gtk_entry_new();
				if (bufList->gstrPredefine){
					gtk_entry_set_text(	(GtkEntry *)bufList->Box, bufList->gstrPredefine->str);
				}
				g_signal_connect ((GtkEntry *)bufList->Box, "activate",G_CALLBACK(MyEasyInputWidget_EntryBoxActivate),dialog);
				gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),bufList->QuestionLabel);
				gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),bufList->Box);
			}
		}
		else if (bufList->iType == 1){	//0=question, 1=checkbox, 2=text
			if (bufList->gstrText != NULL && bufList->gstrVarIdentifier != NULL && bufList->gstrTrueValue != NULL){
				bufList->iValid=1;
				bufList->Box = gtk_check_button_new_with_mnemonic(bufList->gstrText->str);
				gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),bufList->Box);
				if (bufList->gstrPredefine){
					if (strcmp(bufList->gstrPredefine->str, "true")==0 || strcmp(bufList->gstrPredefine->str, "TRUE")==0 || strcmp(bufList->gstrPredefine->str, "True")==0 ){
						gtk_toggle_button_set_active ( (GtkToggleButton *)bufList->Box, TRUE);
                    }
                }
            }
		}
		else if (bufList->iType == 2){	//0=question, 1=checkbox, 2=text
            if (bufList->gstrText != NULL){
                bufList->iValid=1;
                exchangePredefine(bufList->gstrText, predefStruct);
                processBackSpaces(bufList->gstrText);
                bufList->Box = gtk_label_new (bufList->gstrText->str);
                gtk_misc_set_alignment ( (GtkMisc *)bufList->Box, 0.0f, 0.0f);
                gtk_misc_set_padding ( (GtkMisc *)bufList->Box, 4, 0);
                gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),bufList->Box);
            }
		}
		bufList = bufList->NextItem;
	}while (bufList);

    sep1 = gtk_separator_menu_item_new ();
    gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),sep1);

	CheckButton = gtk_check_button_new_with_label("Show me the complete command");
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),CheckButton);


	gtk_widget_show_all(dialog);
	if (gtk_dialog_run(GTK_DIALOG(dialog))== GTK_RESPONSE_ACCEPT){
		bufList = *inputList;	//inputList already is the first item, because the sort function writes it back
		do{
			if (bufList == NULL)
				break;
			if (bufList->iType == 0 && bufList->iValid == 1){	//0=question, 1=checkbox, 2=text
				bufList->gstrValue = g_string_new( gtk_entry_get_text((GtkEntry *)bufList->Box ) );
			}
			else if (bufList->iType == 1 && bufList->iValid == 1){	//0=question, 1=checkbox, 2=text
				if (gtk_toggle_button_get_active ( (GtkToggleButton *)bufList->Box)){	//checkbox checked
					bufList->gstrValue = g_string_new(bufList->gstrTrueValue->str);
				}
				else if (bufList->gstrFalseValue){
					bufList->gstrValue = g_string_new(bufList->gstrFalseValue->str);
				}
			}
			//else if (bufList->iType == 2 && bufList->iValid == 1){	//0=question, 1=checkbox, 2=text
			//}
			bufList = bufList->NextItem;
		}while (bufList);
		if (gtk_toggle_button_get_active ( (GtkToggleButton *)CheckButton)){
			*bShowCommand = TRUE;
		}
		else{
			*bShowCommand = FALSE;
		}

		ret = 0;
	}
	else{
		ret = -100; //cancel
	}
	gtk_widget_destroy(dialog);

	g_string_free(gstrCaption,TRUE);
	g_string_free(gstrLabel,TRUE);
	g_string_free(gstrBuf,TRUE);

	TRACEIT(10,"getUserInput		end");
	return ret;
}


char *getPredefine(char *StartPoint,GString *Predefine, struct predefineStruct *predefStruct){
	TRACEIT(10,"getPredefine		start");
	if (StartPoint==NULL){
        TRACEIT(2,"getPredefine		end, StartPoint NULL");
        return NULL;
	}
	if (Predefine==NULL || predefStruct==NULL){
        TRACEIT(2,"getPredefine		end, Predefine or predefStruct NULL");
        return NULL;
	}
	char *EndPoint=NULL;
	g_string_erase(Predefine,0,-1);
	while ( (*StartPoint == '\"'|| *StartPoint == '>' || *StartPoint==' ' || *StartPoint=='\t') && *StartPoint!='\0'){
		StartPoint++;
	};
	if (*StartPoint != '\0'){
	    if (strncmp (StartPoint, "predefine=", strlen("predefine=")) != 0){
            //Predefine->str[0]='\0';
            g_string_erase(Predefine, 0 , -1);
            TRACEIT(10,"getPredefine		end 0");
			return StartPoint;
	    }
	    /*
		if (StartPoint != mine_strnstr(StartPoint,"predefine=", strlen("predefine="))){ //strnstr is FreeBSD specific
			Predefine->str[0]='\0';
			return StartPoint;
		}*/
		StartPoint += strlen("predefine=");
		EndPoint = StartPoint;
		while( (*StartPoint == ' ' || *StartPoint=='\t') && *StartPoint != '\0'){
			StartPoint++;
		}
		if (*StartPoint == '\"'){	//Text
			EndPoint = getText(StartPoint, Predefine,0);
			if (EndPoint == NULL){
				Predefine->str[0] = '\0';
				TRACEIT(10,"getPredefine		end 1");
				return NULL;
			}
			exchangePredefine(Predefine, predefStruct);
			return EndPoint;
		}
		else if(*StartPoint == '<'){	//predefined variable
			EndPoint = getVarLabel(StartPoint,Predefine);
			if (EndPoint == NULL){
				Predefine->str[0] = '\0';
				TRACEIT(10,"getPredefine		end 2");
				return NULL;
			}
			if (Predefine->str[0] == '\0'){
				TRACEIT(10,"getPredefine		end 3");
				return EndPoint;
			}
			decryptPredefine(Predefine, predefStruct);
		}
	}
	else{
		TRACEIT(10,"getPredefine		end 4");
		return NULL;
	}
	TRACEIT(10,"getPredefine		end");
	return EndPoint;
}


int decryptPredefine(GString *Predefine, struct predefineStruct *predefStruct){
	TRACEIT(10,"decryptPredefine		start");
	char *KnownPredefines[] = KNOWNPREDEFINES;
	int Count = NUMBEROFPREDEFINES;				//Change this if you add or remove a String to/from KownPredefines
	int i;
//	char buf[300];

	for (i=0; i<Count; i++){
		if (strcmp(KnownPredefines[i], Predefine->str) == 0){
			break;
		}
	}

	switch(i){
		case 0:	//ThisDirectory
		case 1:
				g_string_assign(Predefine,predefStruct->ThisDirectory->str);
				break;
		case 2:	//OtherDirectory
		case 3:
				g_string_assign(Predefine,predefStruct->OtherDirectory->str);
				break;
		case 4:	//MarkedItems
		case 5:
				g_string_assign(Predefine,predefStruct->MarkedItems->str);
				break;
		case 6:	//MarkedItemsUnQuoted
		case 7:
				g_string_assign(Predefine,predefStruct->MarkedItemsUnQuoted->str);
				break;
		case 8:	//Home
		case 9:
				//strncpy(buf,(char *)getenv("HOME"),290);
				getHomeDirectory(&Predefine);
				//g_string_assign(Predefine,buf);
				break;
        case 10: //MarkedItemsWithPath
        case 11:
                g_string_assign(Predefine,predefStruct->MarkedItemsWithPath->str);
                break;
        case 12: //MarkedItemsWithPathUnQuoted
        case 13:
                g_string_assign(Predefine,predefStruct->MarkedItemsWithPathUnQuoted->str);
                break;
		default:
				Predefine->str[0] = '\0';
	};
	TRACEIT(10,"decryptPredefine		end");
	return 0;
}


char *getVarLabel(char *StartPoint,GString *VarLabel){
	TRACEIT(10,"getVarLabel		start");
	char *EndPoint=NULL;
	int iSize=0;
	g_string_erase(VarLabel,0,-1);
	while ( (*StartPoint == '\"' || *StartPoint==' ' || *StartPoint=='\t') && *StartPoint!='\0'){
		StartPoint++;
	};
	if (*StartPoint != '\0'){
		if (*StartPoint != '<'){
			VarLabel->str[0]='\0';
			TRACEIT(10,"getVarLabel		end 1");
			return StartPoint;
		}
		StartPoint++;
		EndPoint = StartPoint;
		while (*EndPoint != '>' && *EndPoint != '\0'){
			EndPoint++;
		};
		if (*EndPoint == '\0'){
			TRACEIT(10,"getVarLabel		end 2");
			return NULL;
		}
		iSize = EndPoint - StartPoint;
		g_string_append_len(VarLabel,StartPoint,iSize);
	}
	else{
		TRACEIT(10,"getVarLabel		end 3");
		return NULL;
	}
	TRACEIT(10,"getVarLabel		end");
	return EndPoint;
}


char *getText(char *StartPoint, GString *gstrText, int sizeToJump){
    //searches test within quotes, the StartPoint will be incremented by sizeToJumb
    //after this only " ' ' \t and are allowed, any other character stops this function
    //and no text is copied
	TRACEIT(10,"getText		start");
	char *EndPoint=NULL;
	int iSize;
	int stopexecution=0;
	if (StartPoint == NULL){
		TRACEIT(10,"getText		end 1");
		return NULL;
	}
	g_string_erase(gstrText,0,-1);
	StartPoint += sizeToJump;
	//while( *StartPoint != '\"' && *StartPoint != '\0'){
	while( (*StartPoint != '\"' && *StartPoint != '\0')){
        //printf("startpoint = %s\n",StartPoint);
	    if (*StartPoint != ' ' && *StartPoint != '\t'){
            stopexecution=1;
            break;
	    }
		StartPoint++;
	};
	if (*StartPoint != '\0' && stopexecution==0 ){
		StartPoint++;
		EndPoint = StartPoint;
		while( (*EndPoint != '\"' || *(EndPoint-1)=='\\') && *EndPoint != '\0'){
			EndPoint++;
		};

		if (*EndPoint == '\0'){
			TRACEIT(10,"getText		end 2");
			return NULL;
		}
		iSize = EndPoint - StartPoint;
		g_string_append_len(gstrText,StartPoint,iSize);
	}
	else{
		TRACEIT(10,"getText		end 3");
		return NULL;
	}

	GString *gg = g_string_new("\\n");
	GString *tt = g_string_new("\n");
	exchangeNameWithValue(gstrText,gg,tt);

	g_string_assign(gg,"\\t");
	g_string_assign(tt,"\t");
	exchangeNameWithValue(gstrText,gg,tt);

	g_string_assign(gg,"\\\"");
	g_string_assign(tt,"\"");
	exchangeNameWithValue(gstrText,gg,tt);

	g_string_free(gg,TRUE);
	g_string_free(tt,TRUE);

	TRACEIT(10,"getText		end");
	return EndPoint;
}


int getBlock(char *Blockname, GString *Block, char *WholeCommand){
	TRACEIT(10,"getBlock		start");
	char *cbuf = strstr(WholeCommand,Blockname);
	char *cend;
	if (cbuf == NULL){
		TRACEIT(10,"getBlock		end 1");
		return -1;
	}
	cend = strstr(cbuf,"/>");
	if (cend == NULL){
	    MyMessageWidget(win, "Error user command", "Block seems not to end, check ending '/>'.");
		TRACEIT(10,"getBlock		end 2");
		return -2;
	}
	cbuf += strlen(Blockname);
	while (*cbuf == ' '){
		cbuf++;
	}
	while(cbuf < cend){				//MayToDo: Kann man anders machen!!!!!!!!!!!
		g_string_append_c(Block,*cbuf);
		cbuf++;
	};
	TRACEIT(10,"getBlock		end");
	return 0;
}


int checkIsNewUserCommand(GString *command){	//Means we are locking for '<' and '/>' which are supposed to indicate that this is the new format fo Usercomamnds
	TRACEIT(10,"checkIsNewUserCommand		start");

	if (NULL != strchr(command->str,'<') && NULL != strstr(command->str,"/>")){
		TRACEIT(10,"checkIsNewUserCommand		New UsercommandString");
		return 1;
	}
	else{
		TRACEIT(10,"checkIsNewUserCommand		NOT a UsercommandString");
		return 0;
	}
}

