/*
    Copyright 2007, 2008, 2009 Geyer Klaus

    This file is part of Cat'sEyE.

    Cat'sEyE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Cat'sEyE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Cat'sEyE.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <gtk/gtk.h>
#include <gio/gio.h>

#include "helpers.h"
#include "globalDefinitions.h"
#include "Lists.h"
#include "myfm.h"
#include "fileshelf.h"

#include "mytrace.h"


gboolean DNDSignal_dragdrop (GtkWidget *widget, GdkDragContext *drag_context, gint x, gint y, guint time, gpointer user_data);
void DNDSignal_dragdataget (GtkWidget *widget, GdkDragContext *drag_context, GtkSelectionData *selection_data, guint info, guint time, gpointer user_data);
void DNDSignal_dragdatereceive (GtkWidget *widget, GdkDragContext   *drag_context, gint x, gint y, GtkSelectionData *data, guint info, guint time, gpointer user_data);


enum {
        TARGET_LISTVIEW //treeview
};


static GtkTargetEntry targets[] = {
        { "widget(listview)",    0, TARGET_LISTVIEW }
};

int prepareWidgetForDragnDrop (GtkWidget *widget){    //enables dragndrop for the widget and connects signals
    TRACEIT(10,"prepareWidgetForDragnDrop		start");
    if (widget == NULL){
        TRACEIT(2,"prepareWidgetForDragnDrop		end, widget NULL");
        return -1;
    }

    gtk_drag_dest_set (widget, GTK_DEST_DEFAULT_ALL, targets, 1, GDK_ACTION_COPY);
    gtk_drag_source_set (widget, GDK_BUTTON1_MASK, targets, 1, GDK_ACTION_COPY);

    //g_signal_connect(widget,"drag-drop",G_CALLBACK(DNDSignal_dragdrop),NULL); //Do you remember the problem that we end up in the drag-data-get and receive signal handler twice
    //after comment this out we did not have this Problem again
    g_signal_connect(widget,"drag-data-get",G_CALLBACK(DNDSignal_dragdataget),NULL);
    g_signal_connect(widget,"drag-data-received",G_CALLBACK(DNDSignal_dragdatereceive),NULL);

    TRACEIT(10,"prepareWidgetForDragnDrop		end");
    return 0;
}


////////////////// signal handler ////////////////////

//"drag-drop" signal (drop side)
gboolean DNDSignal_dragdrop (GtkWidget *widget, GdkDragContext *drag_context, gint x, gint y, guint time, gpointer user_data){
    TRACEIT(10,"DNDSignal_dragdrop		start");
    GdkAtom target_type;
    target_type = GDK_POINTER_TO_ATOM (g_list_nth_data (drag_context->targets, TARGET_LISTVIEW));
    if (drag_context->targets){
        gtk_drag_get_data (widget, drag_context, target_type, time);
        TRACEIT(10,"DNDSignal_dragdrop		end");
        return TRUE;    //accepted
    }

    TRACEIT(10,"DNDSignal_dragdrop		end");
    //return TRUE;    //accepted
    return FALSE;  //rejected
}


//"drag-data-get" signal (source side)
void DNDSignal_dragdataget (GtkWidget *widget, GdkDragContext *drag_context, GtkSelectionData *selection_data, guint info, guint time, gpointer user_data){
    TRACEIT(10,"DNDSignal_dragdataget		start");
    //GtkTreeSelection *selection=NULL;


    //this code looks a little bit weird, but if I delete it this signal (and therefore the receive signal too) will be exec<uted 2 times
    //and this code prevent us from doing the same thing twice.
 /*   static guint temp=0;
    if (temp == 0){
        temp = time;
    }
    else if (temp == time){
        temp = 0;
        //return;
    }*/
    if (info == TARGET_LISTVIEW){
        int *testpointer = (int *)widget;
        //MyMessageWidget(NULL, "", "drag-data-get signal");

        //selection  = gtk_tree_view_get_selection((GtkTreeView *)widget /*ListView*/);

        //gtk_selection_data_set (selection_data, selection_data->target, 8,(guchar *)&widget, sizeof(GtkWidget*));
        gtk_selection_data_set (selection_data, selection_data->target, 1,(guchar *)&testpointer, sizeof(int*));

    }
    TRACEIT(10,"DNDSignal_dragdataget		end");
}


//"drag-data-received" signal (drop side)
void DNDSignal_dragdatereceive (GtkWidget *widget, GdkDragContext   *drag_context, gint x, gint y, GtkSelectionData *selection_data, guint info, guint time, gpointer user_data){
    TRACEIT(10,"DNDSignal_dragdatereceive		start");
    GtkTreeSelection	*selection;
    struct SideObjectList *me_sideObject=NULL;
    struct FileShelfObjectList *me_fileshelfObject=NULL;
    struct SideObjectList *source_sideObject=NULL;
    struct FileShelfObjectList *source_fileshelfObject=NULL;
    int side;
    int iWeAre=0;   //1=sideObject, 2=fileshelf
    int iSourceIs=0;//1=sideObject, 2=fileshelf
    GtkWidget *sourceWidget=NULL;
    gboolean DND_Done = FALSE;

    if((selection_data != NULL) && (selection_data-> length >= 0)){
        if (info == TARGET_LISTVIEW){
            //MyMessageWidget(NULL, "", "drag-data-received");
            //int *testpointer = (int *)*(int *)selection_data-> data;
            //sourceWidget = (GtkWidget *)testpointer;
            sourceWidget = gtk_drag_get_source_widget (drag_context);

            //determine own type (the widget is from sideObject or Fileshelf)
            SideObjectList_GetObjectByChildListView(&me_sideObject, (GtkTreeView *)widget, &side);
            if (me_sideObject == NULL){    //maybe we are a FileshelfObject
                FileShelfObjectList_GetActiveObject(&me_fileshelfObject);
                if (me_fileshelfObject!=NULL && me_fileshelfObject->listView == widget){
                    iWeAre=2;
                }
            }
            else{
                iWeAre=1;
            }
            if (iWeAre==0){
                TRACEIT(2,"DNDSignal_dragdatereceive		end, could not determine own type.");
                return;
            }

            //find source type
            SideObjectList_GetObjectByChildListView(&source_sideObject, (GtkTreeView *)sourceWidget, &side);
            if (source_sideObject == NULL){    //maybe we are a FileshelfObject
                FileShelfObjectList_GetObjectByChildListView(&source_fileshelfObject, (GtkTreeView *)sourceWidget);
                if (source_fileshelfObject){
                    iSourceIs=2;
                }
            }
            else{
                iSourceIs=1;
            }
            if (iSourceIs==0){
                TRACEIT(2,"DNDSignal_dragdatereceive		end, could not determine own type.");
                return;
            }


            if (iWeAre == 1 && iSourceIs == 1){ //From sideObject to sideObject
                if (source_sideObject != me_sideObject){
                    copyToAnOtherPannel( source_sideObject, me_sideObject);
                }
                //else{
                //    printf("nothing done\n");
                //}
            }
            else if (iWeAre == 2 && iSourceIs == 1){ //From sideObject to FileShelf
                MenuAddItemToFileshelf(NULL, (void *) source_sideObject);
            }
            else if (iWeAre == 1 && iSourceIs == 2){ //From fileshelf to sideObject
                if (side == SIDE_RIGHT){
                    copyFileShelfToRightPannel(NULL, source_fileshelfObject);
                }
                else if (side == SIDE_LEFT){
                    copyFileShelfToLeftPannel(NULL, source_fileshelfObject);
                }
            }
            else if (iWeAre == 2 && iSourceIs == 2){    //From Fileshelf to FileShelf
                if (source_fileshelfObject != me_fileshelfObject){
                    selection  = gtk_tree_view_get_selection(GTK_TREE_VIEW(source_fileshelfObject->listView));
                    gtk_tree_selection_selected_foreach(selection, addItemsFromFileshelfToFileshelf_Helper, source_fileshelfObject);
                }
            }
            DND_Done=TRUE;
        }
    }

    gtk_drag_finish (drag_context, DND_Done, FALSE, time);


    TRACEIT(10,"DNDSignal_dragdatereceive		end");
}


////////////////////////////////////////////////////////////////////////////////////////////
/*
void                gtk_drag_dest_set                   (GtkWidget *widget,
                                                         GtkDestDefaults flags,
                                                         const GtkTargetEntry *targets,
                                                         gint n_targets,
                                                         GdkDragAction actions);

typedef struct {
  gchar *target;
  guint  flags;     //GTK_TARGET_SAME_APP
  guint  info;
} GtkTargetEntry;

typedef enum
{
  GDK_ACTION_DEFAULT = 1 << 0,
  GDK_ACTION_COPY    = 1 << 1,
  GDK_ACTION_MOVE    = 1 << 2,
  GDK_ACTION_LINK    = 1 << 3,
  GDK_ACTION_PRIVATE = 1 << 4,
  GDK_ACTION_ASK     = 1 << 5
} GdkDragAction;


//some signals
  "drag-begin"                                     : Run Last
  "drag-data-delete"                               : Run Last
  "drag-data-get"                                  : Run Last
  "drag-data-received"                             : Run Last
  "drag-drop"                                      : Run Last
  "drag-end"                                       : Run Last
  "drag-failed"                                    : Run Last
  "drag-leave"                                     : Run Last
  "drag-motion"
*/
