/*
    Copyright 2007, 2008, 2009 Geyer Klaus

    This file is part of Cat'sEyE.

    Cat'sEyE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Cat'sEyE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Cat'sEyE.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <gtk/gtk.h>
#include <dirent.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include "listitems.h"
#include "Lists.h"
#include "fileshelf.h"
#include "helpers.h"

#include "mytrace.h"

extern GtkWindow 	*win;   //the main widget


int SearchDirectoryFor(char *cpDirectory, char *cpSearchString, int *iBreak, int *iDeleteNotification, struct FileShelfObjectList *Shelf, GtkLabel *label);
void searchThread (struct AllInformationAllUseStruct *info);
void search_ActionWidget(struct AllInformationAllUseStruct *info);
int prepare_output(char *cpShelfName, struct FileShelfObjectList **FileShelfObject);
int find_dialog (GString *gstrDirectory, GString *gstrSearchString);



void find_simple(char *cpDirectory){
    //called from the menuitem and the main access point to this functions
    //if cpDirectory is NULL, the home directory of the user is used.
    TRACEIT(10,"find_simple		start");
    GString *gstrSearchString = g_string_sized_new(10);
    GString *gstrDirectory = g_string_sized_new(10);
    GString *gstrBuf = g_string_sized_new(10);

    struct AllInformationAllUseStruct *info=NULL;   //1:FileShelfObject, 2:GString Path, 3:GString Name
    struct FileShelfObjectList *pFileShelfObject=NULL;

    if (cpDirectory){
        g_string_assign(gstrDirectory, cpDirectory);
    }
    else {
        getHomeDirectory(&gstrDirectory);
    }

    if (0 == find_dialog(gstrDirectory, gstrSearchString)){    //start search
        g_string_assign(gstrBuf, gstrSearchString->str);
        cutSpacesFromGString ( gstrBuf);
        if (strlen (gstrBuf->str) == 0){
            g_string_free(gstrSearchString, TRUE);
            g_string_free(gstrDirectory, TRUE);
            g_string_free(gstrBuf, TRUE);
            return;
        }
        info = malloc (sizeof (struct AllInformationAllUseStruct)); //FREEE FREE FREEF FREREFE F EREF EFEREE ITTT
        if (info == NULL){
            g_string_free(gstrSearchString, TRUE);
            g_string_free(gstrDirectory, TRUE);
            g_string_free(gstrBuf, TRUE);
            return;
        }
        g_string_assign(gstrBuf, "search for: ");
        g_string_append(gstrBuf, gstrSearchString->str);
        if (0 == prepare_output(gstrBuf->str, &pFileShelfObject)){
            info->Info_1 = (void *)pFileShelfObject;
            info->Info_2 = (void *)g_string_new(gstrDirectory->str);    //FREE
            info->Info_3 = (void *)g_string_new(gstrSearchString->str); //FREE
            info->source = 0;   //used as iBreak
            info->Info_4 = malloc(sizeof(int)); //for iDeleteNotification
            //info_5 = actionWidget
            //info_6 = label of actionWidget
            search_ActionWidget (info);

            //start the thread which searches for us.
            g_thread_create((GThreadFunc)searchThread, info , FALSE, NULL);
        }
    }

    g_string_free(gstrSearchString, TRUE);
    g_string_free(gstrDirectory, TRUE);
    g_string_free(gstrBuf, TRUE);

    TRACEIT(10,"find_simple		end");
}


int find_dialog (GString *gstrDirectory, GString *gstrSearchString){
    //creates the dialog asking the user for the string to search for
    TRACEIT(10,"find_dialog		start");

    char cpText1[] = "This searches the items in the directory\nyou specify here recursivly for the occurence\nof the string you enter bellow.\nThe result is viewed in a new FileShelf.\nOnly the names are checked.\n\nDirectory:";
    int iRet=0;

    iRet = MyEasyInputWidget_Two (win, "RSearchD", cpText1, gstrDirectory, "\nSearch for occurence of:", gstrSearchString, "Ok", "Cancel", NULL, NULL );

    TRACEIT(10,"find_dialog		end");
    return iRet;
}


int prepare_output(char *cpShelfName, struct FileShelfObjectList **FileShelfObject){
    //creates the Fileshelf where the results show up
    TRACEIT(10,"prepare_output		start");
    if (cpShelfName == NULL){
        TRACEIT(2,"prepare_output		end, parameter NULL");
        return -1;
    }
    if (0 == NewFileShelf (cpShelfName)){
        FileShelfObjectList_GetActiveObject(FileShelfObject);
        TRACEIT(10,"prepare_output		end");
        return 0;
    }

    TRACEIT(5,"prepare_output		end, creating Fileshelf failed");
    return -1;
}


void search_ActionWidget(struct AllInformationAllUseStruct *info){
    //creates a widget which holds a simple label, showing the current directory searched
    //Destroy it when searchThread finishes
    TRACEIT(10,"search_ActionWidget		start");
    if (info == NULL){
        TRACEIT(2,"search_ActionWidget		end, Parameter NULL");
        return;
    }
    GtkWidget *ActionWidget = NULL;
    GtkWidget *label = NULL;

    int *iDeleteNotification = (int *)info->Info_4;
    ActionWidget = CreateMyWindow("search", win, 200, 100, TRUE, TRUE, FALSE, iDeleteNotification, FALSE);
    label = gtk_label_new("");
    gtk_label_set_line_wrap((GtkLabel *)label, TRUE);
    gtk_misc_set_alignment ( (GtkMisc *)label, 0.0f, 0.0f);
    gtk_misc_set_padding ( (GtkMisc *)label, 4, 0);


    gtk_container_add(GTK_CONTAINER(ActionWidget),label);

    info->Info_5 = (void *)ActionWidget;
    info->Info_6 = (void *)label;

    TRACEIT(10,"search_ActionWidget		end");
}

void searchThread (struct AllInformationAllUseStruct *info){
    TRACEIT(10,"searchThread		start");
    //Info_1 = pFileShelfObject
    //Info_2 = gstrDirectory
    //Info_3 = gstrSearchString
    //Info_4 = iDeleteNotification
    //Info_5 = actionWidget
    //Info_6 = gtk_label of actionWidget
    int *iDeleteNotification = (int *)info->Info_4;
    int *iBreak = &(info->source);
    GString *gstrDirectory = (GString *)info->Info_2;
    GString *gstrSearchString = (GString *)info->Info_3;

    gdk_threads_enter();
    gtk_widget_show_all(info->Info_5);
    gdk_threads_leave();

    SearchDirectoryFor(gstrDirectory->str, gstrSearchString->str, iBreak, iDeleteNotification, (struct FileShelfObjectList *)info->Info_1, (GtkLabel *)info->Info_6);

    g_string_free((GString *)info->Info_2, TRUE);
    g_string_free((GString *)info->Info_3, TRUE);
    if (*iDeleteNotification == 1){
        gdk_threads_enter();
        gtk_widget_destroy((GtkWidget *)info->Info_5);
        gdk_threads_leave();
    }
    free(info->Info_4); //an integer for iDeleteNotification
    free(info);

    TRACEIT(10,"searchThread		end");
    g_thread_exit(0);
}


int SearchDirectoryFor(char *cpDirectory, char *cpSearchString, int *iBreak, int *iDeleteNotification, struct FileShelfObjectList *Shelf, GtkLabel *label){
    //searches the directory recursive for the occurence of cpSearchstring
    //every item inherriting the search-string becomes an item on Shelf
    //if label is not NULL, it will be filled with the name of the directory currently searched in
    TRACEIT(10,"SearchDirectoryFor		start");
    if (cpDirectory == NULL || cpSearchString==NULL){
        TRACEIT(2,"SearchDirectoryFor		end, parameter NULL (1)");
        return -1;
    }
    if (iDeleteNotification == NULL || iBreak == NULL || Shelf==NULL){
        TRACEIT(2,"SearchDirectoryFor		end, parameter NULL (2)");
        return -1;
    }


    GString *gstrNames = g_string_sized_new(10);
    GString *gstrPath = g_string_sized_new(10);
    DIR *Directory;
	struct dirent *dirzeiger;
	int DirectoryOpened=0;

    g_string_assign(gstrPath, cpDirectory);
    if (0 != openDirectory(&Directory, &DirectoryOpened, gstrPath)){
        g_string_free(gstrNames, TRUE);
        g_string_free(gstrPath, TRUE);
        TRACEIT(10,"SearchDirectoryFor		end");
        return -1;
    }

    if (label){
        g_string_printf(gstrPath, "searching\n\n%s\n\nfor %s", cpDirectory, cpSearchString);
        gdk_threads_enter();
        gtk_label_set_text (label, gstrPath->str);
        gdk_threads_leave();
    }

    getNextListItem(Directory, &dirzeiger, DirectoryOpened, gstrNames);

    while ( *iBreak == 0 && *iDeleteNotification == 1 && strlen(gstrNames->str) != 0){
		if (strcmp(gstrNames->str,".")!=0 && strcmp(gstrNames->str,"..")!=0){
		    if (NULL != strstr(gstrNames->str, cpSearchString)){
                AddItemToFileshelf (Shelf, gstrNames->str, cpDirectory, FALSE);
		    }
			g_string_assign(gstrPath,cpDirectory);
			checkPathForLastChar(gstrPath);
			g_string_append(gstrPath,gstrNames->str);
			if (TRUE == checkStringIsPath(gstrPath->str)){  //recursive
                SearchDirectoryFor (gstrPath->str, cpSearchString, iBreak, iDeleteNotification, Shelf, label);
			}
		}
		getNextListItem(Directory, &dirzeiger, DirectoryOpened, gstrNames);
	}

    closeDirectory(&Directory);
    g_string_free(gstrNames, TRUE);
    g_string_free(gstrPath, TRUE);

    TRACEIT(10,"SearchDirectoryFor		end");
    return 0;
}


