/*
    Copyright 2007, 2008, 2009 Geyer Klaus

    This file is part of Cat'sEyE.

    Cat'sEyE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Cat'sEyE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Cat'sEyE.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <gtk/gtk.h>
#include <gio/gio.h>
#include <stdlib.h>
#include <string.h>

#include "helpers.h"
//#include "fileshelf.h"
//#include "listitems.h"
#include "Lists.h"
#include "myfm.h"
#include "automaticUpdate.h"
#include "mytrace.h"

struct autoUpdateList *g_listAutoUpdateList;


void autoUpdateList_monitoredDirChanged (GFileMonitor *monitor, GFile *file, GFile *other_file, GFileMonitorEvent event_type, struct autoUpdateList *buflist){
    TRACEIT(10,"autoUpdateList_monitoredDirChanged	    start");
    //printf("monitor: update signal %s\n",buflist->gstrPath->str);
    updateObjectsViewingFolder(buflist->gstrPath->str, TRUE);

    TRACEIT(10,"autoUpdateList_monitoredDirChanged	    end");
}




void autoUpdateList_DeleteObject(struct autoUpdateList *Object){
	TRACEIT(10,"autoUpdateList_DeleteObject	    start");
	if (Object->PrevItem != NULL){
		Object->PrevItem->NextItem = Object->NextItem;
	}
	else{
		g_listAutoUpdateList = Object->NextItem;
		g_listAutoUpdateList->PrevItem = NULL;
	}

	if (Object->NextItem != NULL){
		Object->NextItem->PrevItem = Object->PrevItem;
	}

    if (FALSE == g_file_monitor_cancel ( Object->monitor )){
        TRACEIT(2,"autoUpdateList_DeleteObject	    ends, monitor could not be cancelled");
    }
    g_string_free(Object->gstrPath, TRUE);
    g_object_unref(Object->monitor);
    g_object_unref(Object->gfilePath);

	free(Object);
	TRACEIT(10,"autoUpdateList_DeleteObject	    ends");
}


int autoUpdateList_RemoveMonitor (char *Path){
    TRACEIT(10,"autoUpdateList_RemoveMonitor		start");
    if (Path == NULL){
        TRACEIT(2,"autoUpdateList_RemoveMonitor		end, Parameter NULL");
        return -1;
    }
    //printf("monitor: deleting %s\n",Path);
    //search list for path,
    struct autoUpdateList *buflist = g_listAutoUpdateList;
    while (buflist){
        if (strcmp(Path, buflist->gstrPath->str) == 0){
            buflist->iCountRef--;
            //printf("monitor: countRef decreased\n");
            break;
        }
        buflist = buflist->NextItem;
    }
    if (buflist == NULL){
        TRACEIT(4,"autoUpdateList_RemoveMonitor		end, path not found, not monitored");
        return -1;
    }


    if (buflist->iCountRef == 0){
        //printf("monitor: deleting monitor\n");
        autoUpdateList_DeleteObject(buflist);
    }


    TRACEIT(10,"autoUpdateList_RemoveMonitor		end");
    return 0;
}


int autoUpdateList_AddMonitor (char *Path){
    TRACEIT(10,"autoUpdateList_AddMonitor		start");
    if (Path == NULL){
        TRACEIT(2,"autoUpdateList_AddMonitor		end, Parameter NULL");
        return -1;
    }
    //printf("monitor: adding %s\n",Path);

    struct autoUpdateList *buf2list=NULL;
    //search list for path, if we allready monitor it we increase the reference count
    struct autoUpdateList *buflist = g_listAutoUpdateList;

    while (buflist){
        if (strcmp(Path, buflist->gstrPath->str) == 0){
            buflist->iCountRef++;
            //printf("monitor: CountRef increased\n");
            TRACEIT(10,"autoUpdateList_AddMonitor		end");
            return 0;
        }
        buflist = buflist->NextItem;
    }

    //if we dont find the path, start monitoring it
    if (buflist == NULL){
       // printf("monitor: new monitor\n");
        buflist = malloc( sizeof(struct autoUpdateList) );
        if (buflist){
            buflist->gfilePath = g_file_new_for_path (Path);
            buflist->monitor = g_file_monitor_directory (buflist->gfilePath,G_FILE_MONITOR_NONE,NULL,NULL);
            if (buflist->monitor == NULL){
                g_object_unref(buflist->gfilePath);
                free(buflist);
                TRACEIT(2,"autoUpdateList_AddMonitor		end, cant get monitor");
                return -1;
            }
            buflist->iCountRef=1;
            buflist->gstrPath = g_string_new(Path);
            g_signal_connect(buflist->monitor,"changed",G_CALLBACK(autoUpdateList_monitoredDirChanged),buflist);
        }
        else{
            TRACEIT(2,"autoUpdateList_AddMonitor		end, malloc failed");
            return -1;
        }
    }


    buflist->NextItem = NULL;
    buflist->PrevItem = NULL;
    if (NULL == g_listAutoUpdateList){
        g_listAutoUpdateList = buflist;
    }
    else{
        //find the last item
        for (buf2list = g_listAutoUpdateList; buf2list->NextItem; buf2list=buf2list->NextItem );
        //printf("buf2list=%p\tbuf2list->NextItem=%p",buf2list,buf2list->NextItem);

        buf2list->NextItem = buflist;
        buflist->PrevItem = buf2list;
    }

    //printf("monitor: added\n");
    TRACEIT(10,"autoUpdateList_AddMonitor		end");
    return 0;
}


void autoUpdateList_deleteList (){
    TRACEIT(10,"autoUpdateList_deleteList		start");
    //printf("monitor: deleting LIST\n");

    struct autoUpdateList *ThisObject;

	while (g_listAutoUpdateList){

		ThisObject = g_listAutoUpdateList;
		g_listAutoUpdateList->PrevItem=NULL;


        if (FALSE == g_file_monitor_cancel ( g_listAutoUpdateList->monitor )){
            TRACEIT(2,"autoUpdateList_deleteList	    ends, monitor could not be cancelled");
        }
        g_string_free(g_listAutoUpdateList->gstrPath, TRUE);
        g_object_unref(g_listAutoUpdateList->monitor);
        g_object_unref(g_listAutoUpdateList->gfilePath);


		g_listAutoUpdateList = g_listAutoUpdateList->NextItem;
		free(ThisObject);
	}
//printf("monitor: LIST deleted\n");
    TRACEIT(10,"autoUpdateList_deleteList		end");
}
