/*
    Copyright 2007, 2008, 2009 Geyer Klaus

    This file is part of Cat'sEyE.

    Cat'sEyE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Cat'sEyE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Cat'sEyE.  If not, see <http://www.gnu.org/licenses/>.
*/



#include <stdio.h>
#include <gtk/gtk.h>
#include <ctype.h>	//convert char to lower-char
#include <string.h>
#include <stdlib.h>

#include "confighandling.h"
#include "mytrace.h"
#include "helpers.h"

//int openfiles = 0;    //for debuging info. counts the currently opened files, opend with this set of functions.

////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////
////////////////NEW FILE HANDLING ///////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////

int myFiles_SaveFileSimpleSave(char *cpLine, char *Mode){
    struct myFiles_struct *stru = myFiles_openSaveFile(Mode);
    if(stru){
        myFiles_SaveValueToFile(stru, cpLine);
        myFiles_closeFile(stru);
        return 0;
    }
    return -1;
}

struct myFiles_struct *myFiles_openSaveFile(char *cpMode){
    TRACEIT(10,"myFiles_saveFile		start");

	GString *SaveFile=g_string_sized_new(20);
    getHomeDirectory (&SaveFile);
	checkPathForLastChar(SaveFile);
	g_string_append(SaveFile,SAVE_FILE);

    struct myFiles_struct *stru;
    stru = myFiles_openFile(SaveFile->str, cpMode);

	g_string_free(SaveFile,TRUE);
	TRACEIT(10,"myFiles_saveFile		ends");
    return stru;
}

int myFiles_SaveFileSimpleGetValue(char *Identifier, GString *Value){
	//return 1-> found, return !=0 Not found
    //All spaces are cut within Value
    TRACEIT(10,"myFiles_SaveFileSimpleGetValue		start");
    if (Identifier == NULL){
        TRACEIT(10,"myFiles_SaveFileSimpleGetValue		ends, parameter NULL");
        return -1;
    }
    if (Value == NULL){
        TRACEIT(10,"myFiles_SaveFileSimpleGetValue		ends, parameter NULL");
        return -1;
    }

    struct myFiles_struct *stru = myFiles_openSaveFile("r");
    if (stru == NULL){
       TRACEIT(10,"myFiles_SaveFileSimpleGetValue		ends, error opening configfile");
       return -1;
    }

    if (myFiles_getValueFromIdentifier(stru, Identifier, 0) == 1){
        g_string_assign(Value, stru->gstrValue->str);
        myFiles_closeFile(stru);
        cutSpacesFromGString(Value);
        if (strlen(Value->str) == 0){
            return 0;   //not found
        }
        return 1;   //All good, found
    }
    if (stru){
        myFiles_closeFile(stru);
    }
    return 0;  //not found
    TRACEIT(10,"myFiles_SaveFileSimpleGetValue		ends");

}


int myFiles_ConfigFileSimpleGetValue(char *Identifier, GString *Value){
    //return 1-> found, return !=0 Not found
    //All spaces are cut within Value
    TRACEIT(10,"myFiles_ConfigFileSimpleGetValue		start");
    if (Identifier == NULL){
        TRACEIT(10,"myFiles_ConfigFileSimpleGetValue		ends, parameter NULL");
        return -1;
    }
    if (Value == NULL){
        TRACEIT(10,"myFiles_ConfigFileSimpleGetValue		ends, parameter NULL");
        return -1;
    }

    struct myFiles_struct *stru = myFiles_openConfigTypesFile("r");
    if (stru == NULL){
       TRACEIT(10,"myFiles_ConfigFileSimpleGetValue		ends, error opening configfile");
       return -1;
    }

    if (myFiles_getValueFromIdentifier(stru, Identifier, 0) == 1){
        g_string_assign(Value, stru->gstrValue->str);
        myFiles_closeFile(stru);
        cutSpacesFromGString(Value);
        if (strlen(Value->str) == 0){
            return 0;   //not found
        }
        return 1;   //All good, found
    }
    if (stru){
        myFiles_closeFile(stru);
    }
    return 0;  //not found

    TRACEIT(10,"myFiles_ConfigFileSimpleGetValue		ends");
}

struct myFiles_struct *myFiles_openConfigTypesFile(char *cpMode){
	TRACEIT(10,"myFiles_openConfigTypesFile		start");

	GString *ConfFile=g_string_sized_new(20);
    getHomeDirectory (&ConfFile);
	checkPathForLastChar(ConfFile);
	g_string_append(ConfFile,CONFIG_FILE_TYPES);

    struct myFiles_struct *stru;
    stru = myFiles_openFile(ConfFile->str, cpMode);

	g_string_free(ConfFile,TRUE);
	TRACEIT(10,"myFiles_openConfigTypesFile		ends");
	return stru;
}


struct myFiles_struct *myFiles_openFile(char *FileName, char *cpMode){
    TRACEIT(10,"myFiles_openFile		start");
    GString *gstrBuf = g_string_sized_new(10);
    if (FileName == NULL){
        TRACEIT(3,"myFiles_openFile		end, No FileName");
        g_string_free(gstrBuf,TRUE);
        return NULL;
    }
    FILE *pFile;
    pFile = fopen(FileName,cpMode);

	if (pFile == NULL){
        g_string_sprintf(gstrBuf,"myFiles_openFile		end, Error opening file %s",FileName);
        TRACEIT(3,gstrBuf->str);
        g_string_free(gstrBuf,TRUE);
        return NULL;
	}
//	openfiles++;
//	printf("Opened files: %i\n",openfiles);
    struct myFiles_struct *stru= malloc(sizeof(struct myFiles_struct));
    if (stru == NULL){
        fclose(pFile);
//        openfiles--;
//        printf("Opened files: %i\n",openfiles);
        g_string_free(gstrBuf,TRUE);
        TRACEIT(1,"myFiles_openFile		end, FATAL ERROR, MEMORYALLOCATION");
        return NULL;
    }

    stru->File = pFile;
    stru->gstrFileName = g_string_new(FileName);
    stru->gstrIdentifier = g_string_sized_new(20);
	stru->gstrValue = g_string_sized_new(20);
    stru->allreadystarted=0;

    g_string_free(gstrBuf,TRUE);
    TRACEIT(10,"myFiles_openFile		end");
    return stru;
}


int myFiles_closeFile(struct myFiles_struct *fileInfo){
    TRACEIT(10,"myFiles_closeFile		start");
    if (fileInfo == NULL){
        TRACEIT(3,"myFiles_closeFile		end fileInfo parameter == NULL");
        return -1;
    }

    if (fileInfo->File != NULL){
//        openfiles--;
//        printf("Opened files: %i\n",openfiles);
        fclose(fileInfo->File);
    }
    if (fileInfo->gstrFileName != NULL)
        g_string_free(fileInfo->gstrFileName, TRUE);
    if (fileInfo->gstrIdentifier != NULL)
        g_string_free(fileInfo->gstrIdentifier, TRUE);
    if (fileInfo->gstrValue != NULL)
        g_string_free(fileInfo->gstrValue, TRUE);

    free(fileInfo);

    TRACEIT(10,"myFiles_closeFile		end");
    return 0;
}


int myFiles_getValueFromIdentifier(struct myFiles_struct *fileInfo, char *cpIdentifier, int again){
    TRACEIT(10,"myFiles_getValueFromIdentifier		start");
    if (fileInfo == NULL){
        TRACEIT(3,"myFiles_getValueFromIdentifier		end fileInfo parameter == NULL");
        return -1;
    }
	if (cpIdentifier == NULL){
		TRACEIT(3,"myFiles_getValueFromIdentifier		ends cpIdentifier parameter == NULL");
		return -1;
	}
    if (fileInfo->File == NULL){
		TRACEIT(3,"myFiles_getValueFromIdentifier		ends No file opened");
		return -2;
	}
	char bufLine[5001];
	char *buf;
    gchar *cpctype=NULL;
    GString *gstrCType = g_string_sized_new(15);


    g_string_assign( gstrCType, cpIdentifier);
    cpctype = g_utf8_strdown( gstrCType->str, -1);
    if (cpctype == NULL){
        g_string_free(gstrCType, TRUE);
        TRACEIT(10,"myFiles_getValueFromIdentifier		end");
        return 0;
    }
    g_string_assign(gstrCType, cpctype);
    g_free(cpctype);
    cpctype=NULL;

	g_string_append(gstrCType,TYPESEPARATOR);
    fileInfo->allreadystarted=0;
	if (again == 0){
		fseek(fileInfo->File,0,SEEK_SET);
	}
	while(1){
		if (feof(fileInfo->File))
			break;
		fgets(bufLine,5000,fileInfo->File);
		if ( strncmp(bufLine,gstrCType->str,strlen(gstrCType->str)) == 0 ){
		    buf =  strchr(bufLine,'\n');
			if (buf != NULL){
				*buf='\0';
			}
			g_string_assign(fileInfo->gstrValue,bufLine);
			cutTapsFromGString(fileInfo->gstrValue);
			g_string_assign(fileInfo->gstrIdentifier,gstrCType->str);
			g_string_erase(fileInfo->gstrValue,0,strlen(gstrCType->str));
            g_string_free(gstrCType, TRUE);
			TRACEIT(10,"myFiles_getValueFromIdentifier		ends 1 ");
			return 1;   //found
		}
	};
	g_string_free(gstrCType, TRUE);
    TRACEIT(10,"myFiles_getValueFromIdentifier		end");
    return 0;
}


int myFiles_SeparateMultiValue(struct myFiles_struct *fileInfo, GString *name, GString *command, int ModusSingleOrPair){
	//	ModusSingleOrPair == 1 -> Pair, == 0 -> Single
	//  leading spaces are removed from command, but not from name
	TRACEIT(10,"myFiles_SeparateMultiValue		start");
	if (name == NULL || fileInfo == NULL){
		TRACEIT(10,"myFiles_SeparateMultiValue		ends");
		return -1;
	}
	if (fileInfo->gstrValue == NULL || fileInfo->gstrIdentifier == NULL){
		TRACEIT(10,"myFiles_SeparateMultiValue		ends");
		return -2;
	}

	char *buf1=NULL;
	char *buf2=NULL;
    GString *gstrCType=g_string_sized_new(15);

	const char sep[] = PROGRAMSEPERATOR;
	if (fileInfo->allreadystarted == 0){
		buf1 = (char *)strtok(fileInfo->gstrValue->str,sep);
		if (ModusSingleOrPair==1){
			buf2 = (char *)strtok(NULL,sep);
		}
		fileInfo->allreadystarted = 1;
	}
	else{
		buf1 = (char *)strtok(NULL,sep);
		if (ModusSingleOrPair==1){
			buf2 = (char *)strtok(NULL,sep);
		}
	}

	if (buf1 == NULL || (ModusSingleOrPair == 1 && buf2 == NULL)){
        g_string_assign(gstrCType, fileInfo->gstrIdentifier->str);
        g_string_truncate(gstrCType, strlen(gstrCType->str)-strlen(TYPESEPARATOR));
		if (1 == myFiles_getValueFromIdentifier(fileInfo, gstrCType->str, 1)){
			TRACEIT(10,"getOneCommandAfterEachOther		recursively ends");
			return myFiles_SeparateMultiValue(fileInfo, name, command, ModusSingleOrPair);
		}
		TRACEIT(10,"getOneCommandAfterEachOther		ends");
		return 0;	//Nothing found
	}
	else{
		if (buf1[strlen(buf1)-1]=='\n'){
			buf1[strlen(buf1)-1] = '\0';
		}
		if (ModusSingleOrPair == 1){
			if (buf2[strlen(buf2)-1]=='\n'){
				buf2[strlen(buf2)-1] = '\0';
			}
		}
		g_string_assign(name,buf1);
		if (ModusSingleOrPair == 1){
			g_string_assign(command,buf2);
			cutLeadingSpacesFromGString(command);
		}

		TRACEIT(10,"getOneCommandAfterEachOther		ends");
		return 1;   //found
	}
}


int myFiles_SaveValueToFile(struct myFiles_struct *fileInfo, char* cpLine){
    TRACEIT(10,"myFiles_SaveValueToFile		start");
    if (fileInfo == NULL){
        TRACEIT(3,"myFiles_SaveValueToFile		end fileInfo parameter == NULL");
        return -1;
    }
	if (cpLine == NULL){
		TRACEIT(3,"myFiles_SaveValueToFile		ends cpLine parameter == NULL");
		return -1;
	}
	if (fileInfo->File == NULL){
		TRACEIT(3,"myFiles_SaveValueToFile		ends No file opened");
		return -2;
	}

    fputs(cpLine,fileInfo->File);
	fputc('\n',fileInfo->File);

    TRACEIT(10,"myFiles_SaveValueToFile		ends");
    return 0;
}














