/*
    Copyright 2007, 2008, 2009 Geyer Klaus

    This file is part of Cat'sEyE.

    Cat'sEyE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Cat'sEyE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Cat'sEyE.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <gtk/gtk.h>
#include <gio/gio.h>
#include <gdk/gdkkeysyms.h>
#include <dirent.h>
#include <string.h>
#include <glib.h>
#include <stdlib.h>
#include <stdio.h>

#include "helpers.h"
#include "listitems.h"
#include "Lists.h"
#include "myfm.h"
#include "threads.h"
#include "confighandling.h"
#include "userdata.h"
#include "DragnDrop.h"
#include "fileshelf.h"

#include "mytrace.h"
#include "globalDefinitions.h"



char *Fileshelf_Names[] = {"path","itemname","size","owner","group","perm.","accessed","changed","target","type","TimeStamp_1","TimeStamp_2","type_d","Color","Internal"};
#define COLTYPESFILESHELF G_TYPE_STRING, G_TYPE_STRING, G_TYPE_UINT64, G_TYPE_INT, G_TYPE_INT, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT, G_TYPE_INT,G_TYPE_INT,G_TYPE_STRING,G_TYPE_INT


struct ItemAlreadyOnShelfStruct{
	int ret;
	char *cpName;
	char *cpPath;
};


int g_iFileshelfIsInitialized;
GtkNotebook *g_FileShelfNotebook;

extern GtkWidget *VPanedForFileShelf;
extern struct UserDataList TheTerminalStruct;
extern struct UserDataList TheStandardEditorStruct;
extern GtkWindow 	*win;
extern GtkWidget 	*Notebooks[SIDE_BOTH];
extern struct CopyReMoveList *g_CopyPasteShortcutStruct;

extern GString *g_gstrColorItems;
extern GString *g_gstrColorDirs;
extern GString *g_gstrColorLinks;
extern GString *g_gstrColorNotFound;


void InitFileshelf(char *Name);
void predefine_Fileshelf_Object(struct FileShelfObjectList *Object, char *Name);
int myFileshelfSortfunctionforNamesAndPathes(GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpointer userdata);
int checkIfFileIsAlreadyOnShelf(char *cpName, char *cpPath);
void openFileshelfContextMenu(GtkWidget *treeview, GdkEventButton *event, gpointer data);
gboolean oneFileShelfContextMenuReplied(GtkWidget *treeview, gpointer data);
gboolean oneFileShelfNoteBookContextMenuReplied(GtkWidget *treeview, gpointer data);
gboolean oneFileShelfMouseButtonPressed(GtkWidget *treeview, GdkEventButton *event, gpointer data);
void RemoveInternalMarkedItemsFileShelf (GtkWidget *menuitem, gpointer data);
//void oneFileShelfRowActivated(GtkTreeView *treeview, GtkTreePath *TreePath, GtkTreeViewColumn *col, gpointer data);
void oneRowActivated_FileShelfWrap(GtkTreeView *treeview, GtkTreePath *TreePath, GtkTreeViewColumn *col, gpointer data);
gboolean saveFileShelfItems (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, void *data);
void CopyPasteCreateList_FileShelf(struct FileShelfObjectList *Object, int CopyOrMove);
void CopyPasteProcessList_FileShelf(struct FileShelfObjectList *Object);
int FileShelfChooserDialog(int i_lastSelection, int i_withNewShelfOption, GtkNotebook *notebook, char *text);
void addItemsFromFileshelfToFileshelf(GtkWidget *menuitem, gpointer data);

void TabMenuRemoveActiveShelf(GtkWidget *menuitem, gpointer data);
void TabMenuAddNewFileshelf(GtkWidget *menuitem, gpointer data);
void TabMenuRenameFileshelf(GtkWidget *menuitem, gpointer data);
void TabMenuExportFileshelf(GtkWidget *menuitem, gpointer data);
void TabMenuImportFileshelf(GtkWidget *menuitem, gpointer data);
void TabMenuIntersectionItemsToNewShelf(GtkWidget *menuitem, gpointer data);


void saveFileShelfs(){
	TRACEIT(10,"saveFileShelfs		start");
	struct FileShelfObjectList *Object = FileShelftObjectList_getFirstObject();
	const gchar *Name;
	GString *gstrLine = g_string_sized_new(20);
	int iFileshelf=0;
	int iItem=0;
	struct AllInformationAllUseStruct info;
	info.Info_1 = (void *)&iFileshelf;
	info.Info_2 = (void *)&iItem;
	info.Info_3 = NULL; //becomes struct myFiles_struct

	if (NULL == Object){
		TRACEIT(10,"saveFileShelfs		end, No fileshelfs open");
		g_string_free(gstrLine, TRUE);
		return;
	}

	int i;
	int i_pages = gtk_notebook_get_n_pages (Object->notebook);

    struct myFiles_struct *stru = myFiles_openSaveFile("a+");
    if (stru == NULL){
        TRACEIT(5,"saveFileShelfs		end, error opening savefile");
		g_string_free(gstrLine, TRUE);
		return;
    }
    info.Info_3 = (void *)stru;
    myFiles_SaveValueToFile(stru, "\nThe following lines belong to the FileShelf section.\n");

    //saveFile_saveValue("\nThe following lines belong to the FileShelf section.\n", 0);

	for (i=0; i < i_pages; i++){

		if (0 != FileShelfObjectList_GetObjectByPageNumber(&Object, (guint ) i)){
			continue;
		}
		Name = gtk_label_get_text ((GtkLabel *)(Object->label));
		g_string_printf(gstrLine,SHELFNAMEID,iFileshelf);
		g_string_append(gstrLine,TYPESEPARATOR);
		g_string_append(gstrLine,Name);
		myFiles_SaveValueToFile(stru, gstrLine->str);
		//saveFile_saveValue(gstrLine->str, 0);

		gtk_tree_model_foreach(GTK_TREE_MODEL(Object->listStore),(GtkTreeModelForeachFunc) saveFileShelfItems,(void *)&info);

		iItem = 0;
		iFileshelf++;
	}
	myFiles_closeFile(stru);
	g_string_free(gstrLine,TRUE);
	TRACEIT(10,"saveFileShelfs		end");
}


void loadFileShelfs(char *FileName){
	TRACEIT(10,"loadFileShelfs		start");
	GString *gstrValue = g_string_sized_new(20);
	GString *gstrValue2 = g_string_sized_new(20);
	GString *gstrIdent = g_string_sized_new(10);
	int iItem=0;
	int iFileshelf=0;
	int retFS=-1;
	int retItm=-1;
    struct myFiles_struct *filestruct=NULL;
    if (FileName){
        filestruct=myFiles_openFile(FileName, "r");
    }
    else{
        filestruct=myFiles_openSaveFile("r");
    }
    if (filestruct == NULL){
        TRACEIT(10,"loadFileShelfs		end, error opening file");
    }

	do{
		g_string_printf(gstrIdent, SHELFNAMEID, iFileshelf);
		//if (AOtherFile) rewind(AOtherFile);
		retFS = myFiles_getValueFromIdentifier(filestruct, gstrIdent->str, 0);
		//retFS = saveFile_getValue(gstrIdent->str, gstrValue,AOtherFile);
		if (retFS == 1){
			NewFileShelf(filestruct->gstrValue->str);
			do{
				g_string_printf(gstrIdent, SHELFITEMPATH, iFileshelf,iItem);
				//if (AOtherFile) rewind(AOtherFile);
				retItm = myFiles_getValueFromIdentifier(filestruct, gstrIdent->str, 0);
				if (retItm == 1)
                    g_string_assign(gstrValue,filestruct->gstrValue->str);
				//retItm = saveFile_getValue(gstrIdent->str, gstrValue, AOtherFile);
				g_string_printf(gstrIdent, SHELFITEMNAME, iFileshelf,iItem);
				//if (AOtherFile) rewind(AOtherFile);
				retItm += myFiles_getValueFromIdentifier(filestruct, gstrIdent->str, 0);
				if (retItm == 2)
                    g_string_assign(gstrValue2,filestruct->gstrValue->str);
				//retItm += saveFile_getValue(gstrIdent->str, gstrValue2, AOtherFile);
				//if (AOtherFile) rewind(AOtherFile);
				if (retItm == 2){
					AddItemToFileshelf(NULL, gstrValue2->str, gstrValue->str, TRUE);
				}
				iItem++;
			}while(retItm == 2);
		}
		iItem = 0;
		iFileshelf++;
	}while (retFS == 1);
    myFiles_closeFile(filestruct);
	g_string_free(gstrValue,TRUE);
	g_string_free(gstrIdent,TRUE);
	g_string_free(gstrValue2,TRUE);
	TRACEIT(10,"loadFileShelfs		end");
}


gboolean saveFileShelfItems (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, void *data){
	TRACEIT(10,"saveFileShelfItems		start");
	g_assert ( data != NULL );
	gchar *Name=NULL;
	gchar *Path=NULL;
	GString *gstrLine= g_string_sized_new(20);
	struct AllInformationAllUseStruct *theStruct = (struct AllInformationAllUseStruct *)data;
	int iFileShelf = (theStruct->Info_1!=NULL)?*(int *)(theStruct->Info_1):0;
	int iItem = *(int *)theStruct->Info_2;
	struct myFiles_struct *stru=(struct myFiles_struct *)theStruct->Info_3;
	//FILE *TheFile = (theStruct->Info_3!=NULL)?(FILE *)(theStruct->Info_3):NULL;

	gtk_tree_model_get (model, iter, COL_FLSHLF_NAME, &Name, COL_FLSHLF_PATH, &Path, -1);

	if (Name != NULL && Path != NULL){
		g_string_printf(gstrLine, SHELFITEMNAME, iFileShelf, iItem);
		g_string_append(gstrLine, TYPESEPARATOR);
		g_string_append(gstrLine, Name);

		myFiles_SaveValueToFile(stru, gstrLine->str);
		//if (TheFile){
		//    g_string_append(gstrLine, "\n");
        //    fputs(gstrLine->str,TheFile);
		//}
        //else{
        //    saveFile_saveValue(gstrLine->str,0);
        //}

		g_string_printf(gstrLine, SHELFITEMPATH, iFileShelf, iItem);
		g_string_append(gstrLine, TYPESEPARATOR);
		g_string_append(gstrLine, Path);
		//if (TheFile){
        //    g_string_append(gstrLine, "\n");
        //    fputs(gstrLine->str,TheFile);
		//}
        //else
        //    saveFile_saveValue(gstrLine->str,0);
        myFiles_SaveValueToFile(stru, gstrLine->str);

		(*(int *)theStruct->Info_2)++;
	}

	g_string_free(gstrLine,TRUE);
	if (Name)
		g_free(Name);
	if (Path)
		g_free(Path);
	TRACEIT(10,"saveFileShelfItems		end");
	return FALSE; //go to the next item in list
}


void AddItemToFileshelf (struct FileShelfObjectList *Shelf, char *cpName, char *cpPath, gboolean AddItInAllCases){
    //Shelf may be NULL, in this case, the active fileshelf will get the new item.
	TRACEIT(10,"AddItemToFileshelf		start");
	struct FileAttributs attribs;
	GtkTreeIter iter;
	int iFileSNotHere=0;
	GString *NewName = g_string_sized_new(20);
	struct FileShelfObjectList *ActiveObject=NULL;

    if (cpName == NULL || cpPath == NULL){
        g_string_free(NewName,TRUE);
        TRACEIT(2,"AddItemToFileshelf		end, Parameter NULL");
        return;
    }

	if (g_iFileshelfIsInitialized == 0){
	    if (0 != FileShelf_NameForNewShelf(NewName)){
	        g_string_free(NewName,TRUE);
            TRACEIT(10,"AddItemToFileshelf		end , user cancelled");
	        return;
	    }
		InitFileshelf(NewName->str);
	}
	else if (1 == checkIfFileIsAlreadyOnShelf(cpName,cpPath) ){
        g_string_free(NewName,TRUE);
        TRACEIT(10,"AddItemToFileshelf		end, file is already on FileShelf");
		return;
	}

	iFileSNotHere = fileAttributesGet(NULL, 0, &attribs, cpPath, cpName);
	if (-1 == iFileSNotHere){
		if (AddItInAllCases){
			sprintf(attribs.type,"-");
			attribs.size = 0;
			attribs.owner = 0;
			attribs.group = 0;
			sprintf(attribs.permission,"-");
			sprintf(attribs.accessed,"-");
			sprintf(attribs.changed,"-");
			attribs.type_d = 0;
			attribs.utaccessed = 0;
			attribs.utchanged = 0;
		}
		else{
		    g_string_free(NewName,TRUE);
			TRACEIT(10,"AddItemToFileshelf		end 1");
			return;
		}
	}

    if (Shelf != NULL){
        ActiveObject = Shelf;
    }
    else{
        FileShelfObjectList_GetActiveObject(&ActiveObject);
    }

	if (ActiveObject == NULL){
	    if (attribs.gstrLinkTarget){
            g_string_free(attribs.gstrLinkTarget, TRUE);
        }
	    g_string_free(NewName,TRUE);
		TRACEIT(10,"AddItemToFileshelf		end, No FileShelf");
		return;
	}

	gtk_list_store_append(ActiveObject->listStore,&iter);
	gtk_list_store_set (ActiveObject->listStore, &iter,
		COL_FLSHLF_PATH,cpPath,
		COL_FLSHLF_NAME, cpName,
		COL_FLSHLF_FILETYPE, attribs.type,
		COL_FLSHLF_SIZE, attribs.size,
		COL_FLSHLF_OWNER, attribs.owner,
		COL_FLSHLF_GROUP, attribs.group,
		COL_FLSHLF_PERMISSION, attribs.permission,
		COL_FLSHLF_ACCESSED, attribs.accessed,
		COL_FLSHLF_CHANGED, attribs.changed,
		COL_FLSHLF_TYPE_D, attribs.type_d,
		COL_FLSHLF_TIMESTAMPACCESSED, attribs.utaccessed,
		COL_FLSHLF_TIMESTAMPCHANGED, attribs.utchanged,
		COL_FLSHLF_COLOR,(attribs.type_d==4)?g_gstrColorDirs->str:(attribs.type_d==10)?g_gstrColorLinks->str:(iFileSNotHere)?g_gstrColorNotFound->str:g_gstrColorItems->str,
		COL_FLSHLF_INTERNAL,0,
        -1);
    if (attribs.gstrLinkTarget){
        gtk_list_store_set (ActiveObject->listStore, &iter, COL_FLSHLF_TARGET, attribs.gstrLinkTarget->str, COL_FLSHLF_COLOR, g_gstrColorLinks->str, -1);
        g_string_free(attribs.gstrLinkTarget, TRUE);
    }


    //gtk_widget_queue_draw((GtkWidget *)win); //I really not a fan of this one here! because this could slow down all versy much.
    //hte problem: Add a item to a really big shelf, in a way you can not see the new item emediatly (scroll up or down). when putting the mouse
    //over the listview, the rows change their values. An the sideobject views this problem does exist TOO.

    g_string_free(NewName,TRUE);
	TRACEIT(10,"AddItemToFileshelf		end");
	return;
}


void InitFileshelf(char *Name){
	TRACEIT(10,"InitFileshelf		start");
	if (g_iFileshelfIsInitialized != 0){
		TRACEIT(10,"InitFileshelf		end, already initialized");
		return;
	}

	int NoteBookAlreadyProcessed = 0;
	if (g_FileShelfNotebook == NULL){
		g_FileShelfNotebook=(GtkNotebook *)g_object_new(GTK_TYPE_NOTEBOOK, "show-border",TRUE,"scrollable",TRUE,NULL);
		NoteBookAlreadyProcessed = 0;
	}
	else{
		NoteBookAlreadyProcessed = 1;
	}

	if (NoteBookAlreadyProcessed == 0){
		g_signal_connect (g_FileShelfNotebook,"button-press-event",G_CALLBACK(oneMouseButtonPressedOnNotebook),GINT_TO_POINTER(SIDE_FILESHELF));
		g_signal_connect (g_FileShelfNotebook,"popup-menu",G_CALLBACK(oneFileShelfNoteBookContextMenuReplied),GINT_TO_POINTER(SIDE_FILESHELF));
		g_signal_connect (Notebooks[SIDE_LEFT],"key-press-event",G_CALLBACK(oneKeyPressedOnNotebook),GINT_TO_POINTER(SIDE_FILESHELF));
	}

	FileShelfObjectList_StartWithObjectList();
	FileShelftObjectList_getFirstObject()->notebook = g_FileShelfNotebook;

	predefine_Fileshelf_Object(FileShelftObjectList_getFirstObject(), Name);

	if (NoteBookAlreadyProcessed == 0){
		gtk_paned_pack2(GTK_PANED(VPanedForFileShelf),GTK_WIDGET(g_FileShelfNotebook),TRUE,TRUE);

	}
	gtk_widget_show_all(GTK_WIDGET(g_FileShelfNotebook));
	g_iFileshelfIsInitialized = 1;
	TRACEIT(10,"InitFileshelf		end");
}


int NewFileShelf(char *Name){	//Name can be NULL, than the user is promted for input
    //The newly created shelf is always the active one.
	TRACEIT(10,"NewFileShelf		start");
	struct FileShelfObjectList *Object;
	GString *NewName = g_string_sized_new(20);

    if (NULL == Name){
        if (0 != FileShelf_NameForNewShelf(NewName)){
            g_string_free(NewName,TRUE);
            TRACEIT(10,"NewFileShelf		end , user cancelled");
            return 1;
        }
    }
    else{
        g_string_assign(NewName, Name);
    }

	if (g_iFileshelfIsInitialized == 0){
		InitFileshelf(NewName->str);
	}
	else{
		if (0 != FileShelfObjectList_NewObject(&Object) ){
		    g_string_free(NewName,TRUE);
			TRACEIT(2,"NewFileShelf		end, Error creating new fileshelf");
			return -1;
		}
		else{
			Object->notebook = Object->PrevItem->notebook;//(GtkNotebook *)FileShelfNotebook;
			predefine_Fileshelf_Object(Object, NewName->str);
			gtk_widget_show_all((GtkWidget *)Object->notebook);
			gtk_notebook_set_current_page (Object->notebook, gtk_notebook_get_n_pages (Object->notebook)-1);
		}
	}
	g_string_free(NewName,TRUE);
	TRACEIT(10,"NewFileShelf		end");
    return 0;
}


void predefine_Fileshelf_Object(struct FileShelfObjectList *Object, char *Name){
	TRACEIT(10,"predefine_Fileshelf_Object		start");
	int i;
	GtkTreeViewColumn	*col;
	GtkCellRenderer     	*renderer;
	GtkTreeSelection	*selection;

    if (Name == NULL){
        TRACEIT(1,"predefine_Fileshelf_Object		end Name == NULL!!!!!");
        return;
    }

	Object->ScrollWindow = gtk_scrolled_window_new (NULL,NULL);
	Object->listView = gtk_tree_view_new ();
	prepareWidgetForDragnDrop (Object->listView);
	gtk_tree_view_set_search_column((GtkTreeView *)Object->listView ,COL_FLSHLF_NAME);  //look in the description and you may wonder why calling this here once works all the time, but it does (in linux too?)
	Object->listStore = gtk_list_store_new( NUM_FLSHLF_COLS ,COLTYPESFILESHELF  );

	selection  = gtk_tree_view_get_selection((GtkTreeView *)Object->listView);
	gtk_tree_selection_set_mode(selection, GTK_SELECTION_MULTIPLE);

	for (i=0; i<NUM_FLSHLF_COLS ;i++){
		col=gtk_tree_view_column_new();

		//gtk_tree_view_column_set_alignment(col,1.0);	//only for text in header-buttons
		gtk_tree_view_column_set_resizable(col,TRUE);
		gtk_tree_view_column_set_sort_column_id(col,i);
		gtk_tree_view_column_set_reorderable(col,TRUE);
		gtk_tree_view_column_set_title(col, Fileshelf_Names[i]);
		gtk_tree_view_append_column(GTK_TREE_VIEW(Object->listView),col);
		renderer = gtk_cell_renderer_text_new();
		gtk_tree_view_column_pack_start(col, renderer, TRUE);
		gtk_tree_view_column_add_attribute(col,renderer,"text",i);
		gtk_tree_view_column_add_attribute(col,renderer,"foreground",COL_FLSHLF_COLOR);

//		g_signal_connect (col, "clicked",G_CALLBACK(oneColumnWasClicked),Object);

		if (i==COL_FLSHLF_NAME)
			(Object->actualSortCol) = col;
		//if (i >= COL_FLSHLF_TYPE_D  )
		if (i >= COL_FLSHLF_TIMESTAMPACCESSED )
			gtk_tree_view_column_set_visible(col,FALSE);
	}

	gtk_tree_view_set_model (GTK_TREE_VIEW(Object->listView), GTK_TREE_MODEL(Object->listStore));
	gtk_container_add(GTK_CONTAINER(Object->ScrollWindow),Object->listView);

	//connect Sortfunctions
	gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(Object->listStore),COL_FLSHLF_PATH,mysortfunctionforText,GINT_TO_POINTER(COL_FLSHLF_PATH),NULL);
	gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(Object->listStore),COL_FLSHLF_NAME,myFileshelfSortfunctionforNamesAndPathes,GINT_TO_POINTER(COL_FLSHLF_NAME),NULL);
	gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(Object->listStore),COL_FLSHLF_FILETYPE,mysortfunctionforText,GINT_TO_POINTER(COL_FLSHLF_FILETYPE),NULL);
	gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(Object->listStore),COL_FLSHLF_PERMISSION,mysortfunctionforText,GINT_TO_POINTER(COL_FLSHLF_PERMISSION),NULL);
	gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(Object->listStore),COL_FLSHLF_ACCESSED,mysortfunctionforNumbers,GINT_TO_POINTER(COL_FLSHLF_TIMESTAMPACCESSED),NULL);
	gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(Object->listStore),COL_FLSHLF_CHANGED,mysortfunctionforNumbers,GINT_TO_POINTER(COL_FLSHLF_TIMESTAMPCHANGED),NULL);
	gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(Object->listStore),COL_FLSHLF_SIZE,mysortfunctionforBigNumbers,GINT_TO_POINTER(COL_FLSHLF_SIZE),NULL);
	gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(Object->listStore),COL_FLSHLF_OWNER,mysortfunctionforNumbers,GINT_TO_POINTER(COL_FLSHLF_OWNER),NULL);
	gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(Object->listStore),COL_FLSHLF_GROUP,mysortfunctionforNumbers,GINT_TO_POINTER(COL_FLSHLF_GROUP),NULL);

	//connect some signals to the listview
	g_signal_connect (Object->listView,"row-activated",G_CALLBACK(oneRowActivated_FileShelfWrap),Object);
	g_signal_connect (Object->listView,"key-press-event",G_CALLBACK(oneFileShelfKeyPressedInView),Object);
	g_signal_connect (Object->listView,"button-press-event",G_CALLBACK(oneFileShelfMouseButtonPressed),Object->listView);
	g_signal_connect (Object->listView,"button-release-event",G_CALLBACK(oneFileShelfMouseButtonPressed),Object->listView);
	g_signal_connect (Object->listView,"popup-menu",G_CALLBACK(oneFileShelfContextMenuReplied),Object->listView);

	//Be sure to reread the comment in the gtk-reference not to use this function ...
	//this function is also used in myfm.c
	gtk_tree_view_set_rules_hint ((GtkTreeView *)Object->listView, TRUE);

	Object->label = gtk_label_new(NULL);
//	if (Name == NULL){
//		gtk_label_set_text(GTK_LABEL(Object->label),"New Fileshelf");
//		getNameForFileShelf(Object);
//	}
//	else{
	gtk_label_set_text(GTK_LABEL(Object->label),Name);
//	}

	gtk_tree_view_column_clicked(Object->actualSortCol);

	gtk_notebook_append_page(Object->notebook,GTK_WIDGET(Object->ScrollWindow),Object->label);
	gtk_notebook_set_tab_reorderable(Object->notebook,GTK_WIDGET(Object->ScrollWindow),TRUE);
	TRACEIT(10,"predefine_Fileshelf_Object		end");
}


void FileShelf_DeleteActiveTab(){
	TRACEIT(10,"FileShelf_DeleteActiveTab		start");
	GtkWidget *dialog, *label;
	GtkNotebook *Notebook=NULL;
	struct FileShelfObjectList *ActiveObject;
	FileShelfObjectList_GetActiveObject(&ActiveObject);
	if (ActiveObject == NULL){
		TRACEIT(10,"FileShelf_DeleteActiveTab		end, no active Object");
		return;
	}

	dialog = CreateMyDialog("Are you absolutly sure?", win, 2, "Cancel",GTK_RESPONSE_REJECT,"Ok",GTK_RESPONSE_ACCEPT, FALSE);
	label = gtk_label_new("Do you realy want to remove the selected fileshelf?\n");
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),label);
	gtk_widget_show_all(dialog);

	if (gtk_dialog_run(GTK_DIALOG(dialog))== GTK_RESPONSE_ACCEPT){
		Notebook = ActiveObject->notebook;
		gtk_notebook_remove_page(Notebook, gtk_notebook_get_current_page (Notebook));

		FileShelfObjectList_DeleteObject(ActiveObject);	//When ActiveObject is the firstObject and the lastone too, FirstObject variable becomes NULL too.
		ActiveObject = NULL;

		if (gtk_notebook_get_n_pages(Notebook) == 0){
			g_iFileshelfIsInitialized = 0;
			//g_object_unref (Notebook);
		}
	}
	gtk_widget_destroy(dialog);
	TRACEIT(10,"FileShelf_DeleteActiveTab		end");
}


void DeleteFileshelfThings(){
	TRACEIT(10,"DeleteFileshelfThings		start");
	if (g_iFileshelfIsInitialized == 0){
		TRACEIT(10,"DeleteFileshelfThings		end, FileShelf not initialized");
		return;
	}

	FileShelfObjectList_ClearAllForShutdown();
	g_iFileshelfIsInitialized = 0;
	TRACEIT(10,"DeleteFileshelfThings		end");
}


int FileShelf_NameForNewShelf(GString *NewName){
    TRACEIT(10,"FileShelf_NameForNewShelf		start");
    if (NewName == NULL){
        TRACEIT(10,"FileShelf_NameForNewShelf		end paramenter BNULL");
        return -1;
    }
    int ret;
    g_string_assign(NewName, "New FileShelf");
	ret = MyEasyInputWidget (win, "FileShelf name", "Name for new FileShelf:\n", NewName);



	//getNameForFileShelf(ActiveObject);
	TRACEIT(10,"FileShelf_NameForNewShelf		end");
    return ret;
}


void FileShelf_NewFileShelfFromIntersection(){
	TRACEIT(10,"FileShelf_NewFileShelfFromIntersection		start");
	static int  i_lastSelection=0;
	struct FileShelfObjectList *FirstObject;
	struct FileShelfObjectList *SecondObject;
	FileShelfObjectList_GetActiveObject(&FirstObject);
	if (FirstObject == NULL){
		TRACEIT(5,"FileShefl_NewFileShelfFromIntersection		end, No active FileShelf");
		return;
	}
	//int i_SourcePageNum = gtk_notebook_get_current_page (FirstObject->notebook);
	int i_PageToAdd=-1;

	int columnToSearch=COL_FLSHLF_NAME;
	int columnToUseToo=COL_FLSHLF_PATH;

	struct updateList *FirstFileShelfItemlist=NULL;
	struct AllInformationAllUseStruct Firstinfostru;
	Firstinfostru.Info_1=(void *)&FirstFileShelfItemlist;
	Firstinfostru.Info_2=(void *)&columnToSearch;
	Firstinfostru.Info_3=(void *)&columnToUseToo;
	Firstinfostru.Info_4=NULL;
	Firstinfostru.Info_5=NULL;

	struct updateList *SecondFileShelfItemlist=NULL;
	struct AllInformationAllUseStruct Secondinfostru;
	Secondinfostru.Info_1=(void *)&SecondFileShelfItemlist;
	Secondinfostru.Info_2=(void *)&columnToSearch;
	Secondinfostru.Info_3=(void *)&columnToUseToo;
	Secondinfostru.Info_4=NULL;
	Secondinfostru.Info_5=NULL;

	i_PageToAdd = FileShelfChooserDialog(i_lastSelection, 0, FirstObject->notebook, "Creates a new FileShelf with the items which are\nin the active FileShelf, but removes all items which are\nin the FileShelf you may choose here,\ncomparing only their names.\nIn mathematical terms: C=A\\B\n");
	if ( -1 == i_PageToAdd){
		TRACEIT(10,"addItemsFromFileshelfToFileshelf		ends, chanceled");
		return;
	}
	else{
		i_lastSelection = i_PageToAdd;
		//i_PageToAdd -= 1;
		gtk_notebook_set_current_page (FirstObject->notebook, i_PageToAdd);
	}
	FileShelfObjectList_GetActiveObject(&SecondObject);

	gtk_tree_model_foreach(GTK_TREE_MODEL(FirstObject->listStore),(GtkTreeModelForeachFunc) updateViewsFolder_Helper,&Firstinfostru);

	gtk_tree_model_foreach(GTK_TREE_MODEL(SecondObject->listStore),(GtkTreeModelForeachFunc) updateViewsFolder_Helper,&Secondinfostru);

	updateList_getFirstItem(&FirstFileShelfItemlist);
	updateList_getFirstItem(&SecondFileShelfItemlist);

	while (FirstFileShelfItemlist){
		updateList_getFirstItem(&SecondFileShelfItemlist);
		while(SecondFileShelfItemlist){
			if (strcmp(FirstFileShelfItemlist->gstrName->str, SecondFileShelfItemlist->gstrName->str)==0){
				FirstFileShelfItemlist->found = 1;
			}
			if (SecondFileShelfItemlist->NextItem)
				SecondFileShelfItemlist = SecondFileShelfItemlist->NextItem;
			else
				break;
		};


		if (FirstFileShelfItemlist->NextItem)
			FirstFileShelfItemlist = FirstFileShelfItemlist->NextItem;
		else
			break;
	};

	updateList_getFirstItem(&FirstFileShelfItemlist);
	GString *FlshlfName=g_string_sized_new(20);
	g_string_printf(FlshlfName, "(%s) \\ (%s)", gtk_label_get_text ((GtkLabel *)(FirstObject->label)), gtk_label_get_text ((GtkLabel *)(SecondObject->label)));
	//NewFileShelf("Intersection");
	NewFileShelf(FlshlfName->str);
	g_string_free(FlshlfName, TRUE);
	while (FirstFileShelfItemlist){
		if (FirstFileShelfItemlist->found != 1)
			AddItemToFileshelf (NULL, FirstFileShelfItemlist->gstrName->str, FirstFileShelfItemlist->gstrPath->str, TRUE);
		if (FirstFileShelfItemlist->NextItem)
			FirstFileShelfItemlist = FirstFileShelfItemlist->NextItem;
		else
			break;

	};


	updateList_deleteList(&FirstFileShelfItemlist);
	updateList_deleteList(&SecondFileShelfItemlist);

	//gtk_notebook_set_current_page  (SourceObject->notebook, i_SourcePageNum);

	TRACEIT(10,"FileShelf_NewFileShelfFromIntersection		end");
}

/*
void FileShelf_NewFileShelfFromSymmentricdiff(){

}
*/

void FileShefl_RenameActivePage(){
	TRACEIT(10,"FileShefl_RenameActivePage		start");
	struct FileShelfObjectList *ActiveObject;
	FileShelfObjectList_GetActiveObject(&ActiveObject);
	if (ActiveObject == NULL){
		TRACEIT(5,"FileShefl_RenameActivePage		end, No active FileShelf");
		return;
	}

    const gchar *Name = gtk_label_get_text ((GtkLabel *)(ActiveObject->label));
    GString *buf = g_string_sized_new(20);
    GString *NewName = g_string_new(Name);
    g_string_sprintf(buf,"new name of item:\n\"%s\"",Name);

	if (0 == MyEasyInputWidget (win, "FileShelf name", buf->str, NewName)){
        gtk_label_set_text(GTK_LABEL(ActiveObject->label),NewName->str);
	}
    g_string_free(NewName,TRUE);
	g_string_free(buf,TRUE);
	//getNameForFileShelf(ActiveObject);
	TRACEIT(10,"FileShefl_RenameActivePage		end");
}


void FileShefl_ImportFileShelf(){
    TRACEIT(10,"FileShefl_ImportActivePage		start");
    GtkWidget *dialog=NULL;
    //FILE *TheFile=NULL;
    GString *HomeDir = g_string_sized_new(10);
    char *filename=NULL;

    dialog = gtk_file_chooser_dialog_new ("Import FileShelf",
                    win,
                    GTK_FILE_CHOOSER_ACTION_OPEN,
                    GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                    GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
                    NULL);

    getHomeDirectory(&HomeDir);
    checkPathForLastChar(HomeDir);
    g_string_append(HomeDir,CATSEYEHOMEDIR);

    gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), HomeDir->str);
    if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT){
        filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
        loadFileShelfs(filename);
    }



//    if (TheFile == NULL){
//        MyMessageWidget(win,"FileShelf Import", "Error opening the file for reading\n");
//        if (filename) g_free (filename);
//        g_string_free(HomeDir, TRUE);
//        return;
//    }
 //   loadFileShelfs(TheFile);

    if (filename) g_free (filename);
    g_string_free(HomeDir, TRUE);
    gtk_widget_destroy (dialog);

    TRACEIT(10,"FileShefl_ImportActivePage		end");
}


void FileShefl_ExportActiveFileShelf(){
    TRACEIT(10,"FileShefl_ExportActivePage		start");
	struct FileShelfObjectList *ActiveObject;
	FileShelfObjectList_GetActiveObject(&ActiveObject);
	if (ActiveObject == NULL){
		TRACEIT(5,"FileShefl_ExportActivePage		end, No active FileShelf");
		return;
	}
    //FILE *TheFile=NULL;
    const gchar *Name;
    GString *gstrLine = g_string_sized_new(20);
    GString *HomeDir = g_string_sized_new(10);
	struct AllInformationAllUseStruct info;
    GtkWidget *dialog;
	int iItem=0;
	info.Info_1 = (void *)NULL;
	info.Info_2 = (void *)&iItem;
	struct myFiles_struct *stru=NULL;

	Name = gtk_label_get_text ((GtkLabel *)(ActiveObject->label));
	g_string_printf(gstrLine,SHELFNAMEID,0);
	g_string_append(gstrLine,TYPESEPARATOR);
	g_string_append(gstrLine,Name);

    dialog = gtk_file_chooser_dialog_new ("Export FileShelf",
				      win,
				      GTK_FILE_CHOOSER_ACTION_SAVE,
				      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
				      GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
				      NULL);
    gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (dialog), TRUE);

    getHomeDirectory(&HomeDir);
    checkPathForLastChar(HomeDir);
    g_string_append(HomeDir,CATSEYEHOMEDIR);

    gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), HomeDir->str);
    g_string_assign(HomeDir, FILESHELFFILEPREPEND);
    g_string_append(HomeDir, Name);
    g_string_append(HomeDir, ".");
    g_string_append(HomeDir, FILESHELFFILEIDENTIFIER);
    gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog), HomeDir->str);

    if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT){
        char *filename;

        filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
        stru = myFiles_openFile(filename, "w+");
        if (stru == NULL){
            MyMessageWidget(win, "FileShelf Export", "Error opening the file for writing\n");
            if (filename)
                g_free(filename);
            gtk_widget_destroy (dialog);
            return;
        }
        g_free (filename);
        //g_string_append(gstrLine,"\n");
        myFiles_SaveValueToFile(stru, gstrLine->str);
        //fputs(gstrLine->str ,TheFile);
        //info.Info_3 = (void *)TheFile;
        info.Info_3 = (void *)stru;
        gtk_tree_model_foreach(GTK_TREE_MODEL(ActiveObject->listStore),(GtkTreeModelForeachFunc) saveFileShelfItems,(void *)&info);
        myFiles_closeFile(stru);
    }

    gtk_widget_destroy (dialog);

	g_string_free(gstrLine,TRUE);
	g_string_free(HomeDir,TRUE);
    TRACEIT(10,"FileShefl_ExportActivePage		end");
}


/*
void getNameForFileShelf(struct FileShelfObjectList *Object){
	TRACEIT(10,"getNameForFileShelf		start");
	GtkWidget *dialog, *label;
	dialog = CreateMyDialog("FileShelf name", win, 2, "ok",GTK_RESPONSE_ACCEPT,"cancel",GTK_RESPONSE_REJECT, FALSE);
	const gchar *Name = gtk_label_get_text ((GtkLabel *)(Object->label));
	GtkWidget *EntryBox;
	GString *NewName = g_string_sized_new(20);
	GString *buf = g_string_sized_new(100);

	g_string_sprintf(buf,"new name of item:\n\"%s\"",Name);
	label = gtk_label_new(buf->str);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),label);
	EntryBox = gtk_entry_new();
	gtk_entry_set_max_length(GTK_ENTRY(EntryBox),MAXENTRYBOX);
	gtk_editable_set_editable(GTK_EDITABLE(EntryBox),TRUE);
	g_signal_connect (EntryBox, "activate",G_CALLBACK(NewDirectoryDialogEntryBoxActivate),dialog);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),EntryBox);
	g_object_set(EntryBox, "text", Name, NULL);

	gtk_widget_show_all(dialog);

	if (gtk_dialog_run(GTK_DIALOG(dialog))== GTK_RESPONSE_ACCEPT){
		g_object_get(EntryBox, "text", NewName, NULL);
		gtk_label_set_text(GTK_LABEL(Object->label),NewName->str);
	}
	gtk_widget_destroy(dialog);
	g_string_free(NewName,TRUE);
	g_string_free(buf,TRUE);
	TRACEIT(10,"getNameForFileShelf		end");
}
*/

gboolean checkIfFileIsAlreadyOnShelf_Helper (GtkTreeModel *model,GtkTreePath *path, GtkTreeIter *iter, gpointer data){
	TRACEIT(10,"checkIfFileIsAlreadyOnShelf_Helper		start");
	gchar *cpPath=NULL;
	gchar *cpName=NULL;
	struct ItemAlreadyOnShelfStruct *theStruct = (struct ItemAlreadyOnShelfStruct *)data;

	gtk_tree_model_get(model, iter, COL_FLSHLF_PATH, &cpPath, COL_FLSHLF_NAME, &cpName, -1);
	if (cpName!=NULL && cpPath!=NULL){
		if ( strcmp(theStruct->cpName, cpName)==0 && strcmp(theStruct->cpPath, cpPath)==0 ){
			g_free(cpPath);
			g_free(cpName);
			theStruct->ret=1;
			TRACEIT(10,"checkIfFileIsAlreadyOnShelf_Helper		end 1,");
			return TRUE;
		}
	}

	if (cpPath)
		g_free(cpPath);
	if (cpName)
		g_free(cpName);

	TRACEIT(10,"checkIfFileIsAlreadyOnShelf_Helper		end");
	return FALSE;
}


int checkIfFileIsAlreadyOnShelf(char *cpName, char *cpPath){
	TRACEIT(10,"checkIfFileIsAlreadyOnShelf		start");
	struct ItemAlreadyOnShelfStruct theStruct;
	theStruct.ret = 0;
	theStruct.cpName = cpName;
	theStruct.cpPath = cpPath;

	struct FileShelfObjectList *ActiveObject;
	FileShelfObjectList_GetActiveObject(&ActiveObject);
	if (ActiveObject == NULL){
		TRACEIT(10,"checkIfFileIsAlreadyOnShelf		end, No active FileShelf");
		return 1;
	}

	gtk_tree_model_foreach(GTK_TREE_MODEL(ActiveObject->listStore),checkIfFileIsAlreadyOnShelf_Helper,(void *)&theStruct);

	if (theStruct.ret == 1){
		TRACEIT(10,"checkIfFileIsAlreadyOnShelf		end 1");
		return 1;
	}
	else{
		TRACEIT(10,"checkIfFileIsAlreadyOnShelf		end 0");
		return 0;
	}
}


int myFileshelfSortfunctionforNamesAndPathes(GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpointer userdata){
	TRACEIT(12,"myFileshelfSortfunctionforNamesAndPathes		start");
	int ret=0;
	gchar *string1=NULL;
	gchar *string2=NULL;
	int type_d_1, type_d_2;
	gint sortedColumnId;
	GtkSortType sortOrder;

	struct FileShelfObjectList *ActiveObject;
	FileShelfObjectList_GetActiveObject(&ActiveObject);
	if (ActiveObject == NULL){
		TRACEIT(12,"myFileshelfSortfunctionforNamesAndPathes		end, No Active FileShelf");
		return 1;
	}

	//userdata hold info of which column (name or path) to sort
	gtk_tree_model_get(model, a, GPOINTER_TO_INT(userdata), &string1, COL_FLSHLF_TYPE_D, &type_d_1, -1);
	gtk_tree_model_get(model, b, GPOINTER_TO_INT(userdata), &string2, COL_FLSHLF_TYPE_D, &type_d_2, -1);

	if (string1 == NULL || string2 == NULL){
		if (string1 == NULL && string2 == NULL){
			TRACEIT(12,"myFileshelfSortfunctionforNamesAndPathes		ends ERRORSTRING");
			return ret;
		}
		ret = (string1 == NULL)?-1:1;
	}
	else{
		TRACEIT(12,"myFileshelfSortfunctionforNamesAndPathes		strdown");
		gtk_tree_sortable_get_sort_column_id(GTK_TREE_SORTABLE(ActiveObject->listStore),&sortedColumnId,&sortOrder);
		if (type_d_1 == 4 && type_d_2 != 4){
			if (sortOrder == GTK_SORT_ASCENDING)
				ret = -1;
			else
				ret = 1;
		}
		else if (type_d_1 != 4 && type_d_2 == 4){
			if (sortOrder == GTK_SORT_ASCENDING)
				ret = 1;
			else
				ret = -1;
		}
		else{
            ret = mysortfunctionshelperForCaseInsensitiveComparission(string1, string2, sortOrder);
		}
	}
	if (string1)
		g_free (string1);
	if (string2)
		g_free (string2);

	TRACEIT(12,"myFileshelfSortfunctionforNamesAndPathes		ends");
	return ret;
}


gboolean updateFileshelf_Helper (GtkTreeModel *model,GtkTreePath *path, GtkTreeIter *iter, gpointer data){
	TRACEIT(11,"updateFileshelf_Helper		start");
	gchar *cpPath=NULL;
	gchar *cpName=NULL;
	struct FileAttributs attribs;
	struct FileShelfObjectList *ActiveObject = (struct FileShelfObjectList *)data;

	gtk_tree_model_get(model, iter, COL_FLSHLF_PATH, &cpPath, COL_FLSHLF_NAME, &cpName, -1);
	if (cpPath != NULL && cpName != NULL){
		if (0 == fileAttributesGet(NULL, 0, &attribs, cpPath, cpName)){
			gtk_list_store_set (ActiveObject->listStore, iter,
			//COL_FLSHLF_NAME, cpName,
			COL_FLSHLF_FILETYPE, attribs.type,
			COL_FLSHLF_SIZE, attribs.size,
			//COL_FLSHLF_PATH,cpPath,
			COL_FLSHLF_OWNER, attribs.owner,
			COL_FLSHLF_GROUP, attribs.group,
			COL_FLSHLF_PERMISSION, attribs.permission,
			COL_FLSHLF_ACCESSED, attribs.accessed,
			COL_FLSHLF_CHANGED, attribs.changed,
			COL_FLSHLF_TYPE_D, attribs.type_d,
			COL_FLSHLF_TIMESTAMPACCESSED, attribs.utaccessed,
			COL_FLSHLF_TIMESTAMPCHANGED, attribs.utchanged,
			COL_FLSHLF_COLOR,(attribs.type_d==4)?g_gstrColorDirs->str:(attribs.type_d==10)?g_gstrColorLinks->str:g_gstrColorItems->str,
			-1);
			if (attribs.gstrLinkTarget){
                gtk_list_store_set (ActiveObject->listStore, iter, COL_FLSHLF_TARGET, attribs.gstrLinkTarget->str, COL_FLSHLF_COLOR, g_gstrColorLinks->str, -1);
                g_string_free(attribs.gstrLinkTarget, TRUE);
            }
		}
		else{
			gtk_list_store_set (ActiveObject->listStore, iter, COL_FLSHLF_COLOR, g_gstrColorNotFound->str,-1);
			//gtk_list_store_remove(ActiveObject->listStore,iter);	//I dont think we are allowed to do this here
		}
	}
	if (cpPath)
		g_free(cpPath);
	if (cpName)
		g_free(cpName);

	TRACEIT(11,"updateFileshelf_Helper		end");
	return FALSE;
}


void updateFileshelf(GtkWidget *menuitem, gpointer data){
	TRACEIT(10,"updateFileshelf		start");
	struct FileShelfObjectList *ActiveObject = (struct FileShelfObjectList *)data;
	if (ActiveObject == NULL){
		TRACEIT(10,"updateFileshelf		end, No active FileShelf");
		return;
	}
	ActiveObject->iRecursiveSizeCalculation=-1;
	gtk_tree_model_foreach(GTK_TREE_MODEL(ActiveObject->listStore),updateFileshelf_Helper,(void *)ActiveObject);
	TRACEIT(10,"updateFileshelf		end");
}


gboolean RemoveMissingFiles_Helper(GtkTreeModel *model,GtkTreePath *path, GtkTreeIter *iter, gpointer data){
	TRACEIT(11,"RemoveMissingFiles_Helper		start");
	char *cpColor=NULL;
	struct FileShelfObjectList *ActiveObject = (struct FileShelfObjectList *)data;

	gtk_tree_model_get(model, iter, COL_FLSHLF_COLOR, &cpColor, -1);
	if (cpColor != NULL && 0 == strcmp(cpColor, g_gstrColorNotFound->str)){
		gtk_list_store_set (ActiveObject->listStore, iter, COL_FLSHLF_INTERNAL, 1, -1);
	}

	if (cpColor)
		g_free(cpColor);
	TRACEIT(11,"RemoveMissingFiles_Helper		end");
	return FALSE;
}


void RemoveMissingFiles (GtkWidget *menuitem, gpointer data){
	TRACEIT(10,"RemoveMissingFiles		start");
	struct FileShelfObjectList *ActiveObject = (struct FileShelfObjectList *)data;
	if (ActiveObject == NULL){
		return;
	}

	updateFileshelf(menuitem, data);
	gtk_tree_model_foreach(GTK_TREE_MODEL(ActiveObject->listStore), RemoveMissingFiles_Helper, data);
	RemoveInternalMarkedItemsFileShelf(menuitem, data);
	TRACEIT(10,"RemoveMissingFiles		end");
}


gboolean RemoveInternalMarkedItemsFileShelf_Helper (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, GList **rowref_list){
	TRACEIT(11,"RemoveInternalMarkedItems_Helper		start");
	g_assert ( rowref_list != NULL );
	int i_InternalUse=0;
	GtkTreeRowReference  *rowref;

	gtk_tree_model_get (model, iter, COL_FLSHLF_INTERNAL, &i_InternalUse, -1);

	if (1 == i_InternalUse){
		rowref = gtk_tree_row_reference_new(model, path);
		*rowref_list = g_list_append(*rowref_list, rowref);
	}
	TRACEIT(11,"RemoveInternalMarkedItems_Helper		end");
	return FALSE; //go to the next item in list
}


void RemoveInternalMarkedItemsFileShelf (GtkWidget *menuitem, gpointer data){
	TRACEIT(10,"RemoveInternalMarkedItems		start");
	struct FileShelfObjectList *ActiveObject = (struct FileShelfObjectList *)data;
	GList *rr_list = NULL;    /* list of GtkTreeRowReferences to remove */
	GList *node;
	GtkTreeIter  iter;
	GtkTreePath *path;

	gtk_tree_model_foreach(GTK_TREE_MODEL(ActiveObject->listStore),(GtkTreeModelForeachFunc) RemoveInternalMarkedItemsFileShelf_Helper,&rr_list);

	for ( node = rr_list;  node != NULL;  node = node->next ){
		path = gtk_tree_row_reference_get_path((GtkTreeRowReference*)node->data);

		if (path){
			if (gtk_tree_model_get_iter(GTK_TREE_MODEL(ActiveObject->listStore), &iter, path)){
				gtk_list_store_remove(ActiveObject->listStore, &iter);
			}
		/* FIXME/CHECK: Do we need to free the path here? */
		}
	}

	g_list_foreach(rr_list, (GFunc) gtk_tree_row_reference_free, NULL);
	g_list_free(rr_list);
	TRACEIT(10,"RemoveInternalMarkedItems		end");
}


static void mayRemoveSelectedRows_Helper(GtkTreeModel *model,GtkTreePath *path,GtkTreeIter *iter,gpointer data){
	TRACEIT(11,"mayRemoveSelectedRows_Helper		start");
	struct FileShelfObjectList *ActiveObject = (struct FileShelfObjectList *)data;
	gtk_list_store_set (ActiveObject->listStore, iter, COL_FLSHLF_INTERNAL, 1,-1);
	TRACEIT(11,"mayRemoveSelectedRows_Helper		end");
}


void mayRemoveSelectedRows(GtkWidget *menuitem, gpointer data){	//delete from fileshelf, not the file itself
	TRACEIT(10,"mayRemoveSelectedRows		start");
	GtkWidget *dialog, *label;
	GtkTreeSelection	*selection;
	struct FileShelfObjectList *ActiveObject;

	if (data == NULL){
		FileShelfObjectList_GetActiveObject(&ActiveObject);
		data = (void *)ActiveObject;
	}
	else{
		ActiveObject = (struct FileShelfObjectList *)data;
	}

	if (ActiveObject == NULL){
		TRACEIT(10,"mayRemoveSelectedRows		end, no active FileShelf");
		return;
	}

	dialog = CreateMyDialog("Are you absolutly sure?", win, 2, "Cancel",GTK_RESPONSE_REJECT,"Ok",GTK_RESPONSE_ACCEPT, FALSE);
	label = gtk_label_new("Do you realy want to remove the \nselected Items\nfrom this FileShelf?");
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),label);
	gtk_widget_show_all(dialog);
	if (gtk_dialog_run(GTK_DIALOG(dialog))== GTK_RESPONSE_ACCEPT){
		selection  = gtk_tree_view_get_selection(GTK_TREE_VIEW(ActiveObject->listView));
		gtk_tree_selection_selected_foreach(selection, mayRemoveSelectedRows_Helper, (void *)ActiveObject);
	}
	gtk_widget_destroy(dialog);
	RemoveInternalMarkedItemsFileShelf(menuitem,data);
	TRACEIT(10,"mayRemoveSelectedRows		end");
}


void startItemsWithUserCommand_WraperforFileShelf(GtkWidget *menuitem, gpointer data){
	TRACEIT(10,"startItemsWithUserCommand_WraperforFileShelf		start");
	struct AllInformationAllUseStruct info;

	struct FileShelfObjectList *ActiveObject;
	FileShelfObjectList_GetActiveObject(&ActiveObject);
	if (ActiveObject == NULL){
		TRACEIT(10,"startItemsWithUserCommand_WraperforFileShelf		end, no active FileShelf");
		return;
	}

	info.source = SOURCE_FILESHELF;
	info.Info_1 = (void *)data;	//data is from type userdatastruct
	//info.Info_2 is reserverd!! for use in the function following this.
	info.Info_3 = (void *)ActiveObject->listView;
	startItemsWithUserCommand(menuitem, (void *)&info);
	TRACEIT(10,"startItemsWithUserCommand_WraperforFileShelf		end");
}


void callEachWith_WraperforFileShelf(GtkWidget *menuitem, gpointer data){
	TRACEIT(10,"callEachWith_WraperforFileShelf		start");
	struct AllInformationAllUseStruct info;

	struct FileShelfObjectList *ActiveObject;
	FileShelfObjectList_GetActiveObject(&ActiveObject);
	if (ActiveObject == NULL){
		TRACEIT(10,"callEachWith_WraperforFileShelf		end, no active FileShelf");
		return;
	}

	info.source = SOURCE_FILESHELF;
	info.Info_1 = (void *)ActiveObject->listView;
	callEachWith_Menuitem(menuitem, (void *)&info);
	TRACEIT(10,"callEachWith_WraperforFileShelf		end");
}


void callAllWith_WraperforFileShelf(GtkWidget *menuitem, gpointer data){
	TRACEIT(10,"callAllWith_WraperforFileShelf		start");
	struct AllInformationAllUseStruct info;

	struct FileShelfObjectList *ActiveObject;
	FileShelfObjectList_GetActiveObject(&ActiveObject);
	if (ActiveObject == NULL){
		TRACEIT(10,"callAllWith_WraperforFileShelf		end, no active FileShelf");
		return;
	}

	info.source = SOURCE_FILESHELF;
	info.Info_1 = (void *)ActiveObject->listView;
	callAllWith(menuitem, (void *)&info);
	TRACEIT(10,"callAllWith_WraperforFileShelf		end");
}


void renameItems_FileShelf(GtkWidget *menuitem, gpointer data){
	TRACEIT(10,"renameItems_FileShelf	start");

	GtkTreeSelection	*selection;
	struct AllInformationAllUseStruct info;

	struct FileShelfObjectList *ActiveObject;
	FileShelfObjectList_GetActiveObject(&ActiveObject);
	if (ActiveObject == NULL){
		TRACEIT(10,"renameItems_FileShelf	end, no active FileShelf");
		return;
	}

	selection  = gtk_tree_view_get_selection(GTK_TREE_VIEW(ActiveObject->listView));
	info.source = SOURCE_FILESHELF;
	info.Info_1 = (void *)ActiveObject->listStore;
	info.Info_2 = NULL;
	info.Info_3 = NULL;

	//gtk_tree_selection_selected_foreach(selection, renameItemsHelper, (void *)&info);
	renameItemsHelper(selection, &info);
	updateFileshelf(NULL,NULL);
	TRACEIT(10,"renameItems_FileShelf	end");
}


static void getSizeOfSelectedItemsRecursive_FileShelf_getItems (GtkTreeModel *model,GtkTreePath *path,GtkTreeIter *iter,gpointer data){
	TRACEIT(10,"getSizeOfSelectedItemsRecursive_FileShelf_getItems		start");

	struct RecursiveSizeStructure *stru = (struct RecursiveSizeStructure *)data;
	gchar *gcname=NULL;
	gchar *gcpath=NULL;
	gint type_d;
	GString *gstrBuf = g_string_sized_new(20);
	int ret=0;
	guint64 isize;

    gtk_tree_model_get(model, iter, COL_FLSHLF_PATH, &gcpath, COL_FLSHLF_SIZE, &isize, COL_FLSHLF_NAME, &gcname, COL_FLSHLF_TYPE_D, &type_d, -1);
    if (gcpath != NULL && gcname != NULL && strcmp(gcname,".")!=0 && strcmp(gcname,"..")!=0){
        g_string_assign(gstrBuf,gcpath);
        checkPathForLastChar(gstrBuf);
        g_string_append(gstrBuf, gcname);
        ret =  AddNewGStringToDPointer(&(stru->gstrDPItems),stru->ItemCounter, gstrBuf);
        if (ret == 0){
            stru->ItemStringCounter++;
            ret += AddNewGUInt64ToPointerArray(&(stru->intPSizes), stru->ItemCounter, isize);
        }
        if (ret == 0){
            ret += AddNewGUInt64ToPointerArray(&(stru->intPTypes), stru->ItemCounter, (guint64)type_d);
        }
        if (ret == 0){
            stru->ItemCounter++;
        }
    }

	g_string_free(gstrBuf,TRUE);
	if (gcname)
		g_free(gcname);
	if (gcpath)
		g_free(gcpath);

	TRACEIT(10,"getSizeOfSelectedItemsRecursive_FileShelf_getItems		end");
}


void sizeOfSelectedItemsRecursive_FileShelf (GtkWidget *menuitem, gpointer data){	//Get size if selected Items, directories are searched recursivly
	TRACEIT(10,"sizeOfSelectedItemsRecursive_FileShelf		start");
	struct FileShelfObjectList *Object = (struct FileShelfObjectList *)data;
	GtkTreeSelection	*selection;

	if( !g_thread_supported() ){
		g_thread_init(NULL);
     		//gdk_threads_init();                   // Called to initialize internal mutex "gdk_threads_mutex".
	}
	struct RecursiveSizeStructure *stru=malloc(sizeof(struct RecursiveSizeStructure));
	stru->mainWin = win;

	selection  = gtk_tree_view_get_selection(GTK_TREE_VIEW(Object->listView));

	//Get all selected items
	stru->gstrPath=NULL;
	stru->gstrDPItems=NULL;
	stru->intPSizes = NULL;
	stru->intPTypes = NULL;
	stru->ItemStringCounter=0;
	stru->ItemCounter=0;
	stru->SideObjectOrFileshelfItem=SOURCE_FILESHELF; //FileShelf Identifier
	gtk_tree_selection_selected_foreach(selection, getSizeOfSelectedItemsRecursive_FileShelf_getItems,(void *) stru);

	g_thread_create((GThreadFunc)getSizeOfSelectedItemsRecursive_dialog, (void *)stru , FALSE, NULL);
	TRACEIT(10,"sizeOfSelectedItemsRecursive_FileShelf		end");
}


void searchStringInAndHighlightItems_WraperforFileShelf(GtkWidget *menuitem, gpointer data){
	TRACEIT(10,"searchStringInAndHighlightItems_WraperforFileShelf		start");
	struct AllInformationAllUseStruct info;
	info.source = SOURCE_FILESHELF;
	info.Info_1 = (void *)data;
	searchStringInAndHighlightItems(menuitem, (void *)&info);
	TRACEIT(10,"searchStringInAndHighlightItems_WraperforFileShelf		end");
}

/*
void searchStringInDirectoryRecursive_FileShelf(GtkWidget *menuitem, struct SideObjectList *Object){
	TRACEIT(10,"searchStringInDirectoryRecursive_FileShelf		start");
	find_simple(Object->gstrPath->str);
	TRACEIT(10,"searchStringInDirectoryRecursive_FileShelf		start");
}
*/

void editItemInformation_FileShelf(GtkWidget *menuitem, gpointer data){
    TRACEIT(10,"editItemInformation_FileShelf		start");

    GtkTreeSelection	*selection;
    struct FileShelfObjectList *Object = (struct FileShelfObjectList *)data;
    selection  = gtk_tree_view_get_selection(GTK_TREE_VIEW(Object->listView));

    int ibreakall=0;

    GtkTreeModel *model;
    GList *selList = gtk_tree_selection_get_selected_rows (selection,&model);
    GList *buflist=NULL;
    GtkTreeIter iter;

    gchar *cname=NULL;
    gchar *cpath=NULL;
    GString *gstrPath = g_string_sized_new(10);
    GString *gstrName = g_string_sized_new(10);
    int ret=0;
    int sort_column_id;
    GtkSortType order;

    //Whenever working with Pathlists avoid reordering or updating the treeview.
    gtk_tree_sortable_get_sort_column_id (GTK_TREE_SORTABLE(Object->listStore), &sort_column_id, &order);
    gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE(Object->listStore),GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID, order);

    buflist=selList;
    while(buflist){
        if (ibreakall==1){
            break;
        }
        if (TRUE == gtk_tree_model_get_iter (model, &iter, (GtkTreePath *)buflist->data)){
            gtk_tree_model_get(model, &iter, COL_FLSHLF_PATH, &cpath, COL_FLSHLF_NAME, &cname, -1);
            if (cpath != NULL && cname != NULL){
                g_string_assign(gstrPath,cpath);
                g_string_assign(gstrName,cname);
                ret = MyEasyInputWidget_Two (win, "edit item", "path", gstrPath, "name", gstrName, "Ok", "Cancel all", "Cancel", NULL);
                if (0 == ret){
                    gtk_list_store_set ((GtkListStore *)Object->listStore, &iter, COL_FLSHLF_NAME, gstrName->str,COL_FLSHLF_PATH, gstrPath->str, -1);
                    updateFileshelf_Helper (model, (GtkTreePath *)buflist->data, &iter, data);
                }
                else if (1 == ret){
                    ibreakall=1;
                }
            }
        }
        buflist=buflist->next;
    }

    gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE(Object->listStore),sort_column_id, order);

    g_string_free(gstrName, TRUE);
    g_string_free(gstrPath, TRUE);

	if (cname)
		g_free(cname);
	if (cpath)
		g_free(cpath);

	if (selList != NULL){
		g_list_foreach (selList,(GFunc) gtk_tree_path_free, NULL);
		g_list_free (selList);
	}

    TRACEIT(10,"editItemInformation_FileShelf		end");
}


gboolean oneFileShelfKeyPressedInView(GtkWidget *widget, GdkEventKey *event, gpointer data){
    //data = struct FileShelfObjectList
	TRACEIT(10,"oneFileShelfKeyPressedInView		start");
	guint modifiers;
	modifiers = gtk_accelerator_get_default_mod_mask ();
	struct FileShelfObjectList *Object = (struct FileShelfObjectList *)data;

	if ( (event->keyval == GDK_Delete)/* && (event->state & modifiers) == GDK_CONTROL_MASK*/){
		mayRemoveSelectedRows(NULL, NULL);
		TRACEIT(10,"oneFileShelfKeyPressedInView		end");
		return TRUE;
	}
	else if ( (event->keyval == GDK_Return)){
        //go to directory or start userprogram with marked items
        oneRowActivated_FileShelfWrap(NULL, NULL, NULL, data);
        return TRUE;
	}
	else if (event->keyval == GDK_F2){
        //rename selected
        renameItems_FileShelf(NULL, NULL);
        return TRUE;
	}
	else if (event->keyval == GDK_F4){
	    //open Terminal view with 'F4'
        getTerminal();
        if (TheTerminalStruct.UserCommand != NULL && TheTerminalStruct.UserCommandName != NULL){
            //TheTerminalStruct.Object = (struct SideObjectList *)data;
            startItemsWithUserCommand_WraperforFileShelf(NULL,&TheTerminalStruct);
        }
        return TRUE;
	}
	else if (event->keyval == GDK_F5){
	    //Update view with 'F5'
	    updateFileshelf(NULL, data);
        //updateViewWithNewFolder((struct SideObjectList *)data, NULL);
        return TRUE;
	}
	else if (event->keyval == GDK_F6){
	    //Update view with 'F6'
	    updateFileshelf(NULL, data);

	    struct AllInformationAllUseStruct *stru = malloc (sizeof(struct AllInformationAllUseStruct));
	    if (stru == NULL){
            TRACEIT(1,"oneFileShelfKeyPressedInView		error allocating memory");
            return TRUE;
	    }
        stru->source=SOURCE_FILESHELF;
        stru->Info_1 = data;
        stru->Info_2 = NULL;
        stru->Info_3 = NULL;
        stru->Info_4 = NULL;
        stru->Info_5 = NULL;
        Object->iRecursiveSizeCalculation=0;
        calcDirectorySizeInView_wrap(stru);

        return TRUE;
	}

	else if ((event->keyval == 'c' || event->keyval == 'C') && (event->state & modifiers) == GDK_CONTROL_MASK){
        //Copy Paste // here is copy
        CopyPasteCreateList_FileShelf(data, 1);
        return TRUE;
	}
	else if ((event->keyval == 'x' || event->keyval == 'X') && (event->state & modifiers) == GDK_CONTROL_MASK){
        //Copy Paste // here is cut
        CopyPasteCreateList_FileShelf(data, 2);
        return TRUE;
	}
	else if ((event->keyval == 'v' || event->keyval == 'V') && (event->state & modifiers) == GDK_CONTROL_MASK){
        //Copy Paste // paste to fileshelf (it is never moved)
        CopyPasteProcessList_FileShelf(data);
        return TRUE;
	}
	else if ((event->keyval == 'n' || event->keyval == 'N') && (event->state & modifiers) == GDK_CONTROL_MASK){
        //new fileshelf
        NewFileShelf(NULL);
        return TRUE;
	}
	else if ((event->keyval == 'i' || event->keyval == 'I') && (event->state & modifiers) == GDK_CONTROL_MASK){
        //size of selected
        sizeOfSelectedItemsRecursive_FileShelf (NULL, data);
        return TRUE;
	}
	else if ((event->keyval == 's' || event->keyval == 'S' || event->keyval == 'd' || event->keyval == 'D') && (event->state & modifiers) == GDK_CONTROL_MASK){
	    //add marked items to another shelf
        addItemsFromFileshelfToFileshelf(NULL, data);
	    return TRUE;
	}
	else if ((event->keyval == 'e' || event->keyval == 'E') && (event->state & modifiers) == GDK_CONTROL_MASK){
	    //edit item information
        editItemInformation_FileShelf(NULL, data);
	    return TRUE;
	}
    else if ((event->keyval == 'f' || event->keyval == 'F') && (event->state & modifiers) == GDK_MOD1_MASK){
        //my search function
        searchStringInAndHighlightItems_WraperforFileShelf(NULL, Object->listView);
        return TRUE;
	}


	/*
	else if ((event->keyval == '1') && (event->state & modifiers) == GDK_MOD1_MASK){
        //next page on fileshelf notebook
        gtk_notebook_next_page(Object->notebook);
        return TRUE;
	}
	else if ((event->keyval == '1') && (event->state & modifiers) == GDK_CONTROL_MASK){
        //next page on fileshelf notebook
        gtk_notebook_prev_page(Object->notebook);
        return TRUE;
	}
    */

	TRACEIT(10,"oneFileShelfKeyPressedInView		end");
	return FALSE;
}


void oneRowActivated_FileShelfWrap(GtkTreeView *treeview, GtkTreePath *TreePath, GtkTreeViewColumn *col, gpointer data){
    struct AllInformationAllUseStruct stru;

    stru.source = SOURCE_FILESHELF;
    stru.Info_1 = data; //FileShelfObject
    stru.Info_2 = NULL;
    stru.Info_3 = NULL;
    stru.Info_4 = NULL;
    stru.Info_5 = NULL;

    //oneRowActivated (treeview, TreePath, col, (void *)&stru);
    oneRowActivated (&stru);
}

static void copyFileShelfToPannelHelper(GtkTreeModel *model,GtkTreePath *path,GtkTreeIter *iter,gpointer data){
	TRACEIT(10,"copyFileShelfToPannelHelper		start");
	//struct SideObjectList *Object=(struct SideObjectList *)data;
	struct FileShelfObjectList *Object = (struct FileShelfObjectList *)data;
	struct SideObjectList *otherObject=NULL;

	SideObjectList_GetActiveObject((GtkNotebook *)Notebooks[Object->iCopyReMoveSide], &otherObject);

	if (otherObject == NULL){
		TRACEIT(10,"copyFileShelfToPannelHelper		ends, otherObject = NULL");
		return;
	}

	gchar *Name = NULL;
	gchar *Path = NULL;
	GString *source = g_string_sized_new(300);
	GString *dest = g_string_new(otherObject->gstrPath->str);

	if (Object->data == NULL){
		g_string_free(source,TRUE);
		g_string_free(dest,TRUE);
		TRACEIT(10,"copyFileShelfToPannelHelper		ends, data = NULL");
		return;
	}
	struct CopyReMoveList **FirstItem = (struct CopyReMoveList **)Object->data;

	if (gtk_tree_model_get_iter(model, iter, path)){
		gtk_tree_model_get(model, iter, COL_FLSHLF_NAME, &Name, COL_FLSHLF_PATH, &Path, -1);
		g_string_assign(source,Path);
		checkPathForLastChar(source);
		if (!(strcmp(Name, "..")==0 || strcmp(Name,".")==0)){
			g_string_append(source,Name);
			checkPathForLastCharOtherwise(source);
			checkPathForLastCharOtherwise(dest);
			if (!g_string_equal(source,dest)){
				CopyReMoveList_AddItem ( FirstItem, source->str, dest->str, NULL, otherObject, Object);
			}
		}
	}

	if (Name)
		g_free(Name);
	if (Path)
		g_free(Path);
	g_string_free(source,TRUE);
	g_string_free(dest,TRUE);
	TRACEIT(10,"copyFileShelfToPannelHelper		ends");
}


void copyFileShelfToPannel(GtkWidget *menuitem, gpointer data){
	TRACEIT(10,"copyFileShelfToPannel		start");
	struct FileShelfObjectList *Object = (struct FileShelfObjectList *)data;
	GtkTreeSelection	*selection;

	struct CopyReMoveList *FirstItem=NULL;
	Object->data = (void *)&FirstItem;

	selection  = gtk_tree_view_get_selection(GTK_TREE_VIEW(Object->listView));

	gtk_tree_selection_selected_foreach(selection, copyFileShelfToPannelHelper, data);

	if (FirstItem == NULL){
		TRACEIT(7,"copyFileShelfToPannel		end, FirstItem = NULL");
		return;
	}
	ProcessItemsViaListWrap(FirstItem, 1);

	TRACEIT(10,"copyFileShelfToPannel		ends");
}

void copyFileShelfToRightPannel(GtkWidget *menuitem, gpointer data){
	TRACEIT(10,"copyFileShelfToRightPannel		start");
	struct FileShelfObjectList *Object = (struct FileShelfObjectList *)data;
	Object->iCopyReMoveSide=SIDE_RIGHT;
	copyFileShelfToPannel(menuitem, data);
	TRACEIT(10,"copyFileShelfToRightPannel		end");
}

void copyFileShelfToLeftPannel(GtkWidget *menuitem, gpointer data){
	TRACEIT(10,"copyFileShelfToLeftPannel		start");
	struct FileShelfObjectList *Object = (struct FileShelfObjectList *)data;
	Object->iCopyReMoveSide=SIDE_LEFT;
	copyFileShelfToPannel(menuitem, data);
	TRACEIT(10,"copyFileShelfToLeftPannel		end");
}


void moveFileShelfToPannel(GtkWidget *menuitem, gpointer data){
	TRACEIT(10,"copyFileShelfToPannel		start");
	struct FileShelfObjectList *Object = (struct FileShelfObjectList *)data;
	GtkTreeSelection	*selection;

	struct CopyReMoveList *FirstItem=NULL;
	Object->data = (void *)&FirstItem;

	selection  = gtk_tree_view_get_selection(GTK_TREE_VIEW(Object->listView));

	gtk_tree_selection_selected_foreach(selection, copyFileShelfToPannelHelper, data);

	if (FirstItem == NULL){
		TRACEIT(7,"copyFileShelfToPannel		ends, FirstItem = NULL");
		return;
	}
	ProcessItemsViaListWrap(FirstItem, 2);
	TRACEIT(10,"copyFileShelfToPannel		ends");
}


void moveFileShelfToRightPannel(GtkWidget *menuitem, gpointer data){
	TRACEIT(10,"moveFileShelfToRightPannel		start");
	struct FileShelfObjectList *Object = (struct FileShelfObjectList *)data;
	Object->iCopyReMoveSide=SIDE_RIGHT;
	moveFileShelfToPannel(menuitem, data);
	TRACEIT(10,"moveFileShelfToRightPannel		end");
}


void moveFileShelfToLeftPannel(GtkWidget *menuitem, gpointer data){
	TRACEIT(10,"moveFileShelfToLeftPannel		start");
	struct FileShelfObjectList *Object = (struct FileShelfObjectList *)data;
	Object->iCopyReMoveSide=SIDE_LEFT;
	moveFileShelfToPannel(menuitem, data);
	TRACEIT(10,"moveFileShelfToLeftPannel		end");
}


void CopyPasteCreateList_FileShelf(struct FileShelfObjectList *Object, int CopyOrMove){
    TRACEIT(10,"CopyPasteCreateList_FileShelf		start");
    if (Object == NULL){
        TRACEIT(10,"CopyPasteCreateList_FileShelf		ends");
		return;
    }
	GtkTreeSelection	*selection;
	struct CopyReMoveList *ObjectsList=NULL;
	Object->iCopyReMoveSide=SIDE_LEFT; //needs to be defined but does nothave any effect
	selection  = gtk_tree_view_get_selection(GTK_TREE_VIEW(Object->listView));

    if (g_CopyPasteShortcutStruct != NULL){
        CopyReMoveList_DeleteList(g_CopyPasteShortcutStruct);
        g_CopyPasteShortcutStruct = NULL;
    }

    ObjectsList = (struct CopyReMoveList *)Object->data;
    Object->data = (void *)&g_CopyPasteShortcutStruct;

	gtk_tree_selection_selected_foreach(selection, copyFileShelfToPannelHelper, (void *)Object);

    Object->data = (void *)ObjectsList;

    if (g_CopyPasteShortcutStruct != NULL){
        g_CopyPasteShortcutStruct->CopyMoveOrRemove = CopyOrMove;
	}

    TRACEIT(10,"CopyPasteCreateList_FileShelf		ends");
}


void CopyPasteProcessList_FileShelf(struct FileShelfObjectList *Object){
    TRACEIT(10,"CopyPasteProcessList_FileShelf		start");
    struct CopyReMoveList *buflist=NULL;

    if (g_CopyPasteShortcutStruct == NULL || Object == NULL){
        TRACEIT(10,"CopyPasteProcessList_FileShelf		ends");
        return;
    }
    buflist = g_CopyPasteShortcutStruct;
    GString *gstrPath=g_string_sized_new(10);
    GString *gstrName=g_string_sized_new(10);

    while (buflist){
        g_string_assign(gstrPath, buflist->string1->str);
        g_string_assign(gstrName, buflist->string1->str);
        GetPathFromPathWithName(gstrPath, 1);
        getLastStringInPath(gstrName);
        AddItemToFileshelf (NULL, gstrName->str, gstrPath->str, TRUE);
        buflist = buflist->NextItem;
    };
    g_string_free(gstrPath, TRUE);
    g_string_free(gstrName, TRUE);

    TRACEIT(10,"CopyPasteProcessList_FileShelf		ends");
}


static void mayDeleteSelectedFileShelfFilesHelper(GtkTreeModel *model,GtkTreePath *path,GtkTreeIter *iter,gpointer data){
	TRACEIT(10,"mayDeleteSelectedFileShelfFilesHelper		start");
	gchar *Name=NULL;
	gchar *Path=NULL;
	GString *WholeName=g_string_sized_new(300);
	struct FileShelfObjectList *Object = (struct FileShelfObjectList *)data;

	if (Object->data == NULL){
		g_string_free(WholeName,TRUE);
		TRACEIT(10,"mayDeleteSelectedFileShelfFilesHelper		end, data = NULL");
		return;
	}
	struct CopyReMoveList **FirstItem = (struct CopyReMoveList **)Object->data;

	if (gtk_tree_model_get_iter(model, iter, path)){
		gtk_tree_model_get(model, iter, COL_FLSHLF_NAME, &Name, COL_FLSHLF_PATH, &Path, -1);
		if (!(strcmp(Name, "..")==0 || strcmp(Name,".")==0)){
			g_string_assign(WholeName,Name);
			cutSpacesFromGString(WholeName);
			if (strlen (WholeName->str) == 0){
				if (Name)
					g_free(Name);
				if (Path)
					g_free(Path);
				g_string_free(WholeName,TRUE);
				TRACEIT(10,"mayDeleteSelectedFileShelfFilesHelper		end 1");
				return;
			}
			g_string_assign(WholeName,Path);
			checkPathForLastChar(WholeName);
			g_string_append(WholeName,Name);
			CopyReMoveList_AddItem ( FirstItem, WholeName->str, NULL, NULL, NULL, Object);
		}
	}
	if (Name)
		g_free(Name);
	if (Path)
		g_free(Path);
	g_string_free(WholeName,TRUE);
	TRACEIT(10,"mayDeleteSelectedFileShelfFilesHelper		ends");
}


void mayDeleteSelectedFileShelfFiles(GtkWidget *menuitem, gpointer data){
	TRACEIT(10,"mayDeleteSelectedFileShelfFiles		start");
	GtkWidget *dialog, *label;
	GtkTreeSelection	*selection;
	struct FileShelfObjectList *Object = (struct FileShelfObjectList *)data;
	struct CopyReMoveList *FirstItem=NULL;
	Object->data = (void *)&FirstItem;
	dialog = CreateMyDialog("Are you absolutly sure?", win, 2, "Cancel",GTK_RESPONSE_REJECT,"Ok",GTK_RESPONSE_ACCEPT, FALSE);
	label = gtk_label_new("Do you realy want to delete\nthe selected Items?");
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),label);
	gtk_widget_show_all(dialog);
	if (gtk_dialog_run(GTK_DIALOG(dialog))== GTK_RESPONSE_ACCEPT){
		selection  = gtk_tree_view_get_selection(GTK_TREE_VIEW(Object->listView));
		gtk_tree_selection_selected_foreach(selection, mayDeleteSelectedFileShelfFilesHelper, data);
		gtk_widget_destroy(dialog);
		if (FirstItem == NULL){
			TRACEIT(10,"mayDeleteSelectedFileShelfFiles		end, FirstItem = NULL");
			return;
		}
		ProcessItemsViaListWrap(FirstItem, 3);
	}
	else{
		gtk_widget_destroy(dialog);
	}
	TRACEIT(10,"mayDeleteSelectedFileShelfFiles		ends");
}

/*
void checkIfOneSelectionIsPathFileShelfHelper(GtkTreeModel *model,GtkTreePath *path,GtkTreeIter *iter,gpointer data){
	TRACEIT(10,"checkIfOneSelectionIsPathHelper		start");
	int type_d;
	gint *TheInfo = (gint *)data;

	if (gtk_tree_model_get_iter(model, iter, path)){
		gtk_tree_model_get(model, iter, COL_FLSHLF_TYPE_D, &type_d, -1);
		if (type_d==4 || type_d==10){
			*TheInfo = *TheInfo + 1;
			TRACEIT(10,"checkIfOneSelectionIsPathHelper		ends");
			return;
		}
	}
	TRACEIT(10,"checkIfOneSelectionIsPathHelper		ends");
	return;
}


int checkIfOneSelectionIsPathFileShelf(gpointer data){
	TRACEIT(10,"checkIfOneSelectionIsPath		start");
	GtkTreeSelection	*selection;
	gint IsPath=0;

	selection  = gtk_tree_view_get_selection(GTK_TREE_VIEW(data));
	gtk_tree_selection_selected_foreach(selection, checkIfOneSelectionIsPathFileShelfHelper, &IsPath);

	TRACEIT(10,"checkIfOneSelectionIsPath		ends");
	return IsPath;
}
*/

static void openFileShelfDirectoryItemHelper(GtkTreeModel *model,GtkTreePath *TreePath,GtkTreeIter *iter,gpointer data){
	TRACEIT(10,"openFileShelfDirectoryItemHelper		start");
	gchar *name=NULL;
	gchar *path=NULL;
	gint type_d;
	GString *NewPath = g_string_sized_new(20);

	gtk_tree_model_get_iter(model,iter,TreePath);
	gtk_tree_model_get(model, iter,COL_FLSHLF_NAME,&name,COL_FLSHLF_TYPE_D,&type_d,COL_FLSHLF_PATH,&path, -1);
	if (name != NULL && path != NULL){
		if ((type_d==4 || type_d==10) && strcmp(name,".")!=0 && strcmp(name,"..")!=0){	//Folder!
			g_string_assign(NewPath,path);
			checkPathForLastChar(NewPath);
			g_string_append(NewPath,name);
			SideObjectList_CreateNewViewWithPath(NULL,NewPath->str,NULL);
		}
	}
	g_free(name);
	g_free(path);
	g_string_free(NewPath,TRUE);
	TRACEIT(10,"openFileShelfDirectoryItemHelper		ends");
}


void openFileShelfDirectoryItem(GtkWidget *menuitem, gpointer data){
	TRACEIT(10,"openFileShelfDirectoryItem		start");
	GtkTreeSelection	*selection;
	struct FileShelfObjectList *Object = (struct FileShelfObjectList *)data;
	selection  = gtk_tree_view_get_selection(GTK_TREE_VIEW(Object->listView));
	gtk_tree_selection_selected_foreach(selection, openFileShelfDirectoryItemHelper, data);
	TRACEIT(10,"openFileShelfDirectoryItem		ends");
}


static void openFileShelfDirectoryHelper(GtkTreeModel *model,GtkTreePath *TreePath,GtkTreeIter *iter,gpointer data){
	TRACEIT(10,"openFileShelfDirectoryHelper		start");
	gchar *path=NULL;
	struct AllInformationAllUseStruct *info = (struct AllInformationAllUseStruct *)data ;
	GString ***gstrDPDirectories = (GString ***)info->Info_1;
	int *count = (int *)info->Info_2;
	int found = 0;
	GString *NewPath = g_string_sized_new(10);
	int i;

	gtk_tree_model_get_iter(model,iter,TreePath);
	gtk_tree_model_get(model, iter,COL_FLSHLF_PATH,&path, -1);
	if (path != NULL){
		for (i=0; i < *count; i++){
			if (strcmp(path,((*gstrDPDirectories)[i])->str)==0){
				found = 1;
			}
		}
		if (found == 0){
			g_string_assign(NewPath, path);
			if (0==AddNewGStringToDPointer( gstrDPDirectories, *count, NewPath)){
				(*count)++;
			}
		}
	}
	g_string_free(NewPath,TRUE);
	g_free(path);
	TRACEIT(10,"openFileShelfDirectoryHelper		ends");
}


void openFileShelfDirectory(GtkWidget *menuitem, gpointer data){
	TRACEIT(10,"openFileShelfDirectory		start");
	GtkTreeSelection	*selection;
	GString **gstrDPDirectories=NULL;
	int count = 0;
	int i;

	struct AllInformationAllUseStruct info;
	info.Info_1 = (void *)&gstrDPDirectories;
	info.Info_2 = (void *)(&count);

	struct FileShelfObjectList *Object = (struct FileShelfObjectList *)data;
	selection  = gtk_tree_view_get_selection(GTK_TREE_VIEW(Object->listView));
	gtk_tree_selection_selected_foreach(selection, openFileShelfDirectoryHelper, (void *)&info);

	for (i=0; i<count; i++){
		SideObjectList_CreateNewViewWithPath(NULL,gstrDPDirectories[i]->str,NULL);
		g_string_free(gstrDPDirectories[i],TRUE);
	}
	TRACEIT(10,"openFileShelfDirectory		ends");
}


void addItemsFromFileshelfToFileshelf_Helper(GtkTreeModel *model,GtkTreePath *TreePath,GtkTreeIter *iter,gpointer data){
	TRACEIT(11,"addItemsFromFileshelfToFileshelf_Helper		start");
	gchar *path=NULL;
	gchar *name=NULL;

	gtk_tree_model_get_iter(model,iter,TreePath);
	gtk_tree_model_get(model, iter,COL_FLSHLF_PATH,&path, COL_FLSHLF_NAME, &name, -1);
	if (path != NULL && name != NULL){
		AddItemToFileshelf (NULL, name, path, TRUE);
	}

	if (path)
		g_free(path);
	if (name)
		g_free(name);
	TRACEIT(11,"addItemsFromFileshelfToFileshelf_Helper		ends");
}


int FileShelfChooserDialog(int i_lastSelection, int i_withNewShelfOption, GtkNotebook *notebook, char *text){
    TRACEIT(10,"FileShelfChooserDialog		start");
	GtkWidget *dialog, *combobox, *label;
	int i_Pages=0;
	int i_PageToAdd=-1;
	int i;
	struct FileShelfObjectList *Object;
	const gchar *Name;

	dialog =  CreateMyDialog("add to fileshelf", win, 2, "Ok",GTK_RESPONSE_ACCEPT,"Cancel",GTK_RESPONSE_REJECT, FALSE);
	combobox = gtk_combo_box_new_text ();
	label = gtk_label_new(text);
	i_Pages = gtk_notebook_get_n_pages (notebook);
	if (i_withNewShelfOption == 1)
		gtk_combo_box_append_text ((GtkComboBox *)combobox, "New fileshelf");
	for (i = 0; i < i_Pages; i++){
		if (0 != FileShelfObjectList_GetObjectByPageNumber(&Object, (guint ) i)){
			continue;
		}
		Name = gtk_label_get_text ((GtkLabel *)(Object->label));
		gtk_combo_box_append_text ((GtkComboBox *) combobox, Name);
	}

    gtk_combo_box_set_active ((GtkComboBox *)combobox,(i_lastSelection > i_Pages)?0:i_lastSelection);

	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),label);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),combobox);
	gtk_widget_show_all(dialog);

	if (gtk_dialog_run(GTK_DIALOG(dialog))== GTK_RESPONSE_ACCEPT){
		i_PageToAdd = gtk_combo_box_get_active ((GtkComboBox *) combobox);
	}
	gtk_widget_destroy(dialog);

	return i_PageToAdd;

	TRACEIT(10,"FileShelfChooserDialog		end");
}

void addItemsFromFileshelfToFileshelf(GtkWidget *menuitem, gpointer data){
	TRACEIT(10,"addItemsFromFileshelfToFileshelf		start");
	static int  i_lastSelection=0;
	struct FileShelfObjectList *SourceObject = (struct FileShelfObjectList *)data;
	if (NULL == SourceObject){
		TRACEIT(7,"addItemsFromFileshelfToFileshelf		ends, SourceObject = NULL");
		return;
	}

	int i_SourcePageNum = gtk_notebook_get_current_page (SourceObject->notebook);

	int i_Pages=0;
	int i_PageToAdd;
	GtkTreeSelection	*selection;

	i_Pages = gtk_notebook_get_n_pages (SourceObject->notebook);
	i_PageToAdd = FileShelfChooserDialog(i_lastSelection, 1, SourceObject->notebook, "choose the fileshelf to which you want to\nadd the marked items.\n");

	if ( -1 == i_PageToAdd){
		TRACEIT(10,"addItemsFromFileshelfToFileshelf		ends, chanceled");
		return;
	}
	else if ( i_PageToAdd == 0){	// New fileshelf
		if ( 0 != NewFileShelf(NULL)){
               		TRACEIT(10,"addItemsFromFileshelfToFileshelf		ends, chanceled");
               		return;
		}
		i_lastSelection = i_Pages+1;
	}
	else{
		i_lastSelection = i_PageToAdd;
		i_PageToAdd -= 1;
		gtk_notebook_set_current_page (SourceObject->notebook, i_PageToAdd);
	}
	selection  = gtk_tree_view_get_selection(GTK_TREE_VIEW(SourceObject->listView));
	gtk_tree_selection_selected_foreach(selection, addItemsFromFileshelfToFileshelf_Helper,data);



	gtk_notebook_set_current_page  (SourceObject->notebook, i_SourcePageNum);
	TRACEIT(10,"addItemsFromFileshelfToFileshelf		ends");
}


gboolean oneFileShelfMouseButtonPressed(GtkWidget *treeview, GdkEventButton *event, gpointer data){
	TRACEIT(10,"oneFileShelfMouseButtonPressed		start");
	//data is listview

	 //dragndrop - related
    //I want to explain the following lines till //END-EXPLAINLINES:
    // This lines are here for the dragndrop thing. When marking more than one item and we want them to drag and drop we have to
    // catch this mousebutton event (this is the first if statement).
    // But we also have to unselect all the others when a simple mousebutten event on one item occurs. This is the second if statement.
    // We define an area in which we onyl select the item the mousepointer is over.
	GtkTreeSelection *selection=NULL;
    selection  = gtk_tree_view_get_selection((GtkTreeView *)treeview);
    guint modifiers;
	modifiers = gtk_accelerator_get_default_mod_mask ();
    static double x=0;
    static double y=0;
	if (gtk_tree_selection_count_selected_rows(selection)  > 1 ){   //onyl catch this if more than one item is selected
        if (event->type == GDK_BUTTON_PRESS  &&  event->button == 1){   //and it is the 1st mouse button
            if ((event->state & modifiers) == 0){                       //and no modifiers (such as ctrl or alt) are pressed.
                x=event->x;
                y=event->y;
                return TRUE;
            }
		}
	}
    if (gtk_tree_selection_count_selected_rows(selection)  > 1 ){
        if (event->type == GDK_BUTTON_RELEASE  &&  event->button == 1){
            if ((event->state & modifiers) == 0){
                if (x-event->x < 6.0 && y-event->y < 6.0){
                    gtk_tree_selection_unselect_all(selection);
                    x=0;
                    y=0;
                    if (event->window == gtk_tree_view_get_bin_window ((GtkTreeView *)treeview)){
                        GtkTreePath *path=NULL;
                        gtk_tree_view_get_path_at_pos ( (GtkTreeView *)treeview, (gint) event->x, (gint) event->y, &path, NULL, NULL, NULL);
                        gtk_tree_selection_select_path (selection,  path);
                        gtk_tree_path_free (path);
                    }
                    return TRUE;
                }
            }
		}
	}
    //END-EXPLAINLINES (dragndrop - related)





	if (event->type == GDK_BUTTON_PRESS  &&  event->button == 3){
		openFileshelfContextMenu(treeview,event,data);	//Look near the end of the file
		TRACEIT(10,"oneFileShelfMouseButtonPressed		ends");
		return TRUE;
	}if (event->type == GDK_BUTTON_PRESS  &&  event->button == 2){  //we mark all items of the view which have the same endings as the one allready marked
	     //we fire an mousebutton 1 event to the same position to get the item selected the mouse is over
	    GdkEventButton newEvent;
		newEvent.type = event->type;
		newEvent.window = event->window;
		newEvent.send_event = event->send_event;
		newEvent.time = event->time+1;
		newEvent.x = event->x;
		newEvent.y = event->y;
		newEvent.axes = event->axes;
		newEvent.state = event->state;
		newEvent.button = 1;
		newEvent.device = event->device;
		newEvent.x_root = event->x_root;
		newEvent.y_root = event->y_root;

		gboolean ret;

		g_signal_emit_by_name (treeview,"button-press-event",&newEvent,&ret);
		newEvent.type= GDK_BUTTON_RELEASE;
		g_signal_emit_by_name (treeview,"button-press-event",&newEvent,&ret);

		markItemsWithSameType((GtkTreeView *)data, COL_FLSHLF_FILETYPE);
		TRACEIT(10,"oneMouseButtonPressed		ends");
		return TRUE;
	}
	TRACEIT(10,"oneFileShelfMouseButtonPressed		ends");
	return FALSE;
}


gboolean oneFileShelfContextMenuReplied(GtkWidget *treeview, gpointer data){
	TRACEIT(10,"oneFileShelfContextMenuReplied		start");
	openFileshelfContextMenu(treeview,NULL, data);	//Look near the end of the file
	TRACEIT(10,"oneFileShelfContextMenuReplied		ends");
	return TRUE;
}

gboolean oneFileShelfNoteBookContextMenuReplied(GtkWidget *treeview, gpointer data){
	TRACEIT(10,"oneFileShelfContextMenuReplied		start");
	openFileShelfTabContextMenu(treeview,NULL, NULL);	//Look near the end of the file
	TRACEIT(10,"oneFileShelfContextMenuReplied		ends");
	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////    CONTEXTMENU       /////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


void openFileshelfContextMenu(GtkWidget *treeview, GdkEventButton *event, gpointer data){
	TRACEIT(10,"openFileshelfContextMenu		start");
	GtkWidget *menu, *menuitem, *menuFileOperations, *menuSideDeciderMove, *menuSideDeciderCopy;
	menu = gtk_menu_new();
	struct AllInformationAllUseStruct info;
    int NumOfSelectedRows=0;
	struct FileShelfObjectList *ActiveObject;
	FileShelfObjectList_GetActiveObject(&ActiveObject);
	if (ActiveObject == NULL){
		TRACEIT(10,"openFileshelfContextMenu		end, No active object");
		return;
	}

	menuFileOperations = gtk_menu_new();
	menuSideDeciderMove = gtk_menu_new();
	menuSideDeciderCopy = gtk_menu_new();

	info.source = SOURCE_FILESHELF;
	info.Info_1 = (void *)ActiveObject->listView;

	GtkTreeSelection *selection;
	GtkWidget *sep1, *sep2, *sep3, *sep4, *sep5;

	freeTypesAndUserProgrammArrays();

	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
	if (gtk_tree_selection_count_selected_rows(selection) <= 1 && event != NULL){   //Event can be NULL if the key for contextmenu is presed){
		GtkTreePath *path;
		TRACEIT(11,"openFileshelfContextMenu		mark the item coverd by mousepointer");
		if (gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(treeview), (gint) event->x, (gint) event->y, &path, NULL, NULL, NULL)){
			gtk_tree_selection_unselect_all(selection);
			gtk_tree_selection_select_path(selection, path);
			gtk_tree_path_free(path);
		}
	}
	NumOfSelectedRows = gtk_tree_selection_count_selected_rows(selection);

    TRACEIT(11,"openFileshelfContextMenu		create seperator");
	sep1 = gtk_separator_menu_item_new ();
	sep2 = gtk_separator_menu_item_new ();
	sep3 = gtk_separator_menu_item_new ();
	sep4 = gtk_separator_menu_item_new ();
	sep5 = gtk_separator_menu_item_new ();

	menuitem = gtk_menu_item_new_with_label("update panel");
	g_signal_connect(menuitem,"activate",G_CALLBACK(updateFileshelf),(void *)ActiveObject/*data*/);
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);

    TRACEIT(11,"openFileshelfContextMenu		fill the terminal structure");
	getTerminal();
	if (TheTerminalStruct.UserCommand != NULL && TheTerminalStruct.UserCommandName != NULL){
		//TheTerminalStruct.Object = (struct SideObjectList *)data;
		menuitem = gtk_menu_item_new_with_label(TheTerminalStruct.UserCommandName->str);
		g_signal_connect(menuitem,"activate",G_CALLBACK(startItemsWithUserCommand_WraperforFileShelf),&TheTerminalStruct);
		gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
	}
/*
    menuitem = gtk_menu_item_new_with_label("search for...");
    g_signal_connect(menuitem,"activate",G_CALLBACK(searchStringInDirectoryRecursive_FileShelf),ActiveObject->listView);
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
*/
    if (NumOfSelectedRows >= 1){
        menuitem = gtk_menu_item_new_with_label("search highlighted...");
        g_signal_connect(menuitem,"activate",G_CALLBACK(searchStringInAndHighlightItems_WraperforFileShelf),ActiveObject->listView);
        gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);

        menuitem = gtk_menu_item_new_with_label("size...");
        g_signal_connect(menuitem,"activate",G_CALLBACK(sizeOfSelectedItemsRecursive_FileShelf),ActiveObject);
        gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);

        menuitem = gtk_menu_item_new_with_label("edit...");
        g_signal_connect(menuitem,"activate",G_CALLBACK(editItemInformation_FileShelf),ActiveObject);
        gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);

    }



	gtk_menu_shell_append(GTK_MENU_SHELL(menu), sep1);

	//UserProgams/////////////////////////////////////////
	TRACEIT(11,"openFileshelfContextMenu		starting with user command section");


	//if (checkIfOneSelectionIsPathFileShelf(ActiveObject->listView) == 0 ){
	if (checkIfOneSelectionIsPath(ActiveObject->listView, COL_FLSHLF_TYPE_D ) == 0 ){

		getStandardEditor();
		if (TheStandardEditorStruct.UserCommand != NULL && TheStandardEditorStruct.UserCommandName != NULL){
			//TheStandardEditorStruct.Object = (struct SideObjectList *)data;
			menuitem = gtk_menu_item_new_with_label(TheStandardEditorStruct.UserCommandName->str);
			g_signal_connect(menuitem,"activate",G_CALLBACK(startItemsWithUserCommand_WraperforFileShelf),&TheStandardEditorStruct);
			gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
		}

		getTypesOfSelectedItems(&info);
		getUserProgramsForSelectedItems(NULL);
	}
	//if (checkIfOneSelectionIsPoints(data) == 0){	//it should not be possible to add . or .. to a FileShelf
	getUserProgramsForCountForAllItems(NULL);
	//}

	UserCommand_CreateUserMenu(menu);

	struct UserDataList *UserData = UserDataList_GiveMeFirstItem();
	while (UserData){
		UserData->Object = NULL;

		menuitem = gtk_menu_item_new_with_label(UserData->UserCommandName->str);
		g_signal_connect(menuitem,"activate",G_CALLBACK(startItemsWithUserCommand_WraperforFileShelf),UserData);
		gtk_menu_shell_append(GTK_MENU_SHELL(UserData->MenuToAddTo), menuitem);
		UserData = UserData->NextItem;
	};

	UserCommand_AttachUserMenu();
    TRACEIT(11,"openFileshelfContextMenu		end usercommand section");
	//EndUserPrograms/////////////////////////////////////

		gtk_menu_shell_append(GTK_MENU_SHELL(menu), sep2);

    if (NumOfSelectedRows >= 1){
        menuitem = gtk_menu_item_new_with_label("call all with...");
        g_signal_connect(menuitem,"activate",G_CALLBACK(callAllWith_WraperforFileShelf),data);
        gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);

        menuitem = gtk_menu_item_new_with_label("call each with...");
        g_signal_connect(menuitem,"activate",G_CALLBACK(callEachWith_WraperforFileShelf),data);
        gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
    }

		gtk_menu_shell_append(GTK_MENU_SHELL(menu), sep3);

    if (NumOfSelectedRows >= 1){
        menuitem = gtk_menu_item_new_with_label("open directory");
        g_signal_connect(menuitem,"activate", G_CALLBACK( openFileShelfDirectory),(void *)ActiveObject);
        gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
    }

	if (checkIfOneSelectionIsPath(ActiveObject->listView, COL_FLSHLF_TYPE_D ) != 0){
		menuitem = gtk_menu_item_new_with_label("open directory (item)");
		g_signal_connect(menuitem,"activate", G_CALLBACK( openFileShelfDirectoryItem),(void *)ActiveObject);
		gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
	}

	//FileOperations Submenu!!!!!!!!!

	menuitem = gtk_menu_item_new_with_label("<-left panel");
	g_signal_connect(menuitem,"activate", G_CALLBACK( copyFileShelfToLeftPannel),(void *)ActiveObject);
	gtk_menu_shell_append(GTK_MENU_SHELL(menuSideDeciderCopy), menuitem);

	menuitem = gtk_menu_item_new_with_label("right panel->");
	g_signal_connect(menuitem,"activate", G_CALLBACK( copyFileShelfToRightPannel),(void *)ActiveObject);
	gtk_menu_shell_append(GTK_MENU_SHELL(menuSideDeciderCopy), menuitem);

	menuitem = gtk_menu_item_new_with_label("<-left panel");
	g_signal_connect(menuitem,"activate", G_CALLBACK( moveFileShelfToLeftPannel),(void *)ActiveObject);
	gtk_menu_shell_append(GTK_MENU_SHELL(menuSideDeciderMove), menuitem);

	menuitem = gtk_menu_item_new_with_label("right panel->");
	g_signal_connect(menuitem,"activate", G_CALLBACK( moveFileShelfToRightPannel),(void *)ActiveObject);
	gtk_menu_shell_append(GTK_MENU_SHELL(menuSideDeciderMove), menuitem);

    if (NumOfSelectedRows >= 1){
        menuitem = gtk_menu_item_new_with_label("copy to");
        gtk_menu_item_set_submenu ((GtkMenuItem *)menuitem,menuSideDeciderCopy);
        gtk_menu_shell_append(GTK_MENU_SHELL(menuFileOperations), menuitem);

        menuitem = gtk_menu_item_new_with_label("move to");
        gtk_menu_item_set_submenu ((GtkMenuItem *)menuitem,menuSideDeciderMove);
        gtk_menu_shell_append(GTK_MENU_SHELL(menuFileOperations), menuitem);

        menuitem = gtk_menu_item_new_with_label("rename...");
        g_signal_connect(menuitem,"activate",G_CALLBACK(renameItems_FileShelf),data);
        gtk_menu_shell_append(GTK_MENU_SHELL(menuFileOperations), menuitem);
    }

		gtk_menu_shell_append(GTK_MENU_SHELL(menuFileOperations), sep5);

    if (NumOfSelectedRows >= 1){
        menuitem = gtk_menu_item_new_with_label("remove");
        g_signal_connect(menuitem,"activate",G_CALLBACK(mayDeleteSelectedFileShelfFiles),(void *)ActiveObject);
        gtk_menu_shell_append(GTK_MENU_SHELL(menuFileOperations), menuitem);

        menuitem = gtk_menu_item_new_with_label("fileoperations");
        gtk_menu_item_set_submenu ((GtkMenuItem *)menuitem,menuFileOperations);
        gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);

        menuitem = gtk_menu_item_new_with_label("add to fileshelf ...");
        g_signal_connect(menuitem,"activate",G_CALLBACK(addItemsFromFileshelfToFileshelf),(void *)ActiveObject);
        gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
    }

		gtk_menu_shell_append(GTK_MENU_SHELL(menu), sep4);

	menuitem = gtk_menu_item_new_with_label("remove missing files");
	g_signal_connect(menuitem, "activate", G_CALLBACK( RemoveMissingFiles), (void *)ActiveObject);
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);

    if (NumOfSelectedRows >= 1){
        menuitem = gtk_menu_item_new_with_label("remove from shelf");
        g_signal_connect(menuitem, "activate", G_CALLBACK( mayRemoveSelectedRows), (void *)ActiveObject);
        gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
    }

	gtk_widget_show_all(menu);
	// Note: event can be NULL here when called from view_onPopupMenu;
	// gdk_event_get_time() accepts a NULL argument
	gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, (event != NULL) ? event->button : 0, gdk_event_get_time((GdkEvent*)event));

	TRACEIT(10,"openFileshelfContextMenu		ends");
}


void openFileShelfTabContextMenu(GtkWidget *notebook, GdkEventButton *event, gpointer data){
	TRACEIT(10,"openFileShelfTabContextMenu		start");
	GtkWidget *menu, *menuitem;
	menu = gtk_menu_new();
	GtkWidget *sep1, *sep2, *sep3;

	sep1 = gtk_separator_menu_item_new ();
	sep2 = gtk_separator_menu_item_new ();
	sep3 = gtk_separator_menu_item_new ();

	menuitem = gtk_menu_item_new_with_label("new shelf...");
	g_signal_connect(menuitem,"activate",G_CALLBACK(TabMenuAddNewFileshelf),NULL);
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);

	menuitem = gtk_menu_item_new_with_label("rename shelf...");
	g_signal_connect(menuitem,"activate",G_CALLBACK(TabMenuRenameFileshelf),NULL);
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);

        gtk_menu_shell_append(GTK_MENU_SHELL(menu), sep2);

	menuitem = gtk_menu_item_new_with_label("cut items also on shelf...");
	g_signal_connect(menuitem,"activate",G_CALLBACK(TabMenuIntersectionItemsToNewShelf),NULL);
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
/*
	menuitem = gtk_menu_item_new_with_label("Symmentric diff");
	g_signal_connect(menuitem,"activate",G_CALLBACK(FileShelf_NewFileShelfFromSymmentricdiff),NULL);
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
*/
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), sep3);

	menuitem = gtk_menu_item_new_with_label("export shelf...");
	g_signal_connect(menuitem,"activate",G_CALLBACK(TabMenuExportFileshelf),NULL);
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);

	menuitem = gtk_menu_item_new_with_label("import shelf...");
	g_signal_connect(menuitem,"activate",G_CALLBACK(TabMenuImportFileshelf),NULL);
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);

		gtk_menu_shell_append(GTK_MENU_SHELL(menu), sep1);

	//if (1<gtk_notebook_get_n_pages(GTK_NOTEBOOK(notebook))){
	menuitem = gtk_menu_item_new_with_label("remove this shelf...");
	g_signal_connect(menuitem,"activate",G_CALLBACK(TabMenuRemoveActiveShelf),NULL);
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
	//}

	gtk_widget_show_all(menu);
	// Note: event can be NULL here when called from view_onPopupMenu;
	// gdk_event_get_time() accepts a NULL argument
	gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, (event != NULL) ? event->button : 0, gdk_event_get_time((GdkEvent*)event));

	TRACEIT(10,"openFileShelfTabContextMenu		end");
}

//////////////////////    CONTEXTMENU       /////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////

void TabMenuImportFileshelf(GtkWidget *menuitem, gpointer data){
	FileShefl_ImportFileShelf(NULL);
}

void TabMenuExportFileshelf(GtkWidget *menuitem, gpointer data){
	FileShefl_ExportActiveFileShelf();
}

void TabMenuRenameFileshelf(GtkWidget *menuitem, gpointer data){
	FileShefl_RenameActivePage();
}

void TabMenuRemoveActiveShelf(GtkWidget *menuitem, gpointer data){
	FileShelf_DeleteActiveTab();
}

void TabMenuAddNewFileshelf(GtkWidget *menuitem, gpointer data){
	NewFileShelf(NULL);
}

void TabMenuIntersectionItemsToNewShelf(GtkWidget *menuitem, gpointer data){
	FileShelf_NewFileShelfFromIntersection(NULL);
}
/*
void TabMenuCompIntersectionItemsToNewShelf(GtkWidget *menuitem, gpointer data){
	FileShelf_NewFileShelfFromSymmentricdiff(NULL);
}

*/




