/*
    Copyright 2007, 2008, 2009 Geyer Klaus

    This file is part of Cat'sEyE.

    Cat'sEyE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Cat'sEyE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Cat'sEyE.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdio.h>
#include <stdlib.h>	//for system();
#include <gtk/gtk.h>
#include <gio/gio.h>
#include <sys/types.h>  //Diese hier auch fr die Verzeichnisfunktionen und Attribute / and fork()
#include <sys/stat.h>	//Diese hier auch fr die Verzeichnisfunktionen und Attribute
#include <dirent.h>	//Verzeichnis Zeiger und DIR - Struktur
#include <string.h>
#include <time.h>
#include <unistd.h> 	//e.g. chdir, usleep
#include <errno.h>

#include "Lists.h"
#include "listitems.h"
#include "mytrace.h"
#include "helpers.h"
#include "globalDefinitions.h"
#include "myfm.h"


void CancelButtonSizeDialog(GtkButton *button, gpointer data);


extern GtkWindow 	*win;

extern GString *g_gstrCommandCopy;
extern GString *g_gstrCommandMove;
extern GString *g_gstrCommandRename;
extern GString *g_gstrCommandRemove;
extern GString *g_gstrCommandMakedir;


void callItemsToSystem(void *data){
	TRACEIT(10,"callItemsToSystem		start");

	if (data == NULL){
		TRACEIT(8,"callItemsToSystem		ends -1 ");
		g_thread_exit(0);
	}
	GString *command = g_string_sized_new(40);
	GString *homedir = g_string_sized_new(10);
	GString *QuitReplyText = NULL;
	GString *ActionNoteText = NULL;
	struct updateList *UpdateFolderWhenFinished_List=NULL;
	GString *gstrToken = g_string_sized_new(10);
	struct CallItemsToSystemStructure *stru = (struct CallItemsToSystemStructure *)data;
	struct CopyReMoveList *list = (struct CopyReMoveList *)stru->data;
	int i_exitstatus=0;
	int i_countCommand=0;
	GString *gstrErrorBuf=g_string_sized_new(200);   //used to store the error and, in case it is too big, shrink it before displaying
    gboolean bThreaded = stru->bThreaded;


	if (list){
		if (list->status == -10){
			g_string_free(command,TRUE);
			g_string_free(homedir,TRUE);
			g_string_free(gstrToken,TRUE);
            g_string_free(gstrErrorBuf,TRUE);
			TRACEIT(10,"callItemsToSystem		ends 2");
			if (bThreaded)
                g_thread_exit(0);
            else
                return;
		}
	}

	if (stru->usercommand==NULL || stru->FileList==NULL || strlen(stru->usercommand->str)==0){
		TRACEIT(8,"callItemsToSystem		ends - 2");
		if (list){
			list->status = 1;
		}
		g_string_free(command,TRUE);
		g_string_free(homedir,TRUE);
        g_string_free(gstrToken,TRUE);
        g_string_free(gstrErrorBuf,TRUE);
		if (bThreaded)
            g_thread_exit(0);
        else
            return;
	}

	if (stru->FileList->str[0] == '\0'){	//means, we have a new usercommand, in this case all information is in stru->usercommand
		//g_string_sprintf(command,"%s",stru->usercommand->str);
        g_string_assign(command,stru->usercommand->str);
	}
	else{	//old pair-format
		g_string_sprintf(command,"%s %s",stru->usercommand->str, stru->FileList->str);
	}

	if (list){
		list->status = 0;
	}


	//Copy structure info:
	if (stru->QuitReplyText){
		QuitReplyText = g_string_new(stru->QuitReplyText->str);
	}
	if (stru->ActionNoteText){
	    ActionNoteText = g_string_new(stru->ActionNoteText->str);
	}
	if (stru->UpdateFolderWhenFinished_List){
        UpdateFolderWhenFinished_List = stru->UpdateFolderWhenFinished_List;
	}



	//Calling command to system
	TRACEIT(10,"callItemsToSystem		before Systemscall");
	i_countCommand=0;

	gchar *errors=NULL;
	int int_error=0;
    //gchar *output=NULL;

    //create a little widget, displaying the command currently processing
    int iNotify=0;
    GtkWidget *scrollwidget;
    GtkWidget *ActionNoteWindow=NULL;
    GtkVBox *VBox;
    GtkWidget *label=NULL;
    //char cbuf[100];

    if (ActionNoteText){
        gdk_threads_enter();
        if (strlen (ActionNoteText->str) > 0){
            ActionNoteWindow = CreateMyWindow(ActionNoteText->str, NULL, 400,100, TRUE, FALSE, FALSE, &iNotify, FALSE);
        }
        else{
            ActionNoteWindow = CreateMyWindow("running process", NULL, 400,100, TRUE, FALSE, FALSE, &iNotify, FALSE);
        }
        AddMyIconsToWindow((GtkWindow *)ActionNoteWindow);
        label = gtk_label_new(NULL);
        gtk_label_set_selectable ( (GtkLabel *)label, TRUE);
        gtk_label_set_line_wrap ( (GtkLabel *)label, TRUE);

        VBox = g_object_new(GTK_TYPE_VBOX, "spacing", 5, "homogeneous", FALSE, NULL);

        scrollwidget = gtk_scrolled_window_new(NULL,NULL);
        gtk_scrolled_window_add_with_viewport ( (GtkScrolledWindow *)scrollwidget, label);
        gtk_container_add(GTK_CONTAINER(VBox),scrollwidget);

        //gtk_box_pack_start(GTK_BOX(VBox),label,FALSE,FALSE,1);
        gtk_container_add((GtkContainer *)ActionNoteWindow,(GtkWidget *)VBox);
        gtk_widget_show_all (ActionNoteWindow);
        gdk_threads_leave();
    }

    //COMMAND TO SYSTEM SHELL
	while (myTokenFunction (command, i_countCommand, "&&", gstrToken) == 1){
        //printf( "gstrToken : %s\n",gstrToken->str);
        if (ActionNoteText){
            TRACEIT(10,"callItemsToSystem		ActionNote is defined");
            gdk_threads_enter();
            if (0 != iNotify)
                gtk_label_set_text ((GtkLabel *)label, gstrToken->str);
            gdk_threads_leave();
        }
        TRACEIT(10,"callItemsToSystem		going to call command to the system");
        chdir(stru->WorkPath->str);    //this was some lines above before, but I think its better here when we come up with more than one thread.
		//if (!g_spawn_command_line_sync (gstrToken->str, NULL, &errors, &i_exitstatus, NULL)){


		//getcwd (cbuf,99);
		//printf("%s\n",cbuf);

		if (!myWrapOf__g_spawn_command_line_sync (gstrToken->str, NULL, &errors, &i_exitstatus, NULL)){
		    g_string_prepend(gstrToken, "\tCommand not found:\t\n\n");
		    g_string_append(gstrToken, "\n");
		    gdk_threads_enter();
            MyMessageWidget(win, "Invalid command", gstrToken->str);
            gdk_threads_leave();
            break;
		}
		TRACEIT(10,"callItemsToSystem		Checking for errors....");
        if (i_exitstatus != 0 && errors!=NULL && strlen(errors)>1){
            g_string_assign (gstrErrorBuf, errors);
            if (strlen(gstrErrorBuf->str) > MAXCHARSOFERROROFUSERCOMMAND){
                cleverCutStringToNChars(gstrErrorBuf, "\n..._SOME_TEXT_WAS_DELETED_HERE_...\n",MAXCHARSOFERROROFUSERCOMMAND);
            }
            if (list){  //write error to lists GString
                TRACEIT(10,"callItemsToSystem		Write error to lists GString");
                if (list->Errors == NULL)
                    list->Errors = g_string_new(gstrErrorBuf->str);
                int_error=1;
            }
            else{   //open a message widget to show the error
                TRACEIT(10,"callItemsToSystem		creating error widget");
                g_string_sprintf(homedir,"Command #%d returned with error.\nEvery following process is cancelled.\nError:\n%s",i_countCommand,gstrErrorBuf->str);
                gdk_threads_enter();
                MyMessageWidget(win, "Command quits with error",homedir->str);
                gdk_threads_leave();
                int_error=1;
            }
            TRACEIT(10,"callItemsToSystem		break after error");
            break;
        }

		if (errors){
            g_free(errors);
            errors = NULL;
		}
        i_countCommand++;
	};

	//destroy ActionNoteWindow
	if (ActionNoteWindow && 0 != iNotify){
	    TRACEIT(10,"callItemsToSystem		ActionNoteWindow is going to be destroyed");
	    gdk_threads_enter();
	    gtk_widget_destroy(ActionNoteWindow);
	    gdk_threads_leave();
    }

	TRACEIT(10,"callItemsToSystem		after Systemscall");

	if (list){
		list->status = 1;
	}

    TRACEIT(10,"callItemsToSystem		QuitReplayText?");
	if (QuitReplyText && int_error == 0){
		gdk_threads_enter();
		MyMessageWidget(win, "Process end", QuitReplyText->str);
		gdk_threads_leave();

	}
	TRACEIT(10,"callItemsToSystem		UpdateFolder?");
	struct updateList *UpdateFolderWhenFinished_BUFList=NULL;
	if (UpdateFolderWhenFinished_List){
        updateList_getFirstItem(&UpdateFolderWhenFinished_List);
        UpdateFolderWhenFinished_BUFList = UpdateFolderWhenFinished_List;
        while(UpdateFolderWhenFinished_BUFList){
//            gdk_threads_enter();
//            updateObjectsViewingFolder(UpdateFolderWhenFinished_BUFList->gstrName->str);
//            gdk_threads_leave();
            UpdateFolderWhenFinished_BUFList = UpdateFolderWhenFinished_BUFList->NextItem;
        }
        updateList_deleteList(&UpdateFolderWhenFinished_List);
        UpdateFolderWhenFinished_List=NULL;
	}

	getHomeDirectory(&homedir);
	chdir(homedir->str);
	//getcwd (cbuf,99);
    //printf("%s\n",cbuf);

    TRACEIT(10,"callItemsToSystem		free Mem Section");


    //FREE STRUCTURE
	if (stru->QuitReplyText){
		g_string_free(stru->QuitReplyText,TRUE);
		stru->QuitReplyText =  NULL;
	}
	if (stru->ActionNoteText){
	    g_string_free(stru->ActionNoteText, TRUE);
        stru->ActionNoteText = NULL;
	}
	if (stru->UpdateFolderWhenFinished_List){
        stru->UpdateFolderWhenFinished_List =  NULL;
	}
	g_string_free(stru->usercommand,TRUE);
	stru->usercommand = NULL;
	g_string_free(stru->FileList,TRUE);
	stru->FileList = NULL;
	g_string_free(stru->WorkPath,TRUE);
	stru->WorkPath = NULL;
	free (stru);

    //FREE LOCALS
    if (errors){
        g_free(errors);
        errors = NULL;
    }

    if (QuitReplyText)
        g_string_free(QuitReplyText, TRUE);
    if (ActionNoteText){
        g_string_free(ActionNoteText, TRUE);
    }
	g_string_free(command,TRUE);
	g_string_free(homedir,TRUE);
	g_string_free(gstrToken,TRUE);
	g_string_free(gstrErrorBuf,TRUE);

	TRACEIT(10,"callItemsToSystem		ends");
	//return;
	if (bThreaded)
        g_thread_exit(0);
    else
        return;
}


int makeDirectory(char *Directory, int iShowErrorMessage, GString *gstrErrorStorage){
    //if iShowErrorMessage == 1 then a message widget may pop up, showing some error, otherwise no message widget pops up
    //if gstrErrorStorage is != NULL the error message is written to it.
    //return: -1 internal error
    //return: 0 OK
    //return: 1 Error Message
    int ret=-1;
    if (Directory == NULL){
		TRACEIT(2,"makeDirectory		ends, Parameter NULL");
		return -1;
	}
    GError *Error = NULL;
    GFile *Sourcefile = g_file_new_for_path(Directory);
    GString *gstrText=g_string_sized_new(20);
    if (FALSE == g_file_make_directory (Sourcefile, NULL, &Error) ){
        if (gstrErrorStorage){
            g_string_append(gstrErrorStorage, Error->message);
        }
        MyMessageWidget(win, "Error while creating Directory", Error->message);
        g_string_printf(gstrText, "error created Directory: \"%s\2:%s",Directory, Error->message);
        textbuffer_commandHistory_addText(gstrText->str,"error");
        ret = 1;
    }
    else{
        g_string_printf(gstrText, "created Directory: %s",Directory);
        textbuffer_commandHistory_addText(gstrText->str,NULL);
        ret = 0;
    }

    if (Error)
        g_error_free(Error);
    if (Sourcefile)
        g_object_unref(Sourcefile);
    if (gstrText)
        g_string_free(gstrText, TRUE);
    Sourcefile = NULL;

    TRACEIT(10,"makeDirectory		ends");
	return ret;

/*

	TRACEIT(10,"makeDirectory		start");
	GString *command = g_string_sized_new(10);
	GString *gstrErrors=g_string_sized_new(10);
	gchar *errors=NULL;
    gchar *output=NULL;
    int ret=0;

	if (Directory == NULL){
	    g_string_free(command, TRUE);
	    g_string_free(gstrErrors, TRUE);
		TRACEIT(2,"makeDirectory		ends, Parameter NULL");
		return -1;
	}

	g_string_sprintf(command,"%s \"%s\"",g_gstrCommandMakedir->str, Directory);
	//system(command->str);
	//g_spawn_command_line_sync (command->str,&output,&errors,NULL,NULL);
	myWrapOf__g_spawn_command_line_sync(command->str,&output,&errors,NULL,NULL);

	if ( (errors != NULL && strlen(errors) > 1) || (output != NULL && strlen(output) > 1) ){
        if (errors != NULL && strlen(errors) > 1){
            g_string_append(gstrErrors, errors);
        }
        if (output != NULL && strlen(output) > 1){
            g_string_append(gstrErrors, output);
        }
        if (gstrErrorStorage){
            g_string_assign(gstrErrorStorage, gstrErrors->str);
        }
        if (iShowErrorMessage == 1){
            MyMessageWidget(win, "Error while creating Directory", gstrErrors->str);
        }
        ret = 1;
    }

	if (errors)
        g_free(errors);
    if (output)
        g_free(output);

	g_string_free(command,TRUE);
	g_string_free(gstrErrors, TRUE);
	TRACEIT(10,"makeDirectory		ends");
	return ret;
	*/
}


void  copyItemRecursive_Callback (goffset current_num_bytes, goffset total_num_bytes, gpointer data){
    TRACEIT(11,"copyItemRecursive_Callback       start");
    if (data == NULL){
        TRACEIT(2,"copyItemRecursive_Callback       end, userdate NULL");
        return;
    }
    struct CopyReMoveList *list = (struct CopyReMoveList *)data;
    //struct ListWidgetCommunicationStruct *commstru = (struct ListWidgetCommunicationStruct *)data;

    //static gint64 lastAmount=0;
    //static struct CopyReMoveList *list=NULL;
/*
    if (list != commstru->currentItem || lastAmount > current_num_bytes){
        list = commstru->currentItem;
        //printf("RESET\n");
        lastAmount = 0;
    }
    */
    //when finished we come up here twice, this leads to an percentage of 200%


    double percentage=0.0;
    //goffset my_current_num_bytes = current_num_bytes;
    if (current_num_bytes == total_num_bytes){
        if (list->lastfinishedTotalBytes != total_num_bytes){
            list->iBytesCopied += total_num_bytes;
            list->lastfinishedTotalBytes = total_num_bytes;
        }
        else{
            list->lastfinishedTotalBytes = -1;
        }
        current_num_bytes = 0;
    }

    //handle size (percentage) row
    GString *gstrBuf=g_string_sized_new(10);
    //
    //commstru->currentItem->iBytesCopied += current_num_bytes-lastAmount;
    //lastAmount = current_num_bytes;
    if (list->iBytesToCopyIsValid == 1){
        percentage = 100*(double)(list->iBytesCopied+current_num_bytes)/(double)list->iBytesToCopy;
        g_string_printf(gstrBuf, "%0.2f", percentage);
        //printf("%0.2f\n",percentage);
        //printf("%lld\n",commstru->currentItem->iBytesCopied);
    }
    else{
        g_string_assign(gstrBuf, "na");
    }

    gdk_threads_enter();
    gtk_list_store_set (list->listStore, &(list->iter), 1, gstrBuf->str,-1);
    gdk_threads_leave();


    g_string_free(gstrBuf, TRUE);
    TRACEIT(11,"copyItemRecursive_Callback       end");
    return;
}



int copyReMoveItemRecursive(const char *cpSource, const char *cpDest, struct ListWidgetCommunicationStruct *commstru){
    //return: -1:error, 1=success, 0=copy returned false or any other error, copying failed
    //cpDest is supposed to be a path
    //while cpSource may be a path or filename
    //cpDest may be NULL (for deleting)
    TRACEIT(10,"copyReMoveItemRecursive		start");
    if ( commstru==NULL || cpSource==NULL || (cpDest==NULL && commstru->currentItem->CopyMoveOrRemove != 3) ){
        TRACEIT(2,"copyReMoveItemRecursive		end, Parameter NULL");
        return -1;
    }
    gboolean iDoWeDelete = (commstru->currentItem->CopyMoveOrRemove == 3)?TRUE:FALSE;
    struct CopyReMoveList *currentItem = commstru->currentItem;

    GString *gstrSource=g_string_sized_new(20);
    if (!iDoWeDelete)
        g_string_assign(gstrSource,cpDest);

    //GString *gstrDest=g_string_new(cpSource);
    GString *gstrDest=g_string_sized_new(20);
    g_string_assign(gstrDest, cpSource);
    GString *gstrNames=g_string_sized_new(10);
    gboolean bRet=FALSE;
    GFile *Sourcefile = NULL;
    GFile *Destfile = NULL;

    DIR *Directory;
    struct dirent *dirzeiger;
    int DirectoryOpened=0;

    int iRetRecursivItem=0;
    int iRetFolder=0;
    int iRetErrorCode=0;    //0=error, 1=0k, -1 internal error, thsi is the value returned by this function
    int iRecItemCount=0;
    int iOverwriteRet=0;
    gboolean bCanceled = FALSE;

    bCanceled =  g_cancellable_is_cancelled (commstru->cancellable);

    //build the complete Name of the destination:
    //get the name of the file:
    if (bCanceled == FALSE && currentItem->status==0){

        if (!iDoWeDelete){
            getLastStringInPath(gstrDest);
            checkPathForLastChar(gstrSource);   //gstrSource is filled with cpDest
            g_string_prepend(gstrDest,gstrSource->str); //now we have the complete destination
        }
        g_string_assign(gstrSource, cpSource);  //now the source is valid too.
        //printf("comming in: Source=%s\n",gstrSource->str);

        //check existance of sourceitem
        if (!iDoWeDelete && TRUE == g_file_test( gstrDest->str, G_FILE_TEST_EXISTS)){  //Ask for permission to overwirte existing file
            if (commstru->bOverWriteAllExisting==FALSE && currentItem->bOverWriteExisting==FALSE){
                gdk_threads_enter();
                iOverwriteRet = MyEasyInputWidget_Two ((GtkWindow *)commstru->mainListWidget, "overwrite", "There is allready an item with the\nsame name in the destination directory.\nDo you want t ooverwrite it?\n", NULL, NULL, NULL, "Overwrite", "Overwrite all", "No to all", "No");
                gdk_threads_leave();
				if (iOverwriteRet == 0){  //Overwrite this item
				    //printf("overwrite item\n");
					currentItem->bOverWriteExisting = TRUE;
				}
				else if (iOverwriteRet == 1){ //Overwirte all items
				    //printf("overwrite ALL item\n");
                    currentItem->bOverWriteExisting = TRUE;
                    commstru->bOverWriteAllExisting = TRUE;
				}
				else if (iOverwriteRet == 2){ //No to all, this means we may cancel the whole list.
				    //printf("NO to ALL item\n");
                    commstru->CancelListFunction(commstru);
				}
				else if (iOverwriteRet == 3){ //No to this item, we cancel this item
				    //printf("No to this item\n");
                    currentItem->status=-10;    //mark it as cancelled
				}
            }
        }
        if (currentItem->status == 0){     //item in process, this means this function
            //printf("source = %s\n",gstrSource->str);
            //do not follow links, event if they are pointing to a directory
            if (TRUE == checkStringIsPath( gstrSource->str ) && FALSE == g_file_test( gstrSource->str, G_FILE_TEST_IS_SYMLINK)){ //open directory and give one item after each other recursive to this function
                //printf("Identified as path\n");
                //printf("found dir:%s\n",gstrSource->str);
                if (!iDoWeDelete){
                    if (TRUE == g_file_test( gstrDest->str, G_FILE_TEST_EXISTS)){
                        if (TRUE == checkStringIsPath( gstrDest->str )){ //check if the directory allready exists
                            iRetFolder = 0;    //folder exists, we do not need to build it.
                        }
                        else{
                            iRetFolder = -1; //it seems there is all ready a file with this name, prefents us creating the directory
                        }
                    }
                    else{   //we buld the directory
                        //printf("create Dir: %s\n",gstrDest->str);
                        //gdk_threads_enter();
                        //iRetFolder = makeDirectory(gstrDest->str,0,gstrNames); //create the other directory, stores error message in gstrNames
                        //return 0=ok, 1=error , -1=internal error
                        //gdk_threads_leave();
                        Sourcefile = g_file_new_for_path(gstrDest->str);
                        if (FALSE == g_file_make_directory (Sourcefile, commstru->cancellable, &(commstru->gerror)) ){
                            g_string_append(currentItem->ErrorMessage, commstru->gerror->message);
                            iRetFolder = -1;
                        }
                        else{
                            iRetFolder = 0;
                        }
                        if (Sourcefile)
                            g_object_unref(Sourcefile);
                        Sourcefile = NULL;
                    }
                }
                else{
                    iRetFolder = 0;
                }
                if (0 == iRetFolder){  //directory exists lets get in there
                    iRetErrorCode=1;    //so far so good
                    //printf("created Drectory:%s\n",gstrDest->str);
                    //printf("going to open Dir:%s\n", gstrSource->str);
                    if (0 == openDirectory(&Directory, &DirectoryOpened, gstrSource)){  //read directories entries and process them
                        iRetErrorCode=1;    //so far so good
                        getNextListItem(Directory, &dirzeiger, DirectoryOpened, gstrNames);
                        //printf("directory opened\n");
                        iRecItemCount = 0;
                        iRetRecursivItem = 0;
                        while ( strlen(gstrNames->str) != 0 ){
                            if (strcmp(gstrNames->str,".")!=0 && strcmp(gstrNames->str,"..")!=0){
                                g_string_assign(gstrSource, cpSource);
                                checkPathForLastChar(gstrSource);
                                g_string_append(gstrSource, gstrNames->str);

                                //printf("new source is:%s\nnew dest is:%s\n",gstrSource->str, gstrDest->str);
                                if (currentItem->CopyMoveOrRemove == 1){    //copy
                                    iRetRecursivItem += copyReMoveItemRecursive(gstrSource->str, gstrDest->str, commstru);
                                }
                                else if (currentItem->CopyMoveOrRemove == 2){    //move
                                    //printf("moving %s\n",gstrSource->str);
                                    iRetRecursivItem += copyReMoveItemRecursive(gstrSource->str, gstrDest->str, commstru);
                                }
                                else if (currentItem->CopyMoveOrRemove == 3){    //remove
                                    //printf("deleting %s\n",gstrSource->str);
                                    iRetRecursivItem += copyReMoveItemRecursive(gstrSource->str, NULL, commstru);
                                }
                                iRecItemCount++;
                            }

                            getNextListItem(Directory, &dirzeiger, DirectoryOpened, gstrNames);
                        }
                        closeDirectory(&Directory);
                        if (iRetRecursivItem != iRecItemCount){ //error on one item in the current Directory
                            iRetErrorCode=0;
                        }
                        if (currentItem->CopyMoveOrRemove == 3 || currentItem->CopyMoveOrRemove == 2){  //delete directory when ReMoved
                            if (iRetRecursivItem == iRecItemCount){  //Dont know this makes sence
                                Sourcefile = g_file_new_for_path(cpSource);
                                //printf("try to delete directory %s\n",cpSource);
                                if (commstru->gerror){
                                    g_error_free(commstru->gerror);
                                    commstru->gerror = NULL;
                                }
                                if (FALSE == g_file_delete (Sourcefile, commstru->cancellable, &(commstru->gerror))){
                                    //error deleting source directory
                                    g_string_assign ( currentItem->ErrorMessage, "Error: Failed to delete source directory code 1" );
                                    if (commstru->gerror){
                                        g_string_append(currentItem->ErrorMessage, " Message: ");
                                        g_string_append(currentItem->ErrorMessage, commstru->gerror->message);
                                    }
                                    //printf("error deleting directory 1\n");
                                    //printf("Message: %s\n",commstru->gerror->message);
                                    iRetErrorCode=0;
                                }
                                else{
                                    iRetErrorCode=1;
                                }
                                //printf("going to delete directory:%s\n",cpSource);
                                if (Sourcefile){
                                    g_object_unref(Sourcefile);
                                    Sourcefile=NULL;
                                }
                            }
                            else{   //error deleting source directory, not empty
                                g_string_assign ( currentItem->ErrorMessage, "Error: Failed to delete source directory code 2" );
                                //printf("error deleting directory 2\n");
                                iRetErrorCode=0;
                            }
                        }
                    }
                    else{
                        g_string_assign ( currentItem->ErrorMessage, "Error: Failed to open source directory" );
                        //printf("error opening source directory\n");
                        iRetErrorCode=0;
                        //failed to open source directory
                    }
                }
                else if (iRetFolder == 1){
                    //g_string_assign (currentItem->ErrorMessage, gstrNames->str);
                    iRetErrorCode=0;
                }
                else{
                    g_string_assign (currentItem->ErrorMessage, "Error: Failed to create destination directory");
                    //printf("error create destination directory\n");
                    //failed to create destination directory
                }
            }
            else{   //copy a file
                //printf("Identified as file\n");
                //prepare listitems member
                Sourcefile = g_file_new_for_path(gstrSource->str);
                if (!iDoWeDelete)
                    Destfile = g_file_new_for_path(gstrDest->str);
                if (commstru->gerror != NULL){
                    g_error_free (commstru->gerror);
                    commstru->gerror = NULL;
                    TRACEIT(5,"copyReMoveItemRecursive		ErrorStruct was not NULL, checkpoint 1");
                }
                if (currentItem->CopyMoveOrRemove == 1){    //copy
                    bRet = g_file_copy (  Sourcefile, Destfile, G_FILE_COPY_OVERWRITE | G_FILE_COPY_NOFOLLOW_SYMLINKS, commstru->cancellable, copyItemRecursive_Callback, (void *)currentItem, &(commstru->gerror));
                }
                else if (currentItem->CopyMoveOrRemove == 2){    //move
                    bRet = g_file_move (  Sourcefile, Destfile, G_FILE_COPY_OVERWRITE | G_FILE_COPY_NOFOLLOW_SYMLINKS, commstru->cancellable, copyItemRecursive_Callback, (void *)currentItem, &(commstru->gerror));
                }
                else if (currentItem->CopyMoveOrRemove == 3){    //remove
                    bRet = g_file_delete (Sourcefile, commstru->cancellable, &(commstru->gerror));
                }

                //sprintf("copied: _%s_ TO _%s_\n",gstrSource->str, gstrDest->str);
                if (bRet)
                    iRetErrorCode = 1;
                else
                    iRetErrorCode = 0;
            }
        }
        else{   //list->status!=-0
            iRetErrorCode = 0;
            g_string_assign (currentItem->ErrorMessage, "Cancelled for overwriting");
        }
    }

    if (Sourcefile)
        g_object_unref(Sourcefile);
    if (Destfile)
        g_object_unref(Destfile);

    g_string_free(gstrSource, TRUE);
    g_string_free(gstrDest, TRUE);
    g_string_free(gstrNames, TRUE);

    TRACEIT(10,"copyReMoveItemRecursive		end");
    return iRetErrorCode;
}



void CalcSizeForCopyDialog(struct CopyReMoveList *list){
    TRACEIT(10,"CalcSizeForCopyDialog		start");
    if (list == NULL ){
        TRACEIT(2,"CalcSizeForCopyDialog		end, Parameter NULL");
        g_thread_exit(0);
    }
    if (list->string1 == NULL){
        TRACEIT(2,"CalcSizeForCopyDialog		end, Parameter2 NULL");
        g_thread_exit(0);
    }
    list->iBytesToCopyIsValid=0;
    if (0==getDirectorySize(&(list->iBytesToCopy), list->string1, &(list->ibreak) )){
        list->iBytesToCopyIsValid=1;
    }
    TRACEIT(10,"CalcSizeForCopyDialog		end");
    g_thread_exit(0);
}


/*
void CalcSizeAfterCopyMove(struct CopyReMoveList *list){
    TRACEIT(10,"CalcSizeAfterCopyMove		start");
    if (list == NULL){
        TRACEIT(2,"CalcSizeAfterCopyMove		end, Parameter NULL");
        g_thread_exit(0);
    }
    GString *gstrSource = g_string_new(list->string2->str);
    GString *gstrDest = g_string_new(list->string1->str);
    GString *gstrTarget = g_string_sized_new(10);

    getLastStringInPath(gstrDest);
    checkPathForLastChar(gstrSource);   //gstrSource is filled with cpDest
    g_string_prepend(gstrDest,gstrSource->str); //now we have the complete destination
    g_string_assign(gstrSource, list->string1->str);



    guint64 iBytesCopied=0;
    guint64 iBytesToCopy=0;
    if (list->ibreak == 0){
        if (0==getDirectorySize(&iBytesCopied, gstrDest, &(list->ibreak))){
            while (list->ibreak == 0 && list->iBytesToCopyIsValid==0){
                //printf("test\n");
                usleep(10000); // this can not block us, as the cancel button sets ibreak to 1
            }
            if (list->iBytesToCopyIsValid==1){
                iBytesToCopy = list->iBytesToCopy;
                g_string_printf(gstrTarget, "%0.2f",100*(double)iBytesCopied/(double)iBytesToCopy);

                gdk_threads_enter();
                gtk_list_store_set (list->listStore, &(list->iter), 1, gstrTarget->str,-1);
                gdk_threads_leave();
            }
        }
    }

    g_string_free(gstrTarget, TRUE);
    g_string_free(gstrSource, TRUE);
    g_string_free(gstrDest, TRUE);

    TRACEIT(10,"CalcSizeAfterCopyMove		end");
    g_thread_exit(0);
}
*/


void CopyReMoveItemsViaList(void *data){
    TRACEIT(10,"CopyReMoveItemsViaList		start");
	if (data == NULL){
		TRACEIT(2,"CopyReMoveItemsViaList		ends, parameter NULL" );
		g_thread_exit(0);
	}
	struct ListWidgetCommunicationStruct *commStru=(struct ListWidgetCommunicationStruct *)data;
	struct CopyReMoveList *list = NULL;
	GString *gstrError = g_string_sized_new(20);
	GString *gstrForTextBuffer = g_string_sized_new(20);
    GtkTreePath *pTreePath;
    GtkTreeSelection *selection;
    int iRet=0;
    gdk_threads_enter();
    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(commStru->listView));
    gdk_threads_leave();
    int iSomeError=0;
    GThread *threadCalcSource=NULL;

    commStru->SecondThreadRunning = 1;

    while (commStru->currentItem && commStru->iCancelled==0){
        list = commStru->currentItem;   //for faster writing. and shorter sentences
        if (list->string1==NULL || ( (list->CopyMoveOrRemove == 1 || list->CopyMoveOrRemove == 2) && list->string2==NULL) ){
            list->ibreak=1;
            commStru->currentItem = commStru->currentItem->NextItem;
            continue;
        }
        list->ibreak=0;
        gdk_threads_enter();
        gtk_tree_selection_select_iter(selection, &(list->iter));

        //make the scrolling follow the marked Item
        pTreePath = gtk_tree_model_get_path (gtk_tree_view_get_model ((GtkTreeView *)commStru->listView),&(list->iter));
        gtk_tree_view_scroll_to_cell ((GtkTreeView *)commStru->listView,pTreePath,NULL,TRUE,0.0,0.0);
        gtk_tree_path_free(pTreePath);
        gdk_threads_leave();

        if (list->status == -10 || commStru->iCancelled==1){
            list->ibreak=1;
            break;
        }
        if (list->ErrorMessage)
            g_string_free(list->ErrorMessage,TRUE);
        list->ErrorMessage = g_string_sized_new(10);    //make clear this pointer is valid  //TODO FREE IT AND FILL IT WITH 0 on list creation

        if (commStru->gerror){
            g_error_free(commStru->gerror);
            commStru->gerror=NULL;
            TRACEIT(10,"copyItemRecursive		error struct was not NULL, checkpoint 2");
        }

        //TODO check status before set
        if (list->status == -1){
            list->status = 0;
            //calc size
            list->iBytesToCopy = 0;
            list->iBytesCopied = 0;
            list->iBytesToCopyIsValid = 0;
            if (list->CopyMoveOrRemove != 3){    //do not calculate this when deleting
                threadCalcSource = g_thread_create((GThreadFunc)CalcSizeForCopyDialog, list , TRUE, NULL);
            }
            //tell the textBuffer what we are giong to do:
            if (list->CopyMoveOrRemove == 1){
                g_string_printf(gstrForTextBuffer, "copying \"%s\" TO  \"%s\"",list->string1->str,list->string2->str);
            }
            else if (list->CopyMoveOrRemove == 2){
                if (threadCalcSource){  //we should wait for the calcthread before moving.
                     g_thread_join (threadCalcSource);
                }
                g_string_printf(gstrForTextBuffer, "moving \"%s\" TO  \"%s\"",list->string1->str,list->string2->str);
            }
            else if (list->CopyMoveOrRemove == 3){
                g_string_printf(gstrForTextBuffer, "deleting \"%s\"",list->string1->str);
                // Set the row for percentage to some text, its better to view something than nothing, because nothing means not processed yet
                gdk_threads_enter();
                gtk_list_store_set (commStru->listStore, &(list->iter), 1, "deleting",-1);
                gdk_threads_leave();
            }
            gdk_threads_enter();
            textbuffer_commandHistory_addText(gstrForTextBuffer->str, NULL);
            gdk_threads_leave();
            //start copying, this call is synchron
            iRet = copyReMoveItemRecursive(list->string1->str, (list->string2)?list->string2->str:NULL, commStru);

            if (iRet==1){   //iRet == 1 : Ok
                list->ibreak=0;
                //if (list->CopyMoveOrRemove != 3){
                    //list->threadCalcSizeAfterProcessing = g_thread_create((GThreadFunc)CalcSizeAfterCopyMove, list , TRUE, NULL);
                //}
            }
            else{
                list->ibreak=1;
            }

        }

        if (iRet == 0 ){    //Error copying look GError for information
            g_string_erase(gstrError,0,-1);
            if (commStru->gerror){
                if (G_IO_ERROR_CANCELLED != commStru->gerror->code){
                    g_string_printf(gstrError, "Error: code: %d, message:%s",commStru->gerror->code, commStru->gerror->message);
                    textbuffer_commandHistory_addText(gstrError->str, "error");
                    g_error_free(commStru->gerror);
                    commStru->gerror=NULL;
                    iSomeError = 1;
                }
                else
                    iSomeError = 2; //do not show the hint that there was an error
            }
            else if (strlen (list->ErrorMessage->str) != 0){
                g_string_assign (gstrError, list->ErrorMessage->str);
                textbuffer_commandHistory_addText(gstrError->str, "error");
                iSomeError = 1;
            }
            else{
                g_string_assign(gstrError, "Unknown error");
                textbuffer_commandHistory_addText(gstrError->str, "error");
                iSomeError = 1;
            }
            if (strlen(gstrError->str)>0){
                gdk_threads_enter();
                gtk_list_store_set (commStru->listStore, &(list->iter), 3, gstrError->str,-1);
                gdk_threads_leave();
                iSomeError=1;
            }
        }
        else if(iRet == -1){    //internal Error
            g_string_assign(gstrError, "BAD INTERNAL ERROR, INFORM THE PROGRAMMER");
            textbuffer_commandHistory_addText(gstrError->str, "error");
            gdk_threads_enter();
            gtk_list_store_set (commStru->listStore, &(list->iter), 3, gstrError->str,-1);
            gdk_threads_leave();
            iSomeError=1;
        }
        if (commStru->iCancelled==1){
            list->ibreak=1;
            break;
        }
        commStru->currentItem = commStru->currentItem->NextItem;
    }
    //there might be a CalcSizeAfterCopyMove still running, wait for them:
    //list = commStru->firstItem;
    //while (list){
    //    if (list->threadCalcSizeAfterProcessing){
    //        g_thread_join (list->threadCalcSizeAfterProcessing);
    //    }
    //    list = list->NextItem;
    //}


    //list->ibreak=1; //for security reason, this cancels the size calculation thread
    //we can come up here when there was
    //some error, in this case we may have a cancel
    //just a cancel
    //we finished, no error, no cancel
    //only in the last case we going to close the widget from here


    //printf("after thread 2 loop\n");
    if (commStru->iCancelled==1 || iSomeError!=0){ //error or cancel, its not our turn to close the widget, but to update some things

        gdk_threads_enter();
        if (iSomeError == 1){
            MyMessageWidget((GtkWindow *)commStru->mainListWidget, "ERROR", "There was at least one Error \nwhile processing the list.\nLook at the column errors and scroll up\ntill you find the error message.");
            g_string_append(commStru->WindowName," ERROR");
        }
        else if (commStru->iCancelled == 1){
            g_string_append(commStru->WindowName," Cancelled");
        }
        gtk_window_set_title((GtkWindow *)commStru->mainListWidget,commStru->WindowName->str);

        gtk_widget_show(GTK_WIDGET(commStru->closeButton));
        gtk_widget_hide(GTK_WIDGET(commStru->cancelButton));

        gtk_widget_queue_draw ((GtkWidget *)commStru->mainListWidget);
        gdk_threads_leave();

        commStru->SecondThreadRunning = 0;
	}
	else if (commStru->iCancelDialogRuns==0){
	    gdk_threads_enter();
	    commStru->SecondThreadRunning = 0;
	    (commStru->FreeMemoryAndDeleteWidgetFunction)( commStru);  //here Notify MUST BE 2 other wise this call will lead to a deadlock becuase of SecondThreadRunning
	    gdk_threads_leave();
	}
	else{ //the cancel dialog is open
        commStru->SecondThreadRunning = 0;
        commStru->iDestroyFlag = 1;
	}

    //free things
    g_string_free(gstrError, TRUE);
    g_string_free(gstrForTextBuffer, TRUE);
    //commStru->SecondThreadRunning = 0; NOT HERE, because of the last else if

	TRACEIT(10,"CopyReMoveItemsViaList		ends");
	g_thread_exit(0);
}
/*
void deleteFileViaList(void *data){
	TRACEIT(10,"deleteFile		start");
	if (data == NULL){
		TRACEIT(8,"copyItems		ends -1" );
		g_thread_exit(0);
		//return;
	}
	struct CopyReMoveList *list = (struct CopyReMoveList *)data;
	if (list->string1==NULL ){
		TRACEIT(10,"deleteFile		ends -2");
		g_thread_exit(0);
		//return;
	}
	if (list->status == -10){
		//g_string_free(command,TRUE);
		TRACEIT(10,"deleteFile		ends 2");
		g_thread_exit(0);
		//return;
	}
	//return remove(PathNName);
	list->status = 0;

	GString *command = g_string_new("rm -rf \"");

	g_string_append(command,list->string1->str);
	g_string_append(command,"\"");
	//system(command->str);
	g_spawn_command_line_sync (command->str,NULL,NULL,NULL,NULL);
	//printf("%s\n",command->str);
	list->status = 1;
	g_string_free(command,TRUE);
	TRACEIT(10,"deleteFile		ends");

	g_thread_exit(0);
}


void copyItemsViaList(void *data){
	TRACEIT(10,"copyItems		start");
	if (data == NULL){
		TRACEIT(8,"copyItems		ends -1" );
		g_thread_exit(0);
	}
	struct CopyReMoveList *list = (struct CopyReMoveList *)data;
	GString *command = g_string_new("cp -rf ");

	if (list->string1==NULL || list->string2==NULL){
		TRACEIT(8,"copyItems		ends 1");
		g_string_free(command,TRUE);
		g_thread_exit(0);
	}
	if (list->status == -10){
		g_string_free(command,TRUE);
		TRACEIT(10,"copyItems		ends 2");
		g_thread_exit(0);
	}
	list->status = 0;
	g_string_sprintf(command, "cp -rf \"%s\" \"%s\"",list->string1->str,list->string2->str);

	//system(command->str);
	g_spawn_command_line_sync (command->str,NULL,NULL,NULL,NULL);
//	printf("command:%s\n",command->str);

	list->status = 1;
	g_string_free(command,TRUE);
	TRACEIT(10,"copyItems		ends");
	g_thread_exit(0);
}


void moveItemsViaList(void *data){ //only for moving!!
	TRACEIT(10,"moveItems		start");
	if (data == NULL){
		TRACEIT(8,"moveItems		ends -1" );
		g_thread_exit(0);
	}
	struct CopyReMoveList *list = (struct CopyReMoveList *)data;
	GString *command = g_string_new("mv -f ");
	if (list->string1==NULL || list->string2==NULL){
		g_string_free(command,TRUE);
		TRACEIT(10,"moveItems		ends 1");
		g_thread_exit(0);
	}
	if (list->status == -10){
		g_string_free(command,TRUE);
		TRACEIT(10,"moveItems		ends 2");
		g_thread_exit(0);
	}
	list->status = 0;
	g_string_sprintf(command, "mv -f \"%s\" \"%s\"",list->string1->str,list->string2->str);

	//system(command->str);
	g_spawn_command_line_sync (command->str,NULL,NULL,NULL,NULL);

	list->status = 1;
	g_string_free(command,TRUE);
	TRACEIT(10,"moveItems		ends");
	g_thread_exit(0);
}
*/

int moveItems(char *Source, char* DestinationPath){	//only for renaming!!!!
    //returns   -1 on internal error
    //          0 renaming successful
    //          1 renaming failed
	TRACEIT(10,"moveItems		start");
	GString *command = g_string_sized_new(10);
    GString *gstrErrors=g_string_sized_new(10);
    gchar *errors=NULL;
    gchar *output=NULL;
	if (Source==NULL || DestinationPath==NULL){
        g_string_free(command, TRUE);
        g_string_free(gstrErrors, TRUE);
		TRACEIT(10,"moveItems		ends");
		return -1;
	}
	int ret=0;
	//g_string_sprintf(command, "mv -f \"%s\" \"%s\"",Source,DestinationPath);
	g_string_sprintf(command, "%s \"%s\" \"%s\"",g_gstrCommandRename->str,Source,DestinationPath);

	//system(command->str);
	//g_spawn_command_line_sync (command->str,NULL,NULL,NULL,NULL);

	//g_spawn_command_line_sync (command->str,&output,&errors,NULL,NULL);
	myWrapOf__g_spawn_command_line_sync (command->str,&output,&errors,NULL,NULL);

    if ( (errors != NULL && strlen(errors) > 1) || (output != NULL && strlen(output) > 1) ){
        if (errors != NULL && strlen(errors) > 1){
            g_string_append(gstrErrors, errors);
        }
        if (output != NULL && strlen(output) > 1){
            g_string_append(gstrErrors, output);
        }
        MyMessageWidget(win, "Error while renaming", gstrErrors->str);
        ret=1;
    }

	if (errors)
        g_free(errors);
    if (output)
        g_free(output);

    g_string_free(gstrErrors, TRUE);
	g_string_free(command,TRUE);
	TRACEIT(10,"moveItems		ends");
	return ret;
}


int fileAttributesGet(struct dirent *dirzeiger, int DirectoryOpened, struct FileAttributs *attribs,char *path, char *name){
    //DO NOT FORGETT: FREE MEMORY (GSTRING IF IT IS A LINK)
    //example:
    //      if (attribs->gstrLinkTarget)
    //          g_string_free(attribs->gstrLinkTarget, TRUE);
    //
	TRACEIT(11,"fileAttributesGet		start");
	int i;
	if (attribs == NULL || path == NULL || name==NULL){
		TRACEIT(10,"fileAttributesGet		ends");
		return -1;
	}

//	if (DirectoryOpened==0){	//look comment for dirzeiger, why we uncomment this
//		TRACEIT(10,"fileAttributesGet		ends");
//		return;
//	}
	GString *gstrFullName = g_string_new(path);
	struct stat fileattributes;

    //size
    attribs->size = 0;
    //owner
    attribs->owner = 0;
    //group
    attribs->group = 0;
    //permission
    sprintf(attribs->permission,"-");
    strcat(attribs->permission,"-");
    strcat(attribs->permission,"-");
    strcat(attribs->permission,"-");
    strcat(attribs->permission,"-");
    strcat(attribs->permission,"-");
    strcat(attribs->permission,"-");
    strcat(attribs->permission,"-");
    strcat(attribs->permission,"-");
    //accessed
    sprintf(attribs->accessed,"na");	//keep in mind, that the accessed array is 30 bytes in size (defined in listitems.h)
    //sprintf(attribs->accessed,"na");
    //changed
    sprintf(attribs->changed,"na");
    //for links only
    attribs->gstrLinkTarget=NULL;


    checkPathForLastChar(gstrFullName);
/*	if (path[strlen(path)-1] != '/'){
		g_string_append_c(gstrFullName,'/');
	}*/
	g_string_append(gstrFullName, name);

	if (FALSE == g_file_test(gstrFullName->str,G_FILE_TEST_EXISTS)){
		g_string_free(gstrFullName,TRUE);
		TRACEIT(11,"fileAttributesGet		end, File not found");
		return -1;
	}

	if (dirzeiger != NULL && DirectoryOpened!=0){
		//This call may fall for cds and DVDs, stat() is better in this case
		//We use this because in the first version we always had a directory open when we call this function, therefore there was no reason not to use this
		//and now we need this function for the fileshelf too.
		attribs->type_d = dirzeiger->d_type; //look man dirent for information (4 = directory, 10 = Link)
	}

	if (stat(gstrFullName->str, &fileattributes) == -1){
        sprintf(attribs->accessed,"fstab: errno=%d", errno);	//keep in mind, that the accessed array is 30 bytes in size (defined in listitems.h)
	}
	else{
		//size
		attribs->size = (guint64)fileattributes.st_size;

		//owner
		attribs->owner = fileattributes.st_uid;

		//group
		attribs->group = fileattributes.st_gid;

		//permission
		sprintf(attribs->permission,(S_IRUSR & fileattributes.st_mode)?"r":"-");
		strcat(attribs->permission,(S_IWUSR & fileattributes.st_mode)?"w":"-");
		strcat(attribs->permission,(S_IXUSR & fileattributes.st_mode)?"x":"-");
		strcat(attribs->permission,(S_IRGRP & fileattributes.st_mode)?"r":"-");
		strcat(attribs->permission,(S_IWGRP & fileattributes.st_mode)?"w":"-");
		strcat(attribs->permission,(S_IXGRP & fileattributes.st_mode)?"x":"-");
		strcat(attribs->permission,(S_IROTH & fileattributes.st_mode)?"r":"-");
		strcat(attribs->permission,(S_IWOTH & fileattributes.st_mode)?"w":"-");
		strcat(attribs->permission,(S_IXOTH & fileattributes.st_mode)?"x":"-");

		//type_d _New
		if (attribs->type_d == 0 || !(dirzeiger != NULL && DirectoryOpened!=0) ){
			attribs->type_d = (S_IFDIR & fileattributes.st_mode)?4:0;
		}

		//accessed
		attribs->utaccessed=fileattributes.st_atime;
		sprintf(attribs->accessed,"%s",	ctime(&fileattributes.st_atime));
		attribs->accessed[strlen(attribs->accessed)-1]='\0';

		//changed
		attribs->utchanged=fileattributes.st_mtime;
		sprintf(attribs->changed,"%s",	ctime(&fileattributes.st_mtime));
		attribs->changed[strlen(attribs->changed)-1]='\0';

		//hiddenfile
		if (name[0] == '.' && strcmp(name,".") != 0 && strcmp(name,"..") != 0)
			attribs->hiddenfile = TRUE;
		else
			attribs->hiddenfile = FALSE;
	}
	//check if we have an Link and set the appropriate variables
	attribs->gstrLinkTarget=NULL;
	GFileInfo *SymlinkInfo=NULL;
	GFile *Symfile=NULL;
	const char *cpTarget=NULL;
	int iItsASymlink=0; //1 if it is a symlink, but its no garantee the links exists!!
    if (TRUE == g_file_test( gstrFullName->str, G_FILE_TEST_IS_SYMLINK)){
        //printf("found symlink: %s\n", gstrFullName->str);
        iItsASymlink=1;
        attribs->type_d = 10;
        Symfile = g_file_new_for_path(gstrFullName->str);
        SymlinkInfo = g_file_query_info (Symfile, G_FILE_ATTRIBUTE_STANDARD_SYMLINK_TARGET,
                                                         G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
                                                         NULL,
                                                         NULL);
        cpTarget = g_file_info_get_attribute_byte_string (SymlinkInfo,G_FILE_ATTRIBUTE_STANDARD_SYMLINK_TARGET);
        if (cpTarget){
            //printf("target: %s\n", cpTarget);
            attribs->gstrLinkTarget = g_string_new(cpTarget);
            if (g_file_test( attribs->gstrLinkTarget->str, G_FILE_TEST_EXISTS)){
                if (checkStringIsPath(attribs->gstrLinkTarget->str)){
                    attribs->type_d=4; //handle as path
                }
                else{
                    attribs->type_d=0; //handle as ordinary file
                }
            }
        }
        else{
           // printf("No target found\n");
        }

        if (Symfile)
            g_object_unref(Symfile);
        if (SymlinkInfo)
            g_object_unref(SymlinkInfo);
    }


	//type
	GString *gstrName = g_string_sized_new(10);
	if (iItsASymlink==1 && attribs->gstrLinkTarget !=NULL){
        g_string_assign (gstrName, attribs->gstrLinkTarget->str);
        getLastStringInPath(gstrName);
	}
	else{
        g_string_assign(gstrName, name);
	}
	int len = strlen(gstrName->str);
	int foundit=0;
	for (i=1; i<=MAXTYPELENGTH; i++){
		//was len-i<=1 , but then e.g. 1.jpg was not recognized
		//if (len - i < 1 || dirzeiger->d_type == 4 || dirzeiger->d_type == 10){	//removed because we need this function for the fileshelf too, and do not have an directory opened
		if (len - i < 1 || attribs->type_d == 4 || attribs->type_d == 10){
		    if (attribs->type_d == 10)  //without this, short files will be marked with "-", and than could not be distinguished from directories. important, when marking all items of the same type
                foundit = 10;
			else if (attribs->type_d == 4)
                foundit = 4;
			break;
		}
		if (gstrName->str[len-i] == '.'){
			foundit = 1;
			strncpy (attribs->type,&(gstrName->str[len-i+1]),FIELDSIZE);
			break;
		}
	}
	if (foundit==0)
		strncpy(attribs->type,"-",FIELDSIZE);   //this things are used to distinguish between types, used when marking all items of the same type
    else if(foundit==10)
        strncpy(attribs->type,"---",FIELDSIZE);
    else if(foundit==4)
        strncpy(attribs->type,"--",FIELDSIZE);

    g_string_free(gstrName,TRUE);
	g_string_free(gstrFullName,TRUE);
	TRACEIT(11,"fileAttributesGet		ends");
	return 0;
}


int openDirectory(DIR **Directory,int *DirectoryOpened, GString *dirname){
	TRACEIT(10,"openDirectory		start");
	*Directory=NULL;

	if ((*Directory = opendir (dirname->str)) == 0){
		*DirectoryOpened = 0;
		TRACEIT(10,"openDirectory		ends");
		return -1;
	}
	*DirectoryOpened = 1;
	TRACEIT(10,"openDirectory		ends");
	return 0;
}


void getNextListItem( DIR *Directory, struct dirent **dirzeiger ,int DirectoryOpened, GString *str){
	TRACEIT(11,"getNextListItem		start");

	if (DirectoryOpened==0){
		//*str=NULL;
		g_string_erase(str,0,-1);
		TRACEIT(10,"getNextListItem		ends");
		return;
	}
	*dirzeiger = readdir(Directory);
	if (*dirzeiger == NULL){
		g_string_erase(str,0,-1);
		TRACEIT(10,"getNextListItem		ends");
		return;
	}
	str = g_string_assign(str,(*dirzeiger)->d_name);
	TRACEIT(11,"getNextListItem		ends");
	return;
}


void closeDirectory(DIR **Directory){
	TRACEIT(10,"closeDirectory		start");

	if (*Directory != NULL){
		closedir(*Directory);
		*Directory = NULL;
	}
	TRACEIT(10,"closeDirectory		ends");
}


static void getSizeOfSelectedItemsRecursive(struct RecursiveSizeStructure *stru){
	TRACEIT(10,"getSizeOfSelectedItemsRecursive		start");

	GString *gstrItem=g_string_sized_new(20);
	int i;

	for (i=0; i<stru->ItemCounter; i++){
		if (*(stru->ibreak) != 0){
			break;
		}
		if (stru->intPTypes[i] == 4 /*|| stru->intPTypes[i] == 10*/){
			if (stru->SideObjectOrFileshelfItem==SOURCE_SIDEOBJECT){
	    		g_string_assign(gstrItem, stru->gstrPath->str);
				checkPathForLastChar(gstrItem);
    			g_string_append(gstrItem, (stru->gstrDPItems)[i]->str);
			}
			else{
				g_string_assign(gstrItem, (stru->gstrDPItems)[i]->str);
			}
			if (0 != getDirectorySize(&(stru->guint64_size), gstrItem, stru->ibreak)){  //Error opening directory
                stru->ErrorOccured=1;
			}
        }
    	else{
			stru->guint64_size+=stru->intPSizes[i];
    	}
    	if (*(stru->ibreak) != 0){
    			break;
    	}
    }


	//clean up
	g_string_free(gstrItem,TRUE);

	//We are ready for clean up the structure, tell this the other thread:
	stru->iReadyForCleanUp = 1;

	TRACEIT(10,"getSizeOfSelectedItemsRecursive		ends");
	g_thread_exit(0);
}


void getSizeOfSelectedItemsRecursive_dialog(void *data){
	TRACEIT(10,"getSizeOfSelectedItemsRecursive_dialog		start");
	GtkWidget *dialog, *label;
	int i;
	int len;
	struct RecursiveSizeStructure *stru = (struct RecursiveSizeStructure *) data;
	GString *gstrBuf = g_string_sized_new(10);
	char buf[320];  //just needed to hold the size (only the number), g_printf does not know how to handle 64bit numbers
	GtkButton *CancelButton;
	GtkVBox *VBox;

	gdk_threads_enter();

	stru->ibreak = malloc (sizeof(int));
	*(stru->ibreak) = 0;
	stru->theLabel = gtk_label_new("sdf");
	stru->iReadyForCleanUp=1;
	stru->guint64_size=0;
	stru->ErrorOccured=0;

	int iNotify=0;

	//simple dialog
	dialog = CreateMyWindow("Size of selected items",stru->mainWin,400,250,FALSE, TRUE, TRUE, &iNotify, FALSE);
	//dialog = CreateMyWindow("Size of selected items",NULL,400,250,FALSE, FALSE);
	AddMyIconsToWindow((GtkWindow *)dialog);

	CancelButton = (GtkButton *)gtk_button_new_with_label("Ok");
	g_signal_connect (CancelButton,"clicked",G_CALLBACK(CancelButtonSizeDialog),stru->ibreak);

	VBox = g_object_new(GTK_TYPE_VBOX, "spacing", 5, "homogeneous", FALSE, NULL);

	g_string_printf(gstrBuf,"\tCalculates the size of the selected items.\t\n\tDirectories are searched recursively.\t\n\n\tItems selected: %d\n",stru->ItemCounter);

	label = gtk_label_new(gstrBuf->str);

	gtk_box_pack_start(GTK_BOX(VBox),label,TRUE,TRUE,1);
	gtk_box_pack_start(GTK_BOX(VBox),stru->theLabel,TRUE,TRUE,1);
	gtk_box_pack_start(GTK_BOX(VBox),GTK_WIDGET(CancelButton),FALSE,FALSE,1);
	gtk_container_add((GtkContainer *)dialog,(GtkWidget *)VBox);

	gtk_widget_show_all(dialog);
	gdk_threads_leave();

	if (stru->ItemStringCounter != stru->ItemCounter){	//Error allocation memory for storing ItemInformation
		gtk_label_set_text ((GtkLabel *)stru->theLabel, "Sorry, an error occured while memory allocation.\nUnable to calculate size.");
		*(stru->ibreak) = 1;
		stru->iReadyForCleanUp = 1;
	}
	else{
		stru->iReadyForCleanUp=0;
		g_thread_create((GThreadFunc)getSizeOfSelectedItemsRecursive, (void *)stru , FALSE, NULL);
	}

	do{
		snprintf(buf,99,"%lld",stru->guint64_size);
        g_string_assign(gstrBuf,buf);
		len = strlen(gstrBuf->str)-3;
		while (len>0){
			g_string_insert_c(gstrBuf,len,'.');
			len -=3;
		};
		strcpy(buf,gstrBuf->str);
		if (stru->ErrorOccured == 1){
            g_string_sprintf(gstrBuf,"Selected need %s bytes\t\nBut there was at least one directory\nthat could not be opened.",buf);
		}
		else{
            g_string_sprintf(gstrBuf,"Selected need %s bytes\t",buf);
		}
		gdk_threads_enter();
		if (iNotify==1){    //as long this is 1 the dialog is valid and was not destroyed
            gtk_label_set_text ((GtkLabel *)stru->theLabel, gstrBuf->str);
		}
		else{
            *(stru->ibreak)=1;
		}
		gdk_threads_leave();
		usleep(100000);
	}while(*(stru->ibreak) == 0);

	while (stru->iReadyForCleanUp != 1){
		//The other thread has not finished yet, we should wait for him. but do not block this widget
		usleep(10000);
	}


	gdk_threads_enter();
	if (iNotify==1){    //then, the window was closed by our ok-button and not by upper-right-corner-X, so we have to destroy the window by ourself
        gtk_widget_destroy(dialog);
	}
	gdk_threads_leave();


	//free everything in the struct and the struct itself
	free(stru->ibreak);
	if (stru->gstrPath != NULL){
		g_string_free(stru->gstrPath,TRUE);
	}
	for (i=0; i<stru->ItemStringCounter; i++){
		g_string_free(stru->gstrDPItems[i], TRUE);
	}
    free(stru->gstrDPItems);
	free(stru->intPSizes);
	free(stru->intPTypes);
	g_string_free(gstrBuf,TRUE);

	TRACEIT(10,"getSizeOfSelectedItemsRecursive_dialog		ends");
	g_thread_exit(0);
}


void CancelButtonSizeDialog(GtkButton *button, gpointer data){
	TRACEIT(10,"CancelButton		start");

	int *ibreak = (int *)data;
	*ibreak = 1;

	TRACEIT(10,"CancelButton		ends");
}


int getDirectorySize(guint64 *size, GString *gstrDirectory, int *ibreak){
    //stops when ibreak == 1
	TRACEIT(10,"getDirectorySize		start");
	if (size == NULL || gstrDirectory == NULL || ibreak == NULL){
        TRACEIT(2,"getDirectorySize		end, Parameter NULL");
        return -1;
	}
	if (FALSE == g_file_test( gstrDirectory->str, G_FILE_TEST_EXISTS)){
        TRACEIT(2,"getDirectorySize		end, error opening item");
        return -1;
	}
	GString *gstrNames = g_string_new(gstrDirectory->str);
	GString *Path= g_string_new(gstrDirectory->str);
	struct FileAttributs attribs;
	DIR *Directory;
	struct dirent *dirzeiger;
	int DirectoryOpened=0;

    //check if we have an Directory
    if (FALSE == checkStringIsPath( gstrDirectory->str )){ //then we have no directory
        //g_string_assign(gstrNames, gstrDirectory->str);
        checkPathForLastCharOtherwise(gstrNames);
        getLastStringInPath(gstrNames);
        GetPathFromPathWithName(Path, 1);
        if (strcmp(gstrNames->str,".")!=0 && strcmp(gstrNames->str,"..")!=0){
            fileAttributesGet(NULL, 0, &attribs, Path->str, gstrNames->str);
            if (attribs.gstrLinkTarget)
                g_string_free(attribs.gstrLinkTarget, TRUE);
            *size+=attribs.size;
        }
        g_string_free(gstrNames,TRUE);
        g_string_free(Path,TRUE);
        TRACEIT(10,"getDirectorySize		end");
//printf("found FILE: %s, its size: %lld\n",gstrDirectory->str, attribs.size);
        return 0;
    }

	if (0 != openDirectory(&Directory, &DirectoryOpened, gstrDirectory)){
	    g_string_free(gstrNames,TRUE);
        g_string_free(Path,TRUE);
        TRACEIT(10,"getDirectorySize		end");
        return -1;
    }

	getNextListItem(Directory, &dirzeiger, DirectoryOpened, gstrNames);

	while ( strlen(gstrNames->str) != 0 && *ibreak == 0){

		fileAttributesGet(dirzeiger, DirectoryOpened, &attribs, gstrDirectory->str, gstrNames->str);
		if (attribs.gstrLinkTarget)
                g_string_free(attribs.gstrLinkTarget, TRUE);
		if (strcmp(gstrNames->str,".")!=0 && strcmp(gstrNames->str,"..")!=0 && (attribs.type_d==4/* || attribs.type_d==10 */) ){
			g_string_assign(Path,gstrDirectory->str);
			checkPathForLastChar(Path);
			g_string_append(Path,gstrNames->str);
			getDirectorySize(size, Path, ibreak);
		}
		else if (strcmp(gstrNames->str,".")!=0 && strcmp(gstrNames->str,"..")!=0){
			*size+=attribs.size;
		}
		getNextListItem(Directory, &dirzeiger, DirectoryOpened, gstrNames);
	}

	//Subern:
	closeDirectory(&Directory);
	g_string_free(gstrNames,TRUE);
	g_string_free(Path,TRUE);

    TRACEIT(10,"getDirectorySize		end");
    if (*ibreak == 0)
        return 0;
    else
        return -1;
}



static void mySelectAllFunction_Helper(GtkTreeModel *model,GtkTreePath *path,GtkTreeIter *iter,gpointer data){
    //if (gtk_tree_model_get_iter(model, iter, path)){
    TRACEIT(10,"mySelectAllFunction_Helper		start");

    // 1'st POSSIBILITY
    struct AllInformationAllUseStruct *stru = (struct AllInformationAllUseStruct *)data;
    GtkTreeSelection **selection = (GtkTreeSelection **)stru->Info_2;
	int *found = (int *)stru->Info_1;
	if (*found >= 2){
	    TRACEIT(10,"mySelectAllFunction_Helper		end, . and .. already found");
        return;
	}
    gchar *Name=NULL;
	gtk_tree_model_get(model, iter, COL_NAME, &Name, -1);

	if ((strcmp(Name, "..")==0 || strcmp(Name,".")==0)){
        (*found)+=1;
        gtk_tree_selection_unselect_iter(*selection, iter);
	}

    if (Name)
        g_free(Name);
        // END 1'st POSSIBILITY

/*

    // 2'st POSSIBILITY
    gchar *Name=NULL;
    GtkTreeSelection **selection = (GtkTreeSelection **)data;
    gtk_tree_model_get(model, iter, COL_NAME, &Name, -1);
    //gtk_tree_model_get(model, iter, COL_NAME, &Name, -1);
    if (Name != NULL && !(strcmp(Name, "..")==0 || strcmp(Name,".")==0)){
        gtk_tree_selection_select_iter(*selection, iter);
	}
    if (Name)
        g_free(Name);
    // END 2'st POSSIBILITY
*/
    TRACEIT(10,"mySelectAllFunction_Helper		end");
}


gboolean mySelectAllFunction( GtkTreeView *treeview, gpointer user_data){
    //catches the signel select-all from within the listviews (side views only)
    TRACEIT(10,"mySelectAllFunction		start");

    // 1'st POSSIBILITY
    GtkTreeSelection	*selection;
    struct AllInformationAllUseStruct stru;
    int found=0;
    stru.Info_1 = (void *)&found;
    stru.Info_2 = (void *)&selection;
    stru.Info_3 = NULL;
    stru.Info_4 = NULL;
    stru.Info_5 = NULL;

    selection  = gtk_tree_view_get_selection(treeview);
    gtk_tree_selection_select_all ( selection );
    gtk_tree_selection_selected_foreach(selection, mySelectAllFunction_Helper, (void *)&stru);
    // END 1'st POSSIBILITY


/*
    // 2'st POSSIBILITY
    GtkTreeSelection *selection;    // Tested within the library directory with more than 200 entries: this solution is realy slow!!
    selection = gtk_tree_view_get_selection(treeview);
    struct SideObjectList *Object = (struct SideObjectList *)user_data;
    gtk_tree_model_foreach(GTK_TREE_MODEL(Object->listStore),(GtkTreeModelForeachFunc) mySelectAllFunction_Helper,(void *)&selection);
    // END 2'st POSSIBILITY
*/
    TRACEIT(10,"mySelectAllFunction		end");
    return TRUE;
}



static void markItemsWithSameType_Helper_1(GtkTreeModel *model,GtkTreePath *path,GtkTreeIter *iter,gpointer data){
	TRACEIT(11,"markItemsWithSameType_Helper_1		start");

    struct AllInformationAllUseStruct *stru = (struct AllInformationAllUseStruct *)data;
	struct updateList **typeList=(struct updateList **)stru->Info_1;
	int *ColType = stru->Info_2;

	gchar *type=NULL;
	gchar *typebuf=NULL;

    gtk_tree_model_get(model, iter, *ColType,&type, -1);

    typebuf = g_utf8_strdown(type,-1);
    if (*typeList != NULL){
        if (0==updateList_checkForItem_NoRef(typeList, typebuf)){
            updateList_addItem(typeList, typebuf, NULL, NULL, 0);
        }
    }
    else{
        updateList_addItem(typeList, typebuf, NULL, NULL, 0);
    }

    if (type)
        g_free(type);
    if (typebuf)
        g_free(typebuf);
	TRACEIT(11,"markItemsWithSameType_Helper_1		end");
}



static void markItemsWithSameType_Helper_2(GtkTreeModel *model,GtkTreePath *path,GtkTreeIter *iter,gpointer data){
	TRACEIT(11,"markItemsWithSameType_Helper_2		start");

    struct AllInformationAllUseStruct *stru = (struct AllInformationAllUseStruct *)data;
	struct updateList **typeList=(struct updateList **)stru->Info_1;
	int *ColType = stru->Info_2;
	GtkTreeSelection *selection = (GtkTreeSelection *)stru->Info_3;

	gchar *type=NULL;
	gchar *typebuf=NULL;

    gtk_tree_model_get(model, iter, *ColType,&type, -1);

    typebuf = g_utf8_strdown(type,-1);
    struct updateList *bufList = *typeList;
    updateList_getFirstItem(&bufList);

    if (0==updateList_checkForItem_NoRef(&bufList, typebuf)){
        gtk_tree_selection_unselect_iter(selection, iter);
    }

    if (type)
        g_free(type);
    if (typebuf)
        g_free(typebuf);
	TRACEIT(11,"markItemsWithSameType_Helper_2		end");
}


void markItemsWithSameType(GtkTreeView *treeview, int ColType){
    //selects all items in treeview wich are from the same type as the already marked one.
    //ColType holds the ID of the coloumn with the type information


    //get the types of all marked items and put them to in a list
    struct AllInformationAllUseStruct stru;
    struct updateList *typeList=NULL;

    stru.Info_1 = (void *)&typeList;
    stru.Info_2 = (void *)&ColType;

    GtkTreeSelection *selection;
	selection = gtk_tree_view_get_selection(treeview);
	gtk_tree_selection_selected_foreach(selection, markItemsWithSameType_Helper_1, (void *)&stru);

/*    updateList_getFirstItem(&typeList);
    struct updateList *bufList=typeList;
    while(bufList != NULL){
        printf("names=%s\n",bufList->gstrName->str);
        bufList = bufList->NextItem;
    }
*/
    //The list is ready, now mark all items and walk throught the view and unmark the right ones.
    stru.Info_3 = (void *)selection;
    gtk_tree_selection_select_all(selection);

    gtk_tree_selection_selected_foreach(selection, markItemsWithSameType_Helper_2, (void *)&stru);


    //now the . and .. may be marked: check this
    int found=0;
    stru.Info_1 = (void *)&found;
    stru.Info_2 = (void *)&selection;
    gtk_tree_selection_selected_foreach(selection, mySelectAllFunction_Helper, (void *)&stru);

    updateList_deleteList(&typeList);

}










