/*
    Copyright 2007, 2008, 2009 Geyer Klaus

    This file is part of Cat'sEyE.

    Cat'sEyE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Cat'sEyE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Cat'sEyE.  If not, see <http://www.gnu.org/licenses/>.
*/


#include <gtk/gtk.h>
#include <gio/gio.h>
#include <string.h>
#include <stdlib.h>
#include <dirent.h>	//Verzeichnis Zeiger und DIR - Struktur
#include <unistd.h> //usleep

#include "listitems.h"
#include "mytrace.h"
#include "Lists.h"
#include "helpers.h"
#include "globalDefinitions.h"
#include "CommandParser.h"
#include "myfm.h"
#include "fileshelf.h"




void ProcessItemsViaList(void *data);
gboolean oneMouseButtonPressedInLittleWindow(GtkWidget *treeview, GdkEventButton *event, gpointer data);
void calcDirectorySizeInView_thread(struct AllInformationAllUseStruct *stru);
gboolean ListWidget_DeleteEvent (GtkWidget *widget, GdkEvent  *event, struct ListWidgetCommunicationStruct *commStru);
void ListWidget_CancelButton(GtkButton *button, gpointer data);
void ListWidget_CloseButton(GtkButton *button, struct ListWidgetCommunicationStruct *commStru);
gboolean FreeMemoryAndDeleteWidgetFunction (struct ListWidgetCommunicationStruct *commStru);

extern GtkWindow 	*win;


int callItemsToSystemWrap(GString *usercommand, GString *FileList, GString *FileListWithPath, GString *WorkPath, GString *OtherSidePath, struct CopyReMoveList *list, gboolean bThreaded){
	// if bThreaded==TRUE, the usercommand will be called within its own thread (asynchrone mode)
	// if bThreaded==FALSE, the usercommand will be called directly (synchrone mode)
	TRACEIT(10,"callItemsToSystemWrap		start");
	int i;
	GError *Error=NULL;
	struct CallItemsToSystemStructure *stru = malloc(sizeof(struct CallItemsToSystemStructure));
	if (usercommand == NULL){
		TRACEIT(2,"callItemsToSystemWrap		end, Usercommand= NULL");
		return -1;
	}
	GString *ParsedCommand = g_string_new(usercommand->str);
	GString *QuitReplyText=NULL;
	GString *ActionNoteText=NULL;
	//GString *UpdateFolderWhenFinished=NULL;
	struct updateList *UpdateFolderWhenFinished_List=NULL;


	if(list != NULL && !g_thread_supported() ){
		g_thread_init(NULL);
     		//gdk_threads_init();                   // Called to initialize internal mutex "gdk_threads_mutex".
   	}

   	//Give the usercommand to our commandparser
	if (list)	//when we were started via list, than we are already in a new thread and we need to call thread_enter/leave because ParseUserCommand may creaty a dialog!!
		gdk_threads_enter();

	i = ParseUserCommand(ParsedCommand, FileList, FileListWithPath, WorkPath, OtherSidePath, &ActionNoteText, &QuitReplyText, &UpdateFolderWhenFinished_List);

	if (list){
	   	gdk_threads_leave();
        if (QuitReplyText){     //ignore this property (we need to free it and set it to NULL) when processing via list
            g_string_free(QuitReplyText,TRUE);
        }
        QuitReplyText=NULL;
	}

	if ( i == 0){
		//ParseUserCommand has done his work, we have a new usercommand
		stru->usercommand=g_string_new(ParsedCommand->str);
		stru->WorkPath=g_string_new(WorkPath->str);
		stru->FileList=g_string_sized_new(2);
		stru->FileList->str[0]='\0';
		stru->QuitReplyText = QuitReplyText;
		stru->ActionNoteText = ActionNoteText;
		stru->UpdateFolderWhenFinished_List = UpdateFolderWhenFinished_List;
		stru->data = list;
	}
	else if(i < 0){
		g_string_free(ParsedCommand,TRUE);
		if (QuitReplyText)
            g_string_free(QuitReplyText,TRUE);
        if (ActionNoteText)
            g_string_free(ActionNoteText, TRUE);
        QuitReplyText=NULL;
        if (UpdateFolderWhenFinished_List){
            updateList_deleteList(&UpdateFolderWhenFinished_List);
            //g_string_free(UpdateFolderWhenFinished,TRUE);
            UpdateFolderWhenFinished_List=NULL;
        }
		if (list){
			list->status = 1;
		}
		TRACEIT(10,"callItemsToSystemWrap		end, 1");
		return 0;
	}
	else{
		//It's not a new usercommand, it's an old one
		stru->usercommand=g_string_new(usercommand->str);
		stru->FileList=g_string_new(FileList->str);
		stru->WorkPath=g_string_new(WorkPath->str);
        stru->QuitReplyText=NULL;
        stru->ActionNoteText=NULL;
        stru->UpdateFolderWhenFinished_List=NULL;
		stru->data=list;
	}

	TRACEIT(12,"callItemsToSystemWrap		beginsThread");
	stru->bThreaded=bThreaded;  //callItemsToSystem may or may not end with g_thread_exit() or return
	if (bThreaded){
        g_thread_create((GThreadFunc)callItemsToSystem, (void *)stru , FALSE, &Error);
	}
	else{
	    callItemsToSystem((void *)stru);
	}

	if (Error){
		TRACEIT(2,"callItemsToSystemWrap: error in g_thread_create:");
		TRACEIT(2,Error->message);
		g_error_free(Error);
		Error=NULL;
	}
	TRACEIT(12,"callItemsToSystemWrap		afterThread");
	g_string_free(ParsedCommand,TRUE);
	TRACEIT(10,"callItemsToSystemWrap		ends");
	return 0;
}


int ProcessItemsViaListWrap(struct CopyReMoveList *FirstItem, int process){
	TRACEIT(10,"ProcessItemsViaListWrap		start");
	GError *Error=NULL;
	if (FirstItem == NULL){
		TRACEIT(10,"ProcessItemsViaListWrap		ends -1");
		return -1;
	}
	if( !g_thread_supported() ){
		g_thread_init(NULL);
		//gdk_threads_init();
    }
	//FirstItem->CopyMoveOrRemove=process;
	struct CopyReMoveList *bufitem = FirstItem;
	while (bufitem){
        bufitem->CopyMoveOrRemove=process;
        bufitem = bufitem->NextItem;
	};

	g_thread_create((GThreadFunc)ProcessItemsViaList, (void *)FirstItem , FALSE, &Error);
	if (Error){
		TRACEIT(2,"ProcessItemsViaListWrap: error in g_thread_create:");
		TRACEIT(2,Error->message);
		g_error_free(Error);
		Error=NULL;
	}
	TRACEIT(10,"ProcessItemsViaListWrap		ends");
	return 0;
}

/*
void CloseCopyMoveOrRemoveButton(GtkButton *button, gpointer data){
    TRACEIT(10,"CloseCopyMoveOrRemoveButton		start");

    gtk_main_quit ();

    TRACEIT(10,"CloseCopyMoveOrRemoveButton		ends");
    return;
}
*/
/*
void CancelCopyMoveOrRemoveButton(GtkButton *button, gpointer data){
	TRACEIT(10,"CancelCopyMoveOrRemoveButton		start");
	GtkWidget *dialog, *label;

	dialog = CreateMyDialog("Do you want to cancel Process?", win, 2, "Ok",GTK_RESPONSE_ACCEPT,"Cancel",GTK_RESPONSE_REJECT, TRUE);
	label = gtk_label_new("If you press YES, then the currently processed item will be finished,\nall the other are stopped.\n");
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),label);
	gtk_widget_show_all(dialog);
	if (gtk_dialog_run(GTK_DIALOG(dialog))== GTK_RESPONSE_ACCEPT){
		struct CopyReMoveList *list = (struct CopyReMoveList *)data;
		struct CopyReMoveList *buflist = list;
		while (buflist != NULL){
			if (buflist->status == -1){
				buflist->status = -10;
			}
			buflist = buflist->NextItem;
		};
	}
	gtk_widget_destroy(dialog);
	TRACEIT(10,"CancelCopyMoveOrRemoveButton		ends");
}
*/


void ProcessUserCommandViaList(void *data){
	TRACEIT(10,"ProcessUserCommandViaList		start");
	if (data == NULL){
		TRACEIT(10,"ProcessUserCommandViaList		ends");
		g_thread_exit(0);
	}
	GtkWidget *Window, *scrollWindow, *listView;
	GtkListStore *listStore;
	GtkTreeIter iter;
	GtkTreeViewColumn   	*col_Items, *col_ItemNumbers, *col_Errors;
	GtkCellRenderer     	*renderer, *renderer2;
	GtkButton *CancelButton;
	GtkButton *CloseButton;
	GtkWidget *TheLabel;
	GtkVBox *VBox;
	GString *WindowName = g_string_new(MYNAMESHORT);    //freed when the commStru is freed
	struct CopyReMoveList *list = (struct CopyReMoveList *)data;
	struct CopyReMoveList *listbuf = list;
	int i=0;
	int intItemsCount=0;
    struct ListWidgetCommunicationStruct *commStru = malloc(sizeof(struct ListWidgetCommunicationStruct));
	gdk_threads_enter();


	Window = CreateMyWindow(WindowName->str, NULL, 400,200,TRUE, FALSE, TRUE, NULL, FALSE);
	if (Window == NULL){
        g_string_free(WindowName, TRUE);
        free(commStru);
        TRACEIT(1,"ProcessItemsViaList		end, window could not be created");
        return;
	}
    g_signal_connect (Window,"delete-event",G_CALLBACK(ListWidget_DeleteEvent),commStru);
    //g_signal_connect (Window, "destroy", G_CALLBACK(ListWidget_End_FreeMemory),commStru);

	AddMyIconsToWindow((GtkWindow *)Window);

  	char buf[320];

	scrollWindow = gtk_scrolled_window_new (NULL,NULL);
	listView = gtk_tree_view_new ();
	listStore = gtk_list_store_new( 3 , G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING  );

    col_Items=gtk_tree_view_column_new();
	col_ItemNumbers=gtk_tree_view_column_new();
	col_Errors=gtk_tree_view_column_new();
	gtk_tree_view_column_set_resizable(col_Items,TRUE);
	gtk_tree_view_column_set_resizable(col_ItemNumbers,TRUE);
	gtk_tree_view_column_set_resizable(col_Errors,TRUE);

	gtk_tree_view_column_set_title(col_ItemNumbers, "#");
	gtk_tree_view_append_column(GTK_TREE_VIEW(listView),col_ItemNumbers);
	renderer2 = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(col_ItemNumbers, renderer2, TRUE);
	gtk_tree_view_column_add_attribute(col_ItemNumbers,renderer2,"text",0);

	gtk_tree_view_column_set_title(col_Items, "items to process");
	gtk_tree_view_append_column(GTK_TREE_VIEW(listView),col_Items);
	renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(col_Items, renderer, TRUE);
	gtk_tree_view_column_add_attribute(col_Items,renderer,"text",1);

	gtk_tree_view_column_set_title(col_Errors, "errors");
	gtk_tree_view_append_column(GTK_TREE_VIEW(listView),col_Errors);
	renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(col_Errors, renderer, TRUE);
	gtk_tree_view_column_add_attribute(col_Errors,renderer,"text",2);

	g_signal_connect (listView,"button-press-event",G_CALLBACK(oneMouseButtonPressedInLittleWindow),NULL);

	do{
		intItemsCount++;
		listbuf = listbuf->NextItem;
	}while(listbuf!= NULL);
	listbuf = list;
	i=1;
	do{
		snprintf(buf,319,"%d/%d",i,intItemsCount);
		gtk_list_store_append(listStore,&iter);
		gtk_list_store_set (listStore, &iter, 0, buf, 1, listbuf->string1->str,-1);

        //listbuf->listStore=listStore;   //forget it, this is just for CalcSizeAfterCopyMove
		listbuf->iter = iter;
		listbuf->status = -1;
		i++;

        listbuf->CopyMoveOrRemove = list->CopyMoveOrRemove;

		listbuf = listbuf->NextItem;
	}while(listbuf!= NULL);

	CancelButton = (GtkButton *)gtk_button_new_with_label("cancel");
	g_signal_connect (CancelButton,"clicked",G_CALLBACK(ListWidget_CancelButton),(void *)commStru);

	CloseButton = (GtkButton *)gtk_button_new_with_label("close");
	g_signal_connect (CloseButton,"clicked",G_CALLBACK(ListWidget_CloseButton),commStru);

	gtk_tree_view_set_model (GTK_TREE_VIEW(listView), GTK_TREE_MODEL(listStore));

    if (list->gstrLabel){
		TheLabel = gtk_label_new(list->gstrLabel->str);
	}
	else{
		TheLabel = gtk_label_new(list->UserCommand->str);
	}

	gtk_container_add((GtkContainer *)scrollWindow, listView);
	VBox = g_object_new(GTK_TYPE_VBOX, "spacing", 5, "homogeneous", FALSE, NULL);
	gtk_box_pack_start(GTK_BOX(VBox),TheLabel,FALSE,FALSE,1);
	gtk_box_pack_start(GTK_BOX(VBox),GTK_WIDGET(scrollWindow),TRUE,TRUE,1);
	gtk_box_pack_start(GTK_BOX(VBox),GTK_WIDGET(CancelButton),FALSE,FALSE,1);
    gtk_box_pack_start(GTK_BOX(VBox),GTK_WIDGET(CloseButton),FALSE,FALSE,1);

    gtk_container_add((GtkContainer *)Window,(GtkWidget *)VBox);

    commStru->mainListWidget=Window;
    commStru->listView=listView;
    commStru->cancelButton=CancelButton;
    commStru->closeButton=CloseButton;
    commStru->firstItem=list;
    commStru->currentItem=list;
    commStru->listStore=listStore;
    commStru->cancellable=NULL;//g_cancellable_new();
    commStru->gerror=NULL;
    commStru->SecondThreadRunning=0;
    commStru->TheLabel=TheLabel;
    commStru->WindowName=WindowName;
    commStru->iCancelled=0;  //becomes 1 if the process is cancelled
    commStru->FreeMemoryAndDeleteWidgetFunction = FreeMemoryAndDeleteWidgetFunction;
    commStru->CancelListFunction = NULL;    // we do not need this in here
    commStru->iCancelDialogRuns=0;
    commStru->iDestroyFlag =0;

	gtk_widget_show_all(Window);
	gtk_widget_hide(GTK_WIDGET(CloseButton));

	gdk_threads_leave();


    //start processing the list,
	GString *gstrError = g_string_sized_new(20);
	GString *FileListWithPath = g_string_sized_new(10);
	GString *WorkDir = g_string_sized_new(10);
    GtkTreePath *pTreePath;
    GtkTreeSelection *selection;
    gdk_threads_enter();
    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(commStru->listView));
    gdk_threads_leave();
    int iSomeError=0;

    commStru->SecondThreadRunning = 1;  //I know I know this is the firstThread, but who cares about names.

    while (commStru->currentItem && commStru->iCancelled==0){
        list = commStru->currentItem;   //for faster writing. and shorter sentences
        if (list->string1==NULL || ( (list->CopyMoveOrRemove == 1 || list->CopyMoveOrRemove == 2) && list->string2==NULL) ){
            //list->ibreak=1;
            continue;
        }
        //list->ibreak=0;
        gdk_threads_enter();
        gtk_tree_selection_select_iter(selection, &(list->iter));

        //make the scrolling follow the marked Item
        pTreePath = gtk_tree_model_get_path (gtk_tree_view_get_model ((GtkTreeView *)commStru->listView),&(list->iter));
        gtk_tree_view_scroll_to_cell ((GtkTreeView *)commStru->listView,pTreePath,NULL,TRUE,0.0,0.0);
        gtk_tree_path_free(pTreePath);
        gdk_threads_leave();

        if (list->status == -10 || commStru->iCancelled==1){
            //list->ibreak=1;
            break;
        }
        if (list->ErrorMessage)
            g_string_free(list->ErrorMessage,TRUE);
        list->ErrorMessage = g_string_sized_new(10);    //make clear this pointer is valid  //TODO FREE IT AND FILL IT WITH 0 on list creation

        if (commStru->gerror){
            g_error_free(commStru->gerror);
            commStru->gerror=NULL;
            TRACEIT(10,"copyItemRecursive		error struct was not NULL, checkpoint 2");
        }

        //TODO check status before set
        list->status = 0;

        //collect data
        g_string_assign(FileListWithPath, list->string1->str);
        g_string_assign(WorkDir,list->string1->str);
        cutCharFromGString(WorkDir, '\"', 0);
        GetPathFromPathWithName(WorkDir, 1);
        getLastStringInPath(list->string1);
        g_string_prepend(list->string1, "\"");

        //call item to System (synchron)
        //printf("BEFORE CALLING TO SYSTEM\n");
        callItemsToSystemWrap(commStru->firstItem->UserCommand,list->string1, FileListWithPath, WorkDir, list->string2, list, FALSE);
        //printf("AFTER CALLING TO SYSTEM\n");
        if (commStru->iCancelled==1){
            break;
        }

        if (list->Errors!=NULL){
            gdk_threads_enter();
            gtk_list_store_set (listStore, &(list->iter), 2, list->Errors->str,-1);
            gdk_threads_enter();

            iSomeError=1;
        }

/*
        if (iRet == 0 ){    //Error copying look GError for information
            g_string_erase(gstrError,0,-1);
            if (commStru->gerror){
                if (G_IO_ERROR_CANCELLED != commStru->gerror->code){
                    g_string_printf(gstrError, "Error: code: %d, message:%s",commStru->gerror->code, commStru->gerror->message);
                    g_error_free(commStru->gerror);
                    commStru->gerror=NULL;
                    iSomeError = 1;
                }
                else
                    iSomeError = 2; //do not show the hint that there was an error
            }
            else if (strlen (list->ErrorMessage->str) != 0){
                g_string_assign (gstrError, list->ErrorMessage->str);
                iSomeError = 1;
            }
            else{
                g_string_assign(gstrError, "Unknown error while processing items");
                iSomeError = 1;
            }
            if (strlen(gstrError->str)>0){
                gdk_threads_enter();
                gtk_list_store_set (commStru->listStore, &(list->iter), 3, gstrError->str,-1);
                gdk_threads_leave();
                iSomeError=1;
            }
        }
        else if(iRet == -1){    //internal Error
            g_string_assign(gstrError, "BAD INTERNAL ERROR, INFORM THE PROGRAMMER");
            gdk_threads_enter();
            gtk_list_store_set (commStru->listStore, &(list->iter), 3, gstrError->str,-1);
            gdk_threads_leave();
            iSomeError=1;
        }
        */
        if (commStru->iCancelled==1){
            //list->ibreak=1;
            break;
        }
        commStru->currentItem = commStru->currentItem->NextItem;
    }
    //list->ibreak=1; //for security reason, this cancels the size calculation thread
    //we can come up here when there was
    //some error, in this case we may have a cancel
    //just a cancel
    //we finished, no error, no cancel
    //only in the last case we going to close the widget from here


    //printf("after thread 2 loop\n");
    if (commStru->iCancelled==1 || iSomeError!=0){ //error or cancel, its not our turn to close the widget, but to update some things

        gdk_threads_enter();
        if (iSomeError == 1){
            MyMessageWidget((GtkWindow *)commStru->mainListWidget, "ERROR", "There was at least one Error \nwhile processing the list.\nLook at the column errors and scroll up\ntill you find the error message.");
            g_string_append(commStru->WindowName," ERROR");
        }
        else if (commStru->iCancelled == 1){
            g_string_append(commStru->WindowName," Cancelled");
        }
        gtk_window_set_title((GtkWindow *)commStru->mainListWidget,commStru->WindowName->str);

        gtk_widget_show(GTK_WIDGET(commStru->closeButton));
        gtk_widget_hide(GTK_WIDGET(commStru->cancelButton));

        gtk_widget_queue_draw ((GtkWidget *)commStru->mainListWidget);
        gdk_threads_leave();

        commStru->SecondThreadRunning = 0;
	}
	else if (commStru->iCancelDialogRuns==0){
	    gdk_threads_enter();
	    commStru->SecondThreadRunning = 0;
	    (commStru->FreeMemoryAndDeleteWidgetFunction)( commStru);  //here Notify MUST BE 2 other wise this call will lead to a deadlock becuase of SecondThreadRunning
	    gdk_threads_leave();
	}
	else{ //the cancel dialog is open
        commStru->SecondThreadRunning = 0;
        commStru->iDestroyFlag = 1;
	}

    //free things
    g_string_free(gstrError, TRUE);
    g_string_free(FileListWithPath, TRUE);
    if (WorkDir)
		g_string_free(WorkDir,TRUE);

	TRACEIT(10,"ProcessItemsViaList		ends");
    g_thread_exit(0);
}



gboolean FreeMemoryAndDeleteWidgetFunction (struct ListWidgetCommunicationStruct *commStru){
    //ATTENTION!!
    //do not access commStru after this call, it is free after this.
    //printf("FREE MEMORY\n");
    //free all memory in the list
    if (commStru->cancellable)
        g_object_unref(commStru->cancellable);
    if (commStru->gerror)
        g_error_free (commStru->gerror);
    g_object_unref(commStru->listStore);

    CopyReMoveList_DeleteList (commStru->firstItem);

    g_string_free(commStru->WindowName, TRUE);

    //detroy the widget
    gtk_widget_destroy(commStru->mainListWidget);

    free(commStru);

    return TRUE;
}


void ListWidget_CancelList(struct ListWidgetCommunicationStruct *commStru){
    TRACEIT(10,"ListWidget_CancelList		start");
    //struct CopyReMoveList *list = (struct CopyReMoveList *)commStru->firstItem;
    struct CopyReMoveList *buflist = (struct CopyReMoveList *)commStru->firstItem;
    while (buflist != NULL){
        if (buflist->status == -1){
            buflist->status = -10;
        }
        buflist->ibreak=1;
        buflist = buflist->NextItem;
    };
    if (commStru->cancellable)
        g_cancellable_cancel (commStru->cancellable);
    commStru->iCancelled=1;


    gtk_widget_show(GTK_WIDGET(commStru->closeButton));
    gtk_widget_hide(GTK_WIDGET(commStru->cancelButton));
    TRACEIT(10,"ListWidget_CancelList		end");
}


void ListWidget_CancelButton(GtkButton *button, gpointer data){
    //simply asks the user for canceling
    //and if the answer is ok, sets status of buflist to -10 (cancelled)
    //and cancels  the commstru->cancellable object
	TRACEIT(10,"ListWidget_CancelButton		start");
	GtkWidget *dialog, *label;
	struct ListWidgetCommunicationStruct *commStru = (struct ListWidgetCommunicationStruct *)data;

	if (commStru->iCancelled == 1){
	    TRACEIT(4,"ListWidget_CancelButton		ends, allready cancelled");
        return;
	}
    commStru->iCancelDialogRuns=1;
    dialog = CreateMyDialog("Do you want to cancel Process?", (GtkWindow *)commStru->mainListWidget, 2, "Ok",GTK_RESPONSE_ACCEPT,"Cancel",GTK_RESPONSE_REJECT, TRUE);
    label = gtk_label_new("This cancels the processes in this list\nDo you want to continue?\n");
    gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),label);
    gtk_widget_show_all(dialog);
    if (gtk_dialog_run(GTK_DIALOG(dialog))== GTK_RESPONSE_ACCEPT){
        ListWidget_CancelList(commStru);
    }
    gtk_widget_destroy(dialog);
    commStru->iCancelDialogRuns=0;

    //check if the thread 2 allready exists and we can close all
    if (commStru->iDestroyFlag == 1 && commStru->SecondThreadRunning == 0){
        FreeMemoryAndDeleteWidgetFunction(commStru);
    }

	TRACEIT(10,"ListWidget_CancelButton		ends");
}



gboolean ListWidget_DeleteEvent (GtkWidget *widget, GdkEvent  *event, struct ListWidgetCommunicationStruct *commStru){
    TRACEIT(10,"ListWidget_DeleteEvent		start");
    //this is used to catch the signal emited when the user presses the X in the upper right corner
    //printf("DELETE EVENT\n");
    if (commStru == NULL){
        TRACEIT(2,"ListWidget_DeleteEvent		end, Parameter NULL");
        return FALSE;
    }
    //thread 2 may still run -> we have to ask for canceling
   //we may have allready cancelled -> we have to close

    if (commStru->iCancelled == 1){
        if (commStru->SecondThreadRunning == 1){
            gdk_threads_enter();
            MyMessageWidget((GtkWindow *)commStru->mainListWidget, "Error", "List item is still active, please close the aplication\nwhich is connect to the item first.");
            gdk_threads_leave();
        }
        else
            FreeMemoryAndDeleteWidgetFunction(commStru);
        return TRUE;
    }
    if (commStru->SecondThreadRunning == 1 && commStru->iCancelled == 0){
        ListWidget_CancelButton(NULL, (void *)commStru);
        if (commStru->iCancelled == 1){
            commStru->iDestroyFlag = 1;
        }
        return TRUE;
    }
    if (commStru->iDestroyFlag == 1){
        FreeMemoryAndDeleteWidgetFunction(commStru);
        return TRUE;
    }
    return TRUE;

    TRACEIT(10,"ListWidget_DeleteEvent		end");
//    return gRet;   //when this is TRUE, the widget will stay even when the user tries to close it. And because of our notification we d not close it either
                    //FALSE will destroy the widget
}


void ListWidget_CloseButton(GtkButton *button, struct ListWidgetCommunicationStruct *commStru){
    TRACEIT(10,"CloseCopyMoveOrRemoveButton		start");
    if (commStru == NULL){
        TRACEIT(2,"CloseCopyMoveOrRemoveButton		ends, Parameter NULL");
        return;
    }
    //printf("Close button Event\n");

    //if the close_button is visible, the thread 2 has allready finished there cant be even an cancel dialog when pressing this
    if (commStru->SecondThreadRunning == 1){
        gdk_threads_enter();
        MyMessageWidget((GtkWindow *)commStru->mainListWidget, "Error", "List item is still active, please close the aplication\nwhich is connect to the item first.");
        gdk_threads_leave();
    }
    else
        FreeMemoryAndDeleteWidgetFunction(commStru);

    TRACEIT(10,"CloseCopyMoveOrRemoveButton		ends");
    return;
}


void ProcessItemsViaList(void *data){
	TRACEIT(10,"ProcessItemsViaList		start");
	if (data == NULL){
		TRACEIT(2,"ProcessItemsViaList		ends, parameter NULL");
		g_thread_exit(0);
	}
	GtkWidget *Window, *scrollWindow, *listView;
	GtkListStore *listStore;
	GtkTreeIter iter;
	GtkTreeViewColumn   	*col_Items, *col_ItemNumbers, *col_Errors, *col_percentage;
	GtkCellRenderer     	*renderer, *renderer2;
	GtkButton *CancelButton;
	GtkButton *CloseButton;
	GtkWidget *TheLabel;
	GtkVBox *VBox;
	char charLabelText[301];
	GError *Error=NULL;
	GString *WindowName = g_string_new(MYNAMESHORT); //freed when the commStru is freed
	struct CopyReMoveList *list = (struct CopyReMoveList *)data;
	struct CopyReMoveList *listbuf = list;
	int i=0;
	int intItemsCount=0;
    struct ListWidgetCommunicationStruct *commStru = malloc(sizeof(struct ListWidgetCommunicationStruct));
	gdk_threads_enter();

	Window = CreateMyWindow(WindowName->str, NULL, 400,200,TRUE, FALSE, TRUE, NULL, FALSE);
	if (Window == NULL){
        g_string_free(WindowName, TRUE);
        free(commStru);
        TRACEIT(1,"ProcessItemsViaList		end, window could not be created");
        return;
	}
    g_signal_connect (Window,"delete-event",G_CALLBACK(ListWidget_DeleteEvent),commStru);
    //g_signal_connect (Window, "destroy", G_CALLBACK(ListWidget_End_FreeMemory),commStru);

	AddMyIconsToWindow((GtkWindow *)Window);

  	char buf[320];

	scrollWindow = gtk_scrolled_window_new (NULL,NULL);
	listView = gtk_tree_view_new ();
	listStore = gtk_list_store_new( 4 , G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING  );

    col_Items=gtk_tree_view_column_new();
	col_ItemNumbers=gtk_tree_view_column_new();
	col_Errors=gtk_tree_view_column_new();
	col_percentage=gtk_tree_view_column_new();
	gtk_tree_view_column_set_resizable(col_Items,TRUE);
	gtk_tree_view_column_set_resizable(col_ItemNumbers,TRUE);
	gtk_tree_view_column_set_resizable(col_Errors,TRUE);
	gtk_tree_view_column_set_resizable(col_percentage,TRUE);

	gtk_tree_view_column_set_title(col_ItemNumbers, "#");
	gtk_tree_view_append_column(GTK_TREE_VIEW(listView),col_ItemNumbers);
	renderer2 = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(col_ItemNumbers, renderer2, TRUE);
	gtk_tree_view_column_add_attribute(col_ItemNumbers,renderer2,"text",0);

    gtk_tree_view_column_set_title(col_percentage, "%");
	gtk_tree_view_append_column(GTK_TREE_VIEW(listView),col_percentage);
	renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(col_percentage, renderer, TRUE);
	gtk_tree_view_column_add_attribute(col_percentage,renderer,"text",1);

	gtk_tree_view_column_set_title(col_Items, "items to process");
	gtk_tree_view_append_column(GTK_TREE_VIEW(listView),col_Items);
	renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(col_Items, renderer, TRUE);
	gtk_tree_view_column_add_attribute(col_Items,renderer,"text",2);

	gtk_tree_view_column_set_title(col_Errors, "errors");
	gtk_tree_view_append_column(GTK_TREE_VIEW(listView),col_Errors);
	renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(col_Errors, renderer, TRUE);
	gtk_tree_view_column_add_attribute(col_Errors,renderer,"text",3);

	g_signal_connect (listView,"button-press-event",G_CALLBACK(oneMouseButtonPressedInLittleWindow),NULL);

	do{
		intItemsCount++;
		listbuf = listbuf->NextItem;
	}while(listbuf!= NULL);
	listbuf = list;
	i=1;
	do{
		snprintf(buf,319,"%d/%d",i,intItemsCount);
		gtk_list_store_append(listStore,&iter);
		gtk_list_store_set (listStore, &iter, 0, buf, 2, listbuf->string1->str,-1);

        listbuf->listStore=listStore;
		listbuf->iter = iter;
		listbuf->status = -1;
		list->lastfinishedTotalBytes=0;
		i++;

        listbuf->CopyMoveOrRemove = list->CopyMoveOrRemove;

		listbuf = listbuf->NextItem;
	}while(listbuf!= NULL);

	CancelButton = (GtkButton *)gtk_button_new_with_label("cancel");
	g_signal_connect (CancelButton,"clicked",G_CALLBACK(ListWidget_CancelButton),(void *)commStru);

	CloseButton = (GtkButton *)gtk_button_new_with_label("close");
	g_signal_connect (CloseButton,"clicked",G_CALLBACK(ListWidget_CloseButton),commStru);

	gtk_tree_view_set_model (GTK_TREE_VIEW(listView), GTK_TREE_MODEL(listStore));

    //TheLabel = gtk_label_new(charLabelText);
	if (list->CopyMoveOrRemove == 1){ //COPY
		g_string_append(WindowName," copy");
		snprintf(charLabelText, 300, "Copying listed items to\n%s",list->DestObject->gstrPath->str );
	}
	else if(list->CopyMoveOrRemove == 2){ //MOVE
		g_string_append(WindowName," move");
		snprintf(charLabelText, 300, "Moving listed items to\n%s",list->DestObject->gstrPath->str );
	}
	else{	//REMOVE
		g_string_append(WindowName," remove");
		snprintf(charLabelText, 300, "Deleting listed items");
	}

	TheLabel = gtk_label_new(charLabelText);

	gtk_container_add((GtkContainer *)scrollWindow, listView);
	VBox = g_object_new(GTK_TYPE_VBOX, "spacing", 5, "homogeneous", FALSE, NULL);
	gtk_box_pack_start(GTK_BOX(VBox),TheLabel,FALSE,FALSE,1);
	gtk_box_pack_start(GTK_BOX(VBox),GTK_WIDGET(scrollWindow),TRUE,TRUE,1);
	gtk_box_pack_start(GTK_BOX(VBox),GTK_WIDGET(CancelButton),FALSE,FALSE,1);
    gtk_box_pack_start(GTK_BOX(VBox),GTK_WIDGET(CloseButton),FALSE,FALSE,1);

    gtk_container_add((GtkContainer *)Window,(GtkWidget *)VBox);

    commStru->mainListWidget=Window;
    commStru->listView=listView;
    commStru->cancelButton=CancelButton;
    commStru->closeButton=CloseButton;
    commStru->firstItem=list;
    commStru->currentItem=list;
    commStru->listStore=listStore;
    commStru->cancellable=g_cancellable_new();
    commStru->gerror=NULL;
    commStru->SecondThreadRunning=0;
    commStru->TheLabel=TheLabel;
    commStru->WindowName=WindowName;
    commStru->iCancelled=0;  //becomes 1 if the process is cancelled
    commStru->FreeMemoryAndDeleteWidgetFunction = FreeMemoryAndDeleteWidgetFunction;
    commStru->CancelListFunction = ListWidget_CancelList;
    commStru->iCancelDialogRuns=0;
    commStru->iDestroyFlag =0;
    commStru->bOverWriteAllExisting=FALSE;

	gtk_widget_show_all(Window);
	gtk_widget_hide(GTK_WIDGET(CloseButton));

	gdk_threads_leave();

	g_thread_create((GThreadFunc)CopyReMoveItemsViaList, (void *)commStru , FALSE, &Error);

//do not use nested gtk_main because we can only close the innerst loop
//so we just have to leave this function after we created the dialog.
//the meomory will be freed by the signal "end" function ListWidget_End_FreeMemory

//	gdk_threads_enter();
//	gtk_main();
//	gdk_threads_leave();

    //printf("after main\n");
    //we have to wait for the second thread:
/*    if (commStru->SecondThreadRunning == 1){
        printf("going to cancel\n");
        NEW_CancelCopyMoveOrRemoveButton(NULL, (void *)commStru);
        while (commStru->SecondThreadRunning == 1){
            printf("thread 2 still running\n");
            usleep(100000);
        }
    }
*/
//    gdk_threads_enter();
    //if (commStru->iNotify==1)
 //   gtk_widget_destroy (Window);
 //   commStru->iNotify=0;
 //   gdk_threads_leave();


    if (Error){
        g_error_free(Error);
    }


	TRACEIT(10,"ProcessItemsViaList		ends");
	g_thread_exit(0);
}


gboolean oneMouseButtonPressedInLittleWindow(GtkWidget *treeview, GdkEventButton *event, gpointer data){
	TRACEIT(10,"oneMouseButtonPressedInLittleWindow		start - ends");
	return TRUE;
}


void calcDirectorySizeInView_wrap(struct AllInformationAllUseStruct *stru){
    TRACEIT(10,"calcDirectorySizeInView_wrap		start");
    GError *Error=NULL;
    if(!g_thread_supported() ){
		g_thread_init(NULL);
     		//gdk_threads_init();                   // Called to initialize internal mutex "gdk_threads_mutex".
   	}
    g_thread_create((GThreadFunc)calcDirectorySizeInView_thread, (void *)stru , FALSE, &Error);
    if (Error){
		TRACEIT(2,"callItemsToSystemWrap: error in g_thread_create:");
		TRACEIT(2,Error->message);
		g_error_free(Error);
		Error=NULL;
	}
    TRACEIT(10,"calcDirectorySizeInView_wrap		end");
}


static void calcDirectorySizeInView_thread_helper (GtkTreeModel *model,GtkTreePath *path,GtkTreeIter *iter, void *data){
    TRACEIT(10,"calcDirectorySizeInView_thread_helper		start");
    struct AllInformationAllUseStruct *stru = (struct AllInformationAllUseStruct *)data;
    int type_d;
    gchar *gcharname=NULL;
    gchar *gcharpath=NULL;
    GString *gstrPath=g_string_sized_new(10);
    guint64 size=0;
    int ColumnSize;
    int *iRecursiveSizeCalculation;
    GtkListStore *listStore=NULL;

    if (stru->source==SOURCE_SIDEOBJECT){
        struct SideObjectList* Object = stru->Info_1;
        iRecursiveSizeCalculation = &(Object->iRecursiveSizeCalculation);
        ColumnSize = COL_SIZE;
        listStore = Object->listStore;
        gdk_threads_enter();
        gtk_tree_model_get(model, iter,COL_NAME,&gcharname,COL_TYPE_D,&type_d, -1);
        gdk_threads_leave();
        g_string_assign(gstrPath, Object->gstrPath->str);
    }
    else{
        struct FileShelfObjectList* Object = stru->Info_1;
        iRecursiveSizeCalculation = &(Object->iRecursiveSizeCalculation);
        ColumnSize = COL_FLSHLF_SIZE;
        listStore = Object->listStore;
        gdk_threads_enter();
        gtk_tree_model_get(model, iter, COL_FLSHLF_PATH, &gcharpath, COL_FLSHLF_NAME, &gcharname, COL_FLSHLF_TYPE_D, &type_d, -1);
        gdk_threads_leave();
        g_string_assign(gstrPath, gcharpath);
    }

    if (strcmp(gcharname, ".")!=0 && strcmp(gcharname, "..")!=0 && *iRecursiveSizeCalculation == 0 && type_d == 4){
        checkPathForLastChar(gstrPath);
        g_string_append(gstrPath, gcharname);
        if (0 != getDirectorySize(&size, gstrPath, iRecursiveSizeCalculation)){ //error opening the directory
            size=0;
        }
        gdk_threads_enter();
        gtk_list_store_set (listStore, iter, ColumnSize, size, -1);
        if (stru->source==SOURCE_SIDEOBJECT)
            updateStatusBar((struct SideObjectList *)stru->Info_1);
        gdk_threads_leave();

    }

    if (gcharname)
        g_free(gcharname);
    if (gcharpath)
        g_free(gcharpath);
    g_string_free(gstrPath, TRUE);
    TRACEIT(10,"calcDirectorySizeInView_thread_helper		end");
}


void calcDirectorySizeInView_thread(struct AllInformationAllUseStruct *stru){
    TRACEIT(10,"calcDirectorySizeInView_thread		start");
    GtkListStore		*listStore=NULL;
    int sort_column_id;
    GtkSortType order;

    if (stru->source==SOURCE_SIDEOBJECT){
        struct SideObjectList* Object = stru->Info_1;
        listStore = Object->listStore;

        //When the actual sort column is the size column there is a problem.
        //Every time we update an item with its size, the list will be reordered.
        //This means, when we come to an directory ehich is very little and therefore is placed to the end
        //of the list, the next item in the function for_each... is simply nothing because the list ends
        //with the current item and we break the function and may leave many directories unchecked.
        //To avoid this we set the current sort column to the definition GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID
        //and the list is not going to reorder its items. But we have to set the right column when we are finished.
        gtk_tree_sortable_get_sort_column_id (GTK_TREE_SORTABLE(Object->listStore), &sort_column_id, &order);
        if (COL_SIZE == sort_column_id){
            gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE(Object->listStore),GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID, order);
        }
        //gdk_threads_enter();
        gtk_tree_model_foreach(GTK_TREE_MODEL(listStore),(GtkTreeModelForeachFunc) calcDirectorySizeInView_thread_helper,(void *)stru);
        //gdk_threads_leave();

        if (COL_SIZE == sort_column_id){
            gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE(Object->listStore),COL_SIZE, order);
        }

        Object->iRecursiveSizeCalculation=-1;

    }
    else{
        struct FileShelfObjectList* Object = stru->Info_1;
        listStore = Object->listStore;

        //When the actual sort column is the size column there is a problem.
        //Every time we update an item with its size, the list will be reordered.
        //This means, when we come to an directory ehich is very little and therefore is placed to the end
        //of the list, the next item in the function for_each... is simply nothing because the list ends
        //with the current item and we break the function and may leave many directories unchecked.
        //To avoid this we set the current sort column to the definition GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID
        //and the list is not going to reorder its items. But we have to set the right column when we are finished.
        gtk_tree_sortable_get_sort_column_id (GTK_TREE_SORTABLE(Object->listStore), &sort_column_id, &order);
        if (COL_FLSHLF_SIZE == sort_column_id){
            gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE(Object->listStore),GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID, order);
        }
        //gdk_threads_enter();
        gtk_tree_model_foreach(GTK_TREE_MODEL(listStore),(GtkTreeModelForeachFunc) calcDirectorySizeInView_thread_helper,(void *)stru);
        //gdk_threads_leave();
        if (COL_FLSHLF_SIZE == sort_column_id){
            gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE(Object->listStore),COL_FLSHLF_SIZE, order);
        }

        Object->iRecursiveSizeCalculation=-1;
    }

    TRACEIT(10,"calcDirectorySizeInView_thread		end");
    g_thread_exit(0);
}

























