/*
    Copyright 2007, 2008, 2009 Geyer Klaus

    This file is part of Cat'sEyE.

    Cat'sEyE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Cat'sEyE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Cat'sEyE.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdlib.h>
#include <gtk/gtk.h>
#include <string.h>

#include "userdata.h"
#include "helpers.h"
#include "mytrace.h"

struct UserDataList *First_UserData_Item;
struct MenuObjectList *First_MenuObject_Item;
struct TypesOfSelectedItemsList *First_TypesOfSelectedItems_Item;


void UserCommand_InitUserCommandStructOnStartup(){
	TRACEIT(10,"UserCommand_InitUserCommandStructOnStartup		start");
	First_UserData_Item =  NULL;
	First_MenuObject_Item = NULL;
	First_TypesOfSelectedItems_Item = NULL;
	TRACEIT(10,"UserCommand_InitUserCommandStructOnStartup		end");
}


void UserCommand_DeleteAllReservedThings(){
	TRACEIT(10,"UserCommand_DeleteAllReservedThings		start");
	UserDataList_DeleteList();
	TypesOfSelectedItemsList_DeleteList();
	MenuObjectList_DeleteList();
	TRACEIT(10,"UserCommand_DeleteAllReservedThings		end");
}


void UserCommand_CreateUserMenu(GtkWidget *rootmenu){
	TRACEIT(10,"UserCommand_CreateUserMenu		start");
	struct UserDataList *UserData = First_UserData_Item;
	struct MenuObjectList *MenuObjects = NULL;
	const char sep[] = "/";
	char *buf;
	GString *gstrName = g_string_sized_new(10);
	GString *gstrPath = g_string_sized_new(10);
	int found = 0;
	int wherehere = 0;
	GtkWidget *LastMenu = NULL;
	GtkWidget *MenuBuf = NULL;

	//Create MenuStructure internal
	while (UserData){
		if(strstr(UserData->UserCommandName->str,sep) != NULL){
			g_string_assign(gstrName, UserData->UserCommandName->str);
			g_string_assign(gstrPath, UserData->UserCommandName->str);
			GetPathFromPathWithName(gstrPath, 1);
			getLastStringInPath(gstrName);

			LastMenu = rootmenu;
			wherehere = 0;
			for (buf = strtok(gstrPath->str,sep); buf; buf=strtok(NULL,sep)){
				wherehere = 1;
				MenuObjects = First_MenuObject_Item;
				found = 0;
				while (MenuObjects){
					if (strcmp (MenuObjects->gstrName->str, buf) == 0 && MenuObjects->MenuToAddTo == LastMenu){
						found = 1;
						MenuBuf = MenuObjects->submenu;
						LastMenu = MenuBuf;
						break;
					}
					MenuObjects = MenuObjects->NextItem;
				};
				if (found != 1){
					MenuBuf = gtk_menu_new();
					MenuObjectList_AddItem(buf, MenuBuf, LastMenu);
					LastMenu = MenuBuf;
				}
			}
			if (wherehere != 0){
				g_string_assign(UserData->UserCommandName, gstrName->str);
				UserData->MenuToAddTo = MenuBuf;
			}
		}
		if (UserData->MenuToAddTo == NULL){
			UserData->MenuToAddTo = rootmenu;
		}
		UserData = UserData->NextItem;
	}
	TRACEIT(10,"UserCommand_CreateUserMenu		end");
}


void UserCommand_AttachUserMenu(){
	TRACEIT(10,"UserCommand_AttachUserMenu		start");
	struct MenuObjectList *MenuObjects = NULL;
	GtkWidget *MenuBuf = NULL;

	//Now create Menustructure for viewing
	MenuObjects = First_MenuObject_Item;

	while (MenuObjects){
		MenuBuf = gtk_menu_item_new_with_label(MenuObjects->gstrName->str);
		gtk_menu_item_set_submenu ((GtkMenuItem *)MenuBuf,MenuObjects->submenu);
		gtk_menu_shell_append(GTK_MENU_SHELL(MenuObjects->MenuToAddTo), MenuBuf);
		MenuObjects = MenuObjects->NextItem;
	}
	TRACEIT(10,"UserCommand_AttachUserMenu		end");
}

/////////////////////////////////////////////////////////////////////////////////////////////////
///////////////// USER DATA LIST //////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////

void UserDataList_AddItem(char *UserCommand, char *UserCommandName){
	TRACEIT(10,"UserDataList_AddIte			start");
	if (UserCommand == NULL || UserCommandName == NULL){
		return;
	}
	struct UserDataList *NewItem;
	struct UserDataList *bufItem;

	NewItem = malloc (sizeof (struct UserDataList));

	if (NULL == NewItem){
		TRACEIT(1,"UserDataList_AddItem			end, Error allocating memory");
		return;
	}

	if (NULL == First_UserData_Item){
		First_UserData_Item = NewItem;
		NewItem->PrevItem=NULL;
	}
	else{
		bufItem = First_UserData_Item;
		while(bufItem->NextItem){
			bufItem = bufItem->NextItem;
		};
		bufItem->NextItem = NewItem;
		NewItem->PrevItem = bufItem;
	}

	NewItem->NextItem=NULL;

	NewItem->UserCommand = g_string_new(UserCommand);
	NewItem->UserCommandName = g_string_new(UserCommandName);
	NewItem->MenuToAddTo = NULL;
	TRACEIT(10,"UserDataList_AddItem			end");
}


void UserDataList_DeleteList(){
	TRACEIT(10,"UserDataList_DeleteList			start");
	if (NULL == First_UserData_Item){
		TRACEIT(10,"UserDataList_DeleteList			end 1");
		return;
	}
	struct UserDataList *bufItem;
	do{
		g_string_free(First_UserData_Item->UserCommand,TRUE);
		g_string_free(First_UserData_Item->UserCommandName,TRUE);

		bufItem = First_UserData_Item;
		First_UserData_Item = First_UserData_Item->NextItem;
		free(bufItem);
	}while(First_UserData_Item);
	TRACEIT(10,"UserDataList_DeleteList			end");
}


struct UserDataList *UserDataList_GiveMeFirstItem(){
	return First_UserData_Item;
}



/////////////////////////////////////////////////////////////////////////////////////////////////
///////////////// MENU OBJECT LIST //////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////

void MenuObjectList_AddItem(char *Name, GtkWidget* submenu, GtkWidget *MenuToAddTo){
	TRACEIT(10,"MenuObjectList_AddItem			start");
	if (Name == NULL || submenu == NULL){
		TRACEIT(10,"MenuObjectList_AddItem			end,1 ");
		return;
	}
	struct MenuObjectList *NewItem;
	struct MenuObjectList *bufItem;

	NewItem = malloc (sizeof (struct MenuObjectList));

	if (NULL == NewItem){
		TRACEIT(1,"MenuObjectList_AddItem			end, error allocating memory");
		return;
	}

	if (NULL == First_MenuObject_Item){
		First_MenuObject_Item = NewItem;
		NewItem->PrevItem=NULL;
	}
	else{
		bufItem = First_MenuObject_Item;
		while(bufItem->NextItem){
			bufItem = bufItem->NextItem;
		};
		bufItem->NextItem = NewItem;
		NewItem->PrevItem = bufItem;
	}

	NewItem->NextItem=NULL;


	NewItem->gstrName = g_string_new(Name);
	NewItem->submenu = submenu;
	NewItem->MenuToAddTo = MenuToAddTo;
	TRACEIT(10,"MenuObjectList_AddItem			end");
}


void MenuObjectList_DeleteList(){
	TRACEIT(10,"MenuObjectList_DeleteList			start");
	if (NULL == First_MenuObject_Item){
		TRACEIT(10,"MenuObjectList_DeleteList			end, 1");
		return;
	}
	struct MenuObjectList *bufItem;
	do{
		g_string_free(First_MenuObject_Item->gstrName,TRUE);
		bufItem = First_MenuObject_Item;
		First_MenuObject_Item = First_MenuObject_Item->NextItem;
		free(bufItem);
	}while(First_MenuObject_Item);
	TRACEIT(10,"MenuObjectList_DeleteList			end");
}


struct MenuObjectList *MenuObjectList_GiveMeFirstItem(){
	return First_MenuObject_Item;
}


/////////////////////////////////////////////////////////////////////////////////////////////////
///////////////// TYPES OF SELECTED ITEMS LIST //////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////

void TypesOfSelectedItemsList_AddItem(char *newCharItem){
	TRACEIT(10,"TypesOfSelectedItemsList_AddItem			start");
	if (newCharItem == NULL){
		TRACEIT(10,"TypesOfSelectedItemsList_AddItem			end, 1");
		return;
	}
	struct TypesOfSelectedItemsList *NewItem;
	struct TypesOfSelectedItemsList *bufItem;

	NewItem = malloc (sizeof (struct TypesOfSelectedItemsList));

	if (NULL == NewItem){
		TRACEIT(1,"TypesOfSelectedItemsList_AddItem			end, error allocation memory");
		return;
	}

	if (NULL == First_TypesOfSelectedItems_Item){
		First_TypesOfSelectedItems_Item = NewItem;
		NewItem->PrevItem=NULL;
	}
	else{
		bufItem = First_TypesOfSelectedItems_Item;
		while(bufItem->NextItem){
			bufItem = bufItem->NextItem;
		};
		bufItem->NextItem = NewItem;
		NewItem->PrevItem = bufItem;
	}
	NewItem->NextItem=NULL;
	NewItem->gstrType = g_string_new(newCharItem);
	TRACEIT(10,"TypesOfSelectedItemsList_AddItem			end");
}


void TypesOfSelectedItemsList_DeleteList(){
	TRACEIT(10,"TypesOfSelectedItemsList_DeleteList			start");
	if (NULL == First_TypesOfSelectedItems_Item){
		TRACEIT(10,"TypesOfSelectedItemsList_DeleteList			end, 1");
		return;
	}
	struct TypesOfSelectedItemsList *bufItem;
	do{
		g_string_free(First_TypesOfSelectedItems_Item->gstrType,TRUE);
		bufItem = First_TypesOfSelectedItems_Item;
		First_TypesOfSelectedItems_Item = First_TypesOfSelectedItems_Item->NextItem;
		free(bufItem);
	}while(First_TypesOfSelectedItems_Item);
	TRACEIT(10,"TypesOfSelectedItemsList_DeleteList			end");
}


struct TypesOfSelectedItemsList *TypesOfSelectedItemsList_GiveMeFirstItem(){
	return First_TypesOfSelectedItems_Item;

}



